@@ -41,6 +41,9 @@ public class PreferenceManager implements PropertyManager {
 
     private static Logger log = Logger.getLogger(PreferenceManager.class);
 
+    public static final String DEFAULT_GENOME = "hg18";
+
+    public static final String SKIP_VERSION = "SKIP_VERSION";
 
     public static final String INITIAL_TRACK_HEIGHT = "15";
 
@@ -70,6 +73,7 @@ public class PreferenceManager implements PropertyManager {
     public static final String IONTORRENT_RESULTS = "IONTORRENT.RESULTS";
 
     public static final String SAM_ALLELE_THRESHOLD = "SAM.ALLELE_THRESHOLD";
+    public static final String SAM_ALLELE_USE_QUALITY = "SAM.ALLELE_USE_QUALITY";
     public static final String SAM_QUALITY_THRESHOLD = "SAM.QUALITY_THRESHOLD";
     public static final String SAM_MAX_INSERT_SIZE_THRESHOLD = "SAM.INSERT_SIZE_THRESHOLD";
     public static final String SAM_MIN_INSERT_SIZE_THRESHOLD = "SAM.MIN_INSERT_SIZE_THRESHOLD";
@@ -85,6 +89,7 @@ public class PreferenceManager implements PropertyManager {
     public static final String SAM_BASE_QUALITY_MAX = "SAM.BASE_QUALITY_MAX";
     public static final String SAM_FILTER_ALIGNMENTS = "SAM.FILTER_ALIGNMENTS";
     public static final String SAM_FILTER_SECONDARY_ALIGNMENTS = "SAM.FILTER_SECONDARY_ALIGNMENTS";
+    public static final String SAM_FILTER_SUPPLEMENTARY_ALIGNMENTS = "SAM.FILTER_SUPPLEMENTARY_ALIGNMENTS";
     public static final String SAM_FILTER_URL = "SAM.FILTER_URL";
     public static final String SAM_MAX_VISIBLE_RANGE = "SAM.MAX_VISIBLE_RANGE";
     public static final String SAM_SHOW_DUPLICATES = "SAM.SHOW_DUPLICATES";
@@ -93,6 +98,7 @@ public class PreferenceManager implements PropertyManager {
     public static final String SAM_SAMPLING_COUNT = "SAM.MAX_LEVELS"; // Sampling count
     public static final String SAM_SAMPLING_WINDOW = "SAM.SAMPLING_WINDOW";
     public static final String SAM_DOWNSAMPLE_READS = "SAM.DOWNSAMPLE_READS";
+    public static final String SAM_SORT_OPTION = "SAM.SORT_OPTION";
 
     public static final String SAM_COLOR_BY = "SAM.COLOR_BY";
     public static final String SAM_COLOR_BY_TAG = "SAM.COLOR_BY_TAG";
@@ -115,24 +121,37 @@ public class PreferenceManager implements PropertyManager {
     public static final String SAM_FLAG_LARGE_INSERTIONS = "SAM.FLAG_LARGE_INSERTIONS";
     public static final String SAM_LARGE_INSERTIONS_THRESHOLD = "SAM.LARGE_INSERTIONS_THRESOLD";
 
+    public static final String SAM_SHOW_GROUP_SEPARATOR = "SAM.SHOW_GROUP_SEPARATOR";
+    public static final String SAM_COMPLETE_READS_ONLY = "SAM.COMPLETE_READS_ONLY";
+
+    public static final String COLOR_A = "COLOR.A";
+    public static final String COLOR_C = "COLOR.C";
+    public static final String COLOR_T = "COLOR.T";
+    public static final String COLOR_G = "COLOR.G";
+    public static final String COLOR_N = "COLOR.N";
+    public static final String SAM_COLOR_A = "SAM.COLOR.A";
+    public static final String SAM_COLOR_C = "SAM.COLOR.C";
+    public static final String SAM_COLOR_T = "SAM.COLOR.T";
+    public static final String SAM_COLOR_G = "SAM.COLOR.G";
+    public static final String SAM_COLOR_N = "SAM.COLOR.N";
 
     public static final String EXPAND_FEAUTRE_TRACKS = "EXPAND_FEATURE_TRACKS";
     public static final String PORT_ENABLED = "PORT_ENABLED";
     public static final String PORT_NUMBER = "PORT_NUMBER";
     public static final String COLOR_SCALE_KEY = "COLOR_SCALE_";
-    final public static String FRAME_BOUNDS_KEY = "IGV.Bounds";
-    final public static String FRAME_STATE_KEY = "IGV.Frame.ExtendedState";
-    final public static String RECENT_SESSION_KEY = "IGV.Session.recent.sessions";
-    final public static String TRACK_HEIGHT_KEY = "IGV.track.height";
-    final public static String CHART_TRACK_HEIGHT_KEY = "IGV.chart.track.height";
-    final public static String CHART_SHOW_ALL_HEATMAP = "CHART.SHOW_ALL_HEATMAP";
-    final public static String SHOW_MISSING_DATA_KEY = "IGV.track.show.missing.data";
-    final public static String SHOW_ATTRIBUTE_VIEWS_KEY = "IGV.track.show.attribute.views";
-    final public static String SHOW_SINGLE_TRACK_PANE_KEY = "IGV.single.track.pane";
-    final public static String GENOMES_SERVER_URL = "IGV.genome.sequence.dir";
-    final public static String JOIN_ADJACENT_SEGMENTS_KEY = "IGV.join.adjacent.segments";
-    final public static String SHOW_REGION_BARS = "SHOW_REGION_BARS";
-    final public static String LAST_EXPORTED_REGION_DIRECTORY = "LAST_EXPORTED_REGION_DIRECTORY";
+    public static final String FRAME_BOUNDS_KEY = "IGV.Bounds";
+    public static final String FRAME_STATE_KEY = "IGV.Frame.ExtendedState";
+    public static final String RECENT_SESSION_KEY = "IGV.Session.recent.sessions";
+    public static final String TRACK_HEIGHT_KEY = "IGV.track.height";
+    public static final String CHART_TRACK_HEIGHT_KEY = "IGV.chart.track.height";
+    public static final String CHART_SHOW_ALL_HEATMAP = "CHART.SHOW_ALL_HEATMAP";
+    public static final String SHOW_MISSING_DATA_KEY = "IGV.track.show.missing.data";
+    public static final String SHOW_ATTRIBUTE_VIEWS_KEY = "IGV.track.show.attribute.views";
+    public static final String SHOW_SINGLE_TRACK_PANE_KEY = "IGV.single.track.pane";
+    public static final String GENOMES_SERVER_URL = "IGV.genome.sequence.dir";
+    public static final String JOIN_ADJACENT_SEGMENTS_KEY = "IGV.join.adjacent.segments";
+    public static final String SHOW_REGION_BARS = "SHOW_REGION_BARS";
+    public static final String LAST_EXPORTED_REGION_DIRECTORY = "LAST_EXPORTED_REGION_DIRECTORY";
     final static public String LAST_TRACK_DIRECTORY = "LAST_TRACK_DIRECTORY";
     final static public String LAST_SNAPSHOT_DIRECTORY = "LAST_SNAPSHOT_DIRECTORY";
     final static public String LAST_GENOME_IMPORT_DIRECTORY = "LAST_GENOME_IMPORT_DIRECTORY";
@@ -142,69 +161,73 @@ public class PreferenceManager implements PropertyManager {
     final static public String HISTORY_DELIMITER = ";";
     final static public String GENOME_ID_DISPLAY_LIST_KEY = "GENOME_LIST";
     final static public String DETAILS_BEHAVIOR_KEY = "DETAILS_BEHAVIOR";
+    final static public String DEFAULT_VISIBILITY_WINDOW = "DEFAULT_VISIBILITY_WINDOW";
 
-    final public static String MUTATION_COLOR_TABLE = "MUTATION_COLOR_TABLE";
-    final public static String MUTATION_INDEL_COLOR_KEY = "MUTATION_INDEL_COLOR_KEY";
-    final public static String MUTATION_MISSENSE_COLOR_KEY = "MUTATION_MISSENSE_COLOR_KEY";
-    final public static String MUTATION_NONSENSE_COLOR_KEY = "MUTATION_NONSENSE_COLOR_KEY";
-    final public static String MUTATION_SPLICE_SITE_COLOR_KEY = "MUTATION_SPLICE_SITE_COLOR_KEY";
-    final public static String MUTATION_SYNONYMOUS_COLOR_KEY = "MUTATION_SYNONYMOUS_COLOR_KEY";
-    final public static String MUTATION_TARGETED_REGION_COLOR_KEY = "MUTATION_TARGETED_REGION_COLOR_KEY";
-    final public static String MUTATION_UNKNOWN_COLOR_KEY = "MUTATION_UNKNOWN_COLOR_KEY";
-    final public static String OVERLAY_MUTATION_TRACKS = "OVERLAY_TRACKS_KEY";
-    final public static String SHOW_ORPHANED_MUTATIONS = "SHOW_ORPHANED_MUTATIONS";
-    final public static String OVERLAY_ATTRIBUTE_KEY = "OVERLAY_ATTRIBUTE_KEY";
-    final public static String OVERLAY_MUTATIONS_WHOLE_GENOME = "OVERLAY_MUTATIONS_WHOLE_GENOME";
-    final public static String COLOR_MUTATIONS = "COVER_OVERLAY_KEY";
-    final public static String TRACK_ATTRIBUTE_NAME_KEY = "TRACK_ATTRIBUTE_NAME_KEY";
-    final public static String DATA_SERVER_URL_KEY = "MASTER_RESOURCE_FILE_KEY";
-    //final public static String CHECKED_RESOURCES_KEY = "CHECKED_RESOURCES_KEY";
-    final public static String DEFINE_GENOME_INPUT_DIRECTORY_KEY = "DEFINE_GENOME_INPUT_DIRECTORY_KEY";
+    public static final String MUTATION_COLOR_TABLE = "MUTATION_COLOR_TABLE";
+    public static final String MUTATION_INDEL_COLOR_KEY = "MUTATION_INDEL_COLOR_KEY";
+    public static final String MUTATION_MISSENSE_COLOR_KEY = "MUTATION_MISSENSE_COLOR_KEY";
+    public static final String MUTATION_NONSENSE_COLOR_KEY = "MUTATION_NONSENSE_COLOR_KEY";
+    public static final String MUTATION_SPLICE_SITE_COLOR_KEY = "MUTATION_SPLICE_SITE_COLOR_KEY";
+    public static final String MUTATION_SYNONYMOUS_COLOR_KEY = "MUTATION_SYNONYMOUS_COLOR_KEY";
+    public static final String MUTATION_TARGETED_REGION_COLOR_KEY = "MUTATION_TARGETED_REGION_COLOR_KEY";
+    public static final String MUTATION_UNKNOWN_COLOR_KEY = "MUTATION_UNKNOWN_COLOR_KEY";
+    public static final String OVERLAY_MUTATION_TRACKS = "OVERLAY_TRACKS_KEY";
+    public static final String SHOW_ORPHANED_MUTATIONS = "SHOW_ORPHANED_MUTATIONS";
+    public static final String OVERLAY_ATTRIBUTE_KEY = "OVERLAY_ATTRIBUTE_KEY";
+    public static final String OVERLAY_MUTATIONS_WHOLE_GENOME = "OVERLAY_MUTATIONS_WHOLE_GENOME";
+    public static final String COLOR_MUTATIONS = "COVER_OVERLAY_KEY";
+    public static final String TRACK_ATTRIBUTE_NAME_KEY = "TRACK_ATTRIBUTE_NAME_KEY";
+    public static final String DATA_SERVER_URL_KEY = "MASTER_RESOURCE_FILE_KEY";
+    //public static final String CHECKED_RESOURCES_KEY = "CHECKED_RESOURCES_KEY";
+    public static final String DEFINE_GENOME_INPUT_DIRECTORY_KEY = "DEFINE_GENOME_INPUT_DIRECTORY_KEY";
 
-    final public static String PROBE_MAPPING_KEY = "PROBE_MAPPING_KEY";
-    final public static String PROBE_MAPPING_FILE = "PROBE_MAPPING_FILE";
-    final public static String USE_PROBE_MAPPING_FILE = "USE_PROBE_MAPPING_FILE";
+    public static final String PROBE_MAPPING_KEY = "PROBE_MAPPING_KEY";
+    public static final String PROBE_MAPPING_FILE = "PROBE_MAPPING_FILE";
+    public static final String USE_PROBE_MAPPING_FILE = "USE_PROBE_MAPPING_FILE";
 
-    final public static String SEARCH_ZOOM = "SEARCH_ZOOM";
-    final public static String NORMALIZE_COVERAGE = "NORMALIZE_COVERAGE";
+    public static final String SEARCH_ZOOM = "SEARCH_ZOOM";
+    public static final String NORMALIZE_COVERAGE = "NORMALIZE_COVERAGE";
     public static final String SHOW_EXPAND_ICON = "SHOW_EXPAND_ICON";
+    public static final String SHOW_DEFAULT_TRACK_ATTRIBUTES = "SHOW_DEFAULT_TRACK_ATTRIBUTES";
 
     public static final String SHOW_SIZE_WARNING = "SHOW_SIZE_WARNING";
     public static final String SHOW_GENOME_SERVER_WARNING = "SHOW_GENOME_SERVER_WARNING";
 
-    final public static String USE_PROXY = "PROXY.USE";
-    final public static String PROXY_HOST = "PROXY.HOST";
-    final public static String PROXY_PORT = "PROXY.PORT";
-    final public static String PROXY_AUTHENTICATE = "PROXY.AUTHENTICATE";
-    final public static String PROXY_NTLM = "PROXY.NTLM";
-    final public static String PROXY_USER = "PROXY.USERNAME";
-    final public static String PROXY_PW = "PROXY.PW";
-    final public static String PROXY_TYPE = "PROXY.TYPE";
+    public static final String USE_PROXY = "PROXY.USE";
+    public static final String PROXY_HOST = "PROXY.HOST";
+    public static final String PROXY_PORT = "PROXY.PORT";
+    public static final String PROXY_AUTHENTICATE = "PROXY.AUTHENTICATE";
+    public static final String PROXY_NTLM = "PROXY.NTLM";
+    public static final String PROXY_USER = "PROXY.USERNAME";
+    public static final String PROXY_PW = "PROXY.PW";
+    public static final String PROXY_TYPE = "PROXY.TYPE";
+    public static final String PROXY_WHITELIST = "PROXY.WHITELIST";
 
-    final public static String KNOWN_SNPS = "KNOWN_SNPS_FILE";
+    public static final String KNOWN_SNPS = "KNOWN_SNPS_FILE";
 
-    public static final String FLANKING_REGION = "FLAKING_REGIONS";
+    public static final String FLANKING_REGION = "FLANKING_REGION";
 
     public static final String SHOW_SEQUENCE_TRANSLATION = "SHOW_SEQUENCE_TRANSLATION";
     public static final String MAX_SEQUENCE_RESOLUTION = "MAX_SEQUENCE_RESOLUTION";
 
     public static final String AUTO_UPDATE_GENOMES = "AUTO_UPDATE_GENOMES";
 
-    final public static String GWAS_TRACK_HEIGHT = "GWAS_TRACK_HEIGHT";
-    final public static String GWAS_DESCRIPTION_CACHE_SIZE = "GWAS_DESCRIPTION_CACHE_SIZE";
-    final public static String GWAS_MIN_POINT_SIZE = "GWAS_MIN_POINT_SIZE";
-    final public static String GWAS_MAX_POINT_SIZE = "GWAS_MAX_POINT_SIZE";
-    final public static String GWAS_USE_CHR_COLORS = "GWAS_USE_CHR_COLORS";
-    final public static String GWAS_SINGLE_COLOR = "GWAS_SINGLE_COLOR";
-    final public static String GWAS_ALTERNATING_COLORS = "GWAS_ALTERNATING_COLORS";
-    final public static String GWAS_PRIMARY_COLOR = "GWAS_PRIMARY_COLOR";
-    final public static String GWAS_SECONDARY_COLOR = "GWAS_SECONDARY_COLOR";
-    final public static String GWAS_SHOW_AXIS = "GWAS_SHOW_AXIS";
+    public static final String GWAS_TRACK_HEIGHT = "GWAS_TRACK_HEIGHT";
+    public static final String GWAS_DESCRIPTION_CACHE_SIZE = "GWAS_DESCRIPTION_CACHE_SIZE";
+    public static final String GWAS_MIN_POINT_SIZE = "GWAS_MIN_POINT_SIZE";
+    public static final String GWAS_MAX_POINT_SIZE = "GWAS_MAX_POINT_SIZE";
+    public static final String GWAS_USE_CHR_COLORS = "GWAS_USE_CHR_COLORS";
+    public static final String GWAS_SINGLE_COLOR = "GWAS_SINGLE_COLOR";
+    public static final String GWAS_ALTERNATING_COLORS = "GWAS_ALTERNATING_COLORS";
+    public static final String GWAS_PRIMARY_COLOR = "GWAS_PRIMARY_COLOR";
+    public static final String GWAS_SECONDARY_COLOR = "GWAS_SECONDARY_COLOR";
+    public static final String GWAS_SHOW_AXIS = "GWAS_SHOW_AXIS";
 
     public static final String DEFAULT_FONT_SIZE = "DEFAULT_FONT_SIZE";
     public static final String DEFAULT_FONT_FAMILY = "DEFAULT_FONT_FAMILY";
     public static final String DEFAULT_FONT_ATTRIBUTE = "DEFAULT_FONT_ATTRIBUTE";
     public static final String ENABLE_ANTIALISING = "ENABLE_ANTIALIASING";
+    public static final String SCALE_FONTS = "SCALE_FONTS";
 
     public static final String NAME_PANEL_WIDTH = "NAME_PANEL_WIDTH";
     public static final String BACKGROUND_COLOR = "BACKGROUND_COLOR";
@@ -227,11 +250,22 @@ public class PreferenceManager implements PropertyManager {
     public static final String DB_HOST = "DB_HOST";
     public static final String DB_NAME = "DB_NAME";
     public static final String DB_PORT = "DB_PORT";
-    final public static String DEFAULT_GENOME_URL = "http://igv.broadinstitute.org/genomes/genomes.txt";
-    final public static String DEFAULT_DATA_URL = "http://www.broadinstitute.org/igvdata/$$_dataServerRegistry.txt";
+    public static final String DEFAULT_GENOME_URL = "http://igv.broadinstitute.org/genomes/genomes.txt";
+    public static final String DEFAULT_DATA_URL = "http://www.broadinstitute.org/igvdata/$$_dataServerRegistry.txt";
 
     public static final String IGV_PLUGIN_LIST_KEY = "IGV_PLUGIN_LIST";
 
+    public static final String SASHIMI_SHOW_COVERAGE = "SASHIMI.SHOW_COVERAGE";
+
+    public static final String GOOGLE_API_KEY = "GOOGLE_API_KEY";
+    public static final String ENABLE_GOOGLE_MENU = "ENABLE_GOOGLE_MENU";
+    public static final String SAVE_GOOGLE_CREDENTIALS = "SAVE_GOOGLE_CREDENTIALS";
+
+    public static final String BLAT_URL = "BLAT_URL";
+
+    public static final String GENE_LIST_BED_FORMAT = "GENE_LIST_BED_FORMAT";
+
+
     IGVPreferences preferences;
     Map<String, String> defaultValues;
 
@@ -271,6 +305,10 @@ public class PreferenceManager implements PropertyManager {
         return get(key, defaultValues.get(key));
     }
 
+    public boolean hasExplicitValue(String key) {
+        return preferences.userPreferences.containsKey(key);
+    }
+
     /**
      * Get the default value for the specified key.
      * May be null.
@@ -679,7 +717,7 @@ public class PreferenceManager implements PropertyManager {
 
     public String getDefaultGenome() {
 
-        String genome = get(DEFAULT_GENOME_KEY, Globals.DEFAULT_GENOME);
+        String genome = get(DEFAULT_GENOME_KEY, DEFAULT_GENOME);
         return genome;
     }
 
@@ -891,6 +929,7 @@ public class PreferenceManager implements PropertyManager {
         remove(PreferenceManager.PROXY_USER);
         remove(PreferenceManager.PROXY_PW);
         remove(PreferenceManager.PROXY_TYPE);
+        remove(PreferenceManager.PROXY_WHITELIST);
         HttpUtils.getInstance().updateProxySettings();
     }
 
@@ -931,6 +970,7 @@ public class PreferenceManager implements PropertyManager {
         defaultValues.put(SHOW_MISSING_DATA_KEY, "false");
         defaultValues.put(SHOW_SINGLE_TRACK_PANE_KEY, "false");
         defaultValues.put(SHOW_EXPAND_ICON, "false");
+        defaultValues.put(SHOW_DEFAULT_TRACK_ATTRIBUTES, "false");
 
         defaultValues.put(IONTORRENT_FLOWDIST_HIDE_FIRST_HP, "true");
         defaultValues.put(IONTORRENT_FLOWDIST_BINSIZE, "15");
@@ -961,6 +1001,7 @@ public class PreferenceManager implements PropertyManager {
         defaultValues.put(SAM_SHADE_BASES, ShadeBasesOption.QUALITY.toString());
         defaultValues.put(SAM_FILTER_ALIGNMENTS, "false");
         defaultValues.put(SAM_FILTER_SECONDARY_ALIGNMENTS, "false");
+        defaultValues.put(SAM_FILTER_SUPPLEMENTARY_ALIGNMENTS, "false");
         defaultValues.put(SAM_FILTER_FAILED_READS, "true");
         defaultValues.put(SAM_DOWNSAMPLE_READS, "true");
         defaultValues.put(SAM_SAMPLING_WINDOW, "50");
@@ -970,6 +1011,7 @@ public class PreferenceManager implements PropertyManager {
         defaultValues.put(SAM_FILTER_URL, null);
         defaultValues.put(SAM_QUALITY_THRESHOLD, "0");
         defaultValues.put(SAM_ALLELE_THRESHOLD, "0.2f");
+        defaultValues.put(SAM_ALLELE_USE_QUALITY, "true");
         defaultValues.put(SAM_MIN_INSERT_SIZE_THRESHOLD, "50");
         defaultValues.put(SAM_MAX_INSERT_SIZE_THRESHOLD, "1000");
         defaultValues.put(SAM_MIN_INSERT_SIZE_PERCENTILE, "0.5");
@@ -990,6 +1032,9 @@ public class PreferenceManager implements PropertyManager {
         defaultValues.put(SAM_COUNT_DELETED_BASES_COVERED, "false");
         defaultValues.put(SAM_FLAG_LARGE_INSERTIONS, "false");
         defaultValues.put(SAM_LARGE_INSERTIONS_THRESHOLD, "1");
+        defaultValues.put(SAM_SORT_OPTION, "NUCLEOTIDE");
+        defaultValues.put(SAM_SHOW_GROUP_SEPARATOR, "true");
+        defaultValues.put(SAM_COMPLETE_READS_ONLY, "false");
 
         defaultValues.put(NORMALIZE_COVERAGE, "false");
 
@@ -1000,7 +1045,7 @@ public class PreferenceManager implements PropertyManager {
 
         defaultValues.put(GENOMES_SERVER_URL, DEFAULT_GENOME_URL);
         defaultValues.put(OVERLAY_ATTRIBUTE_KEY, "LINKING_ID");
-        defaultValues.put(DEFAULT_GENOME_KEY, Globals.DEFAULT_GENOME);
+        defaultValues.put(DEFAULT_GENOME_KEY, DEFAULT_GENOME);
 
         defaultValues.put(USE_PROXY, "false");
         defaultValues.put(PROXY_AUTHENTICATE, "false");
@@ -1027,7 +1072,10 @@ public class PreferenceManager implements PropertyManager {
         defaultValues.put(DEFAULT_FONT_SIZE, "10");
         defaultValues.put(DEFAULT_FONT_FAMILY, "Arial");
         defaultValues.put(DEFAULT_FONT_ATTRIBUTE, String.valueOf(Font.PLAIN));
-        defaultValues.put(ENABLE_ANTIALISING, "true");
+        defaultValues.put(SCALE_FONTS, "false");
+
+        boolean isMac = System.getProperty("os.name").toLowerCase().startsWith("mac");
+        defaultValues.put(ENABLE_ANTIALISING, String.valueOf(isMac));
 
         defaultValues.put(NAME_PANEL_WIDTH, "160");
         defaultValues.put(BACKGROUND_COLOR, "250,250,250");
@@ -1073,6 +1121,31 @@ public class PreferenceManager implements PropertyManager {
         defaultValues.put(DETAILS_BEHAVIOR_KEY, IGVCommandBar.SHOW_DETAILS_BEHAVIOR.HOVER.name());
 
         defaultValues.put(SHOW_SIZE_WARNING, "true");
+
+        defaultValues.put(SKIP_VERSION, "");
+
+        defaultValues.put(COLOR_A, "0,150,0");
+        defaultValues.put(COLOR_C, "0,0,255");
+        defaultValues.put(COLOR_T, "255,0,0");
+        defaultValues.put(COLOR_G, "209,113,5");
+        defaultValues.put(COLOR_N, ColorUtilities.colorToString(Color.gray));
+        defaultValues.put(SAM_COLOR_A, "0,255,0");
+        defaultValues.put(SAM_COLOR_C, "0,0,255");
+        defaultValues.put(SAM_COLOR_T, "255,0,0");
+        defaultValues.put(SAM_COLOR_G, "209,113,5");
+        defaultValues.put(SAM_COLOR_N, ColorUtilities.colorToString(Color.gray.brighter()));
+
+        defaultValues.put(SASHIMI_SHOW_COVERAGE, "true");
+
+        defaultValues.put(ENABLE_GOOGLE_MENU, "false");
+        defaultValues.put(SAVE_GOOGLE_CREDENTIALS, "true");
+
+        defaultValues.put(DEFAULT_VISIBILITY_WINDOW, "-1");
+
+        defaultValues.put(BLAT_URL, "http://genome.cse.ucsc.edu/cgi-bin/hgBlat");
+
+        defaultValues.put(GENE_LIST_BED_FORMAT, "false");
+
     }
 
     /**
@@ -1081,11 +1154,8 @@ public class PreferenceManager implements PropertyManager {
      * @param s
      */
     public void setPrefsFile(String s) {
-        if (preferences == null) {
-            preferences = new IGVPreferences(new File(s));
-        } else {
-            preferences.setPrefFile(new File(s));
-        }
+        preferences = new IGVPreferences(new File(s));
+        clearCaches();
     }
 
     public static String generateGenomeIdString(Collection<GenomeListItem> genomeListItems) {
@@ -1166,7 +1236,7 @@ public class PreferenceManager implements PropertyManager {
         return String.format("%s:%s:%s", pluginId, toolName.replace(' ', '_'), key.replace(' ', '_'));
     }
 
-    public void putArgumentValue(String pluginId, String toolName, String command, String argName, String argValue){
+    public void putArgumentValue(String pluginId, String toolName, String command, String argName, String argValue) {
         String key = genArgKey(pluginId, toolName, command, argName);
         put(key, argValue);
     }
@@ -1175,7 +1245,7 @@ public class PreferenceManager implements PropertyManager {
         return get(genArgKey(pluginId, toolName, commandName, argId));
     }
 
-    private String genArgKey(String pluginId, String toolName, String command, String argId){
+    private String genArgKey(String pluginId, String toolName, String command, String argId) {
         return genToolKey(pluginId, toolName, String.format("%s:%s", command, argId));
     }
 
