@@ -16,6 +16,9 @@
 
 package org.broad.igv.batch;
 
+import com.google.common.collect.Iterables;
+import com.google.common.eventbus.EventBus;
+import com.google.common.eventbus.Subscribe;
 import org.apache.log4j.Logger;
 import org.broad.igv.Globals;
 import org.broad.igv.PreferenceManager;
@@ -28,17 +31,21 @@ import org.broad.igv.sam.AlignmentTrack;
 import org.broad.igv.track.RegionScoreType;
 import org.broad.igv.track.Track;
 import org.broad.igv.ui.IGV;
+import org.broad.igv.ui.event.DataLoadedEvent;
 import org.broad.igv.ui.panel.FrameManager;
 import org.broad.igv.ui.util.MessageUtils;
 import org.broad.igv.ui.util.SnapshotUtilities;
 import org.broad.igv.util.*;
-import org.broad.igv.util.collections.LRUCache;
 
 import java.awt.*;
 import java.io.File;
 import java.io.IOException;
+import java.net.URI;
+import java.net.URISyntaxException;
 import java.util.*;
 import java.util.List;
+import java.util.concurrent.ExecutionException;
+import java.util.concurrent.Future;
 
 public class CommandExecutor {
 
@@ -68,16 +75,12 @@ public class CommandExecutor {
         List<String> subArgs = Collections.emptyList();
         if (args.size() > 1) subArgs = args.subList(1, args.size());
         try {
-            Object ocommand = Class.forName(cmd).newInstance();
-            Command command = (Command) ocommand;
+            Object ocmmand = RuntimeUtils.loadInstanceForName(cmd, null);
+            Command command = (Command) ocmmand;
             return command.run(subArgs);
-        } catch (InstantiationException e) {
-            return e.getMessage();
-        } catch (IllegalAccessException e) {
-            return e.getMessage();
         } catch (ClassNotFoundException e) {
             return null;
-        } catch (ClassCastException e) {
+        } catch (Exception e) {
             return e.getMessage();
         }
     }
@@ -123,7 +126,7 @@ public class CommandExecutor {
                     String filename = param1;
                     result = createSnapshot(filename, param2);
                 } else if ((cmd.equalsIgnoreCase("loadfile") || cmd.equalsIgnoreCase("load")) && param1 != null) {
-                    result = load(param1, param2, param3);
+                    result = load(param1, param2, param3, param4);
                 } else if (cmd.equalsIgnoreCase("genome") && args.size() > 1) {
                     result = genome(param1);
                 } else if (cmd.equalsIgnoreCase("new") || cmd.equalsIgnoreCase("reset") || cmd.equalsIgnoreCase("clear")) {
@@ -135,14 +138,17 @@ public class CommandExecutor {
                 } else if (cmd.equalsIgnoreCase("group")) {
                     group(param1);
                 } else if (cmd.equalsIgnoreCase("collapse")) {
-                    String trackName = param1 == null ? null : param1.replace("\"", "").replace("'", "");
+                    String trackName = parseTrackName(param1);
                     igv.setTrackDisplayMode(Track.DisplayMode.COLLAPSED, trackName);
                 } else if (cmd.equalsIgnoreCase("expand")) {
-                    String trackName = param1 == null ? null : param1.replace("\"", "").replace("'", "");
+                    String trackName = parseTrackName(param1);
                     igv.setTrackDisplayMode(Track.DisplayMode.EXPANDED, trackName);
                 } else if (cmd.equalsIgnoreCase("squish")) {
-                    String trackName = param1 == null ? null : param1.replace("\"", "").replace("'", "");
+                    String trackName = parseTrackName(param1);
                     igv.setTrackDisplayMode(Track.DisplayMode.SQUISHED, trackName);
+                } else if (cmd.equalsIgnoreCase("remove")) {
+                    String trackName = parseTrackName(param1);
+                    result = removeTrack(trackName);
                 } else if (cmd.equalsIgnoreCase("tweakdivider")) {
                     igv.tweakPanelDivider();
                 } else if (cmd.equalsIgnoreCase("setDataRange")) {
@@ -178,8 +184,7 @@ public class CommandExecutor {
             igv.doRefresh();
 
             if (RuntimeUtils.getAvailableMemoryFraction() < 0.5) {
-                log.debug("Clearing caches");
-                LRUCache.clearCaches();
+                log.debug("Running garbage collection");
                 System.gc();
             }
             log.debug("Finished execution: " + command + "  sleeping ....");
@@ -199,6 +204,21 @@ public class CommandExecutor {
         return result;
     }
 
+    private String removeTrack(String trackName) {
+        if (trackName == null) return "Error: NULL TRACK NAME";
+        for (Track track : igv.getAllTracks()) {
+            if (track.getName().equals(trackName)) {
+                igv.removeTracks(Arrays.asList(track));
+                return "OK";
+            }
+        }
+        return String.format("Error: Track %s not found", trackName);
+    }
+
+    static String parseTrackName(String param1) {
+        return param1 == null ? null : StringUtils.stripQuotes(param1);
+    }
+
     private String overridePreference(String prefKey, String prefVal) {
         PreferenceManager.getInstance().override(prefKey, prefVal);
         return "OK";
@@ -226,6 +246,7 @@ public class CommandExecutor {
         for (Track track : tracks) {
             if (trackName == null || trackName.equalsIgnoreCase(track.getName())) {
                 track.setDataRange(range);
+                track.setAutoScale(false);
             }
         }
         return "OK";
@@ -322,7 +343,7 @@ public class CommandExecutor {
         }
         if (ParsingUtils.pathExists(genomePath)) {
             try {
-                igv.loadGenome(genomePath, null);
+                igv.loadGenome(genomePath, null, true);
             } catch (IOException e) {
                 throw new RuntimeException("Error loading genome: " + genomeID);
             }
@@ -338,40 +359,52 @@ public class CommandExecutor {
     /**
      * Load function for port and batch script
      *
-     * @param fileList
+     * @param fileString
      * @param param2
      * @param param3
      * @return
      * @throws IOException
      */
-    private String load(String fileList, String param2, String param3) throws IOException {
-
-        String fileString = fileList.replace("\"", "").replace("'", "");  // Todo <= what is this for?
+    private String load(String fileString, String param2, String param3, String param4) throws IOException {
 
         // Default for merge is "true" for session files,  "false" otherwise
-        String file = fileString;
-        boolean merge;
-        if (file.endsWith(".xml") || file.endsWith(".php") || file.endsWith(".php3")) {
-            // Session file
-            merge = false;
-        } else {
-            // Data file
-            merge = true;
-        }
+        String tmpFile = StringUtils.stripQuotes(fileString);
+        boolean merge = !(tmpFile.endsWith(".xml") || tmpFile.endsWith(".php") || tmpFile.endsWith(".php3"));
 
-        // remaining parameters might be "merge" or "name"
+        // remaining parameters might be "merge", "name", or "index"
         String name = null;
+        String index = null;
+        String coverage = null;
+        String format = null;
         for (String param : Arrays.asList(param2, param3)) {
             if (param != null && param.startsWith("name=")) {
                 name = param.substring(5);
             } else if (param != null && param.startsWith("merge=")) {
                 String mergeString = param.substring(6);
                 merge = mergeString.equalsIgnoreCase("true");
+            } else if (param != null && param.startsWith("index=")) {
+                index = param.substring(6);
+            } else if (param != null && param.startsWith("coverage=")) {
+                coverage = param.substring(9);
+            } else if(param != null && param.startsWith("format=")) {
+                format = param.substring(7);
             }
         }
         // Locus is not specified from port commands
         String locus = null;
-        return loadFiles(fileString, null, merge, name);
+        Map<String, String> params = null;
+        return loadFiles(fileString, index, coverage, name, format, locus, merge, params);
+    }
+
+    String loadFiles(final String fileString,
+                     final String indexString,
+                     final String coverageString,
+                     final String nameString,
+                     final String formatString,
+                     final String locus,
+                     final boolean merge,
+                     Map<String, String> params) throws IOException {
+        return loadFiles(fileString, indexString, coverageString, nameString, formatString, locus, merge, params, null, null);
     }
 
     /**
@@ -380,34 +413,66 @@ public class CommandExecutor {
      * @param fileString
      * @param locus
      * @param merge
-     * @param name
+     * @param nameString
+     * @param params
+     * @param sort
+     * @param sortTag    Used iff sort == SortOption.TAG
      * @return
      * @throws IOException
      */
-    String loadFiles(final String fileString, final String locus, final boolean merge, String name) throws IOException {
-        return loadFiles(fileString, locus, merge, name, null);
-    }
-
-    String loadFiles(final String fileString, final String locus, final boolean merge, String nameString, Map<String, String> params) throws IOException {
+    String loadFiles(final String fileString,
+                     final String indexString,
+                     final String coverageString,
+                     final String nameString,
+                     final String formatString,
+                     final String locus,
+                     final boolean merge,
+                     Map<String, String> params,
+                     String sort,
+                     String sortTag) throws IOException {
 
 
         log.debug("Run load files");
 
-        String[] files = fileString.split(",");
-        String[] names = nameString != null ? nameString.split(",") : null;
-        if (files.length == 1) {
+        List<String> files = StringUtils.breakQuotedString(fileString, ',');
+        List<String> names = StringUtils.breakQuotedString(nameString, ',');
+        List<String> indexFiles = StringUtils.breakQuotedString(indexString, ',');
+        List<String> coverageFiles = StringUtils.breakQuotedString(coverageString, ',');
+        List<String> formats = StringUtils.breakQuotedString(formatString, ',');
+
+        if (files.size() == 1) {
             // String might be URL encoded
-            files = fileString.split("%2C");
-            names = nameString != null ? nameString.split("%2C") : null;
+            files = StringUtils.breakQuotedString(fileString.replaceAll("%2C", ","), ',');
+            names = nameString != null ? StringUtils.breakQuotedString(nameString.replaceAll("%2C", ","), ',') : null;
+            indexFiles = indexString != null ? StringUtils.breakQuotedString(indexString.replaceAll("%2C", ","), ',') : null;
+            coverageFiles = coverageString != null ? StringUtils.breakQuotedString(coverageString.replaceAll("%2C", ","), ',') : null;
+            formats = formatString != null ? StringUtils.breakQuotedString(formatString.replaceAll("%2C", ","), ',') : null;
         }
 
-        if (names != null && names.length != files.length) {
-            return "Error: If files is a comma-separated list, names must also be a comma-separated list of the same length";
+        if (names != null && names.size() != files.size()) {
+            return "Error: If file is a comma-separated list, names must also be a comma-separated list of the same length";
         }
 
+        if (indexFiles != null && indexFiles.size() != files.size()) {
+            return "Error: If file is a comma-separated list, index must also be a comma-separated list of the same length";
+        }
+
+
         // Must decode URLs (local or remote), but leave local file paths only
-        for (int ii = 0; ii < files.length; ii++) {
-            files[ii] = decodeFileString(files[ii]);
+        for (int ii = 0; ii < files.size(); ii++) {
+            files.set(ii, decodeFileString(files.get(ii).replace("\"", "")));
+            if (names != null) {
+                names.set(ii, names.get(ii).replace("\"", ""));
+            }
+            if (indexFiles != null) {
+                indexFiles.set(ii, decodeFileString(indexFiles.get(ii).replace("\"", "")));
+            }
+            if (coverageFiles != null) {
+                coverageFiles.set(ii, decodeFileString(coverageFiles.get(ii).replace("\"", "")));
+            }
+            if (formatString != null) {
+                formats.set(ii, decodeFileString(formats.get(ii).replace("\"", "")));
+            }
         }
 
         List<ResourceLocator> fileLocators = new ArrayList<ResourceLocator>();
@@ -433,8 +498,11 @@ public class CommandExecutor {
         }
 
         // Loop through files
-        int fi = 0;
-        for (String f : files) {
+
+        for (int fi = 0; fi < files.size(); fi++) {
+
+            String f = files.get(fi);
+
             // Skip already loaded files TODO -- make this optional?  Check for change?
             if (loadedFiles.contains(f)) continue;
 
@@ -451,7 +519,18 @@ public class CommandExecutor {
                 }
 
                 if (names != null) {
-                    rl.setName(names[fi]);
+                    rl.setName(names.get(fi));
+                }
+                if (indexFiles != null) {
+                    rl.setIndexPath(indexFiles.get(fi));
+                }
+                if (coverageFiles != null) {
+                    rl.setCoverage(coverageFiles.get(fi));
+                }
+                if(formats != null) {
+                    String format = formats.get(fi);
+                    if(!format.startsWith(".")) format = "." + format;
+                    rl.setType(format);
                 }
                 if (params != null) {
                     String trackLine = createTrackLine(params);
@@ -459,46 +538,98 @@ public class CommandExecutor {
                 }
                 fileLocators.add(rl);
             }
-            fi++;
         }
 
         for (String sessionPath : sessionPaths) {
             igv.restoreSessionSynchronous(sessionPath, locus, merge);
         }
 
-        igv.loadTracks(fileLocators);
+        final Future loadTask = igv.loadTracks(fileLocators);
 
         if (locus != null && !locus.equals("null")) {
             igv.goToLocus(locus);
+            //If locus is a single base, we sort by base
+            String[] tokens = locus.split(":", 2);
+            if (tokens.length == 2) {
+                String chr = tokens[0];
+                try {
+                    int pos = Integer.parseInt(tokens[1].replace(",", ""));
+                    if (pos >= 0 && sort == null) sort = "base";
+
+                } catch (Exception e) {
+                    //pass
+                }
+            }
+
         }
 
-        return "OK";
+        if (sort != null) {
+            submitPerformSort(loadTask, sort, sortTag);
+        }
+
+        return CommandListener.OK;
+    }
+
+    private void submitPerformSort(final Future loadTask, final String sort, final String sortTag) {
+        final AlignmentTrack.SortOption sortOption = getAlignmentSortOption(sort);
+        Runnable runnable = new Runnable() {
+            @Override
+            public void run() {
+                try {
+                    //We need to wait until the track is loaded. If loadTask is null,
+                    //it was loaded synchronously
+                    if (loadTask != null) {
+                        Object res = loadTask.get();
+                    }
+                    //Thought we were done waiting, huh? Guess again
+                    //Alignment tracks load alignment data asynchronously from the track
+
+                    //Since sorting applies to all tracks, we only need to have 1 handler
+                    AlignmentTrack track = null;
+                    try {
+                        track = Iterables.filter(igv.getAllTracks(), AlignmentTrack.class).iterator().next();
+                        EventBus bus = track.getDataManager().getEventBus();
+                        bus.register(new SortAlignmentsHandler(igv, bus, sortOption, sortTag));
+                    } catch (NoSuchElementException e) {
+                        //No alignment tracks found.
+                        log.warn("Sort argument provided but no alignment tracks found");
+                    }
+                } catch (InterruptedException e) {
+                    log.error(e.getMessage(), e);
+                } catch (ExecutionException e) {
+                    log.error(e.getMessage(), e);
+                }
+
+            }
+        };
+        LongRunningTask.submit(runnable);
     }
 
     /**
      * If {@code fileString} is a URL and can be decoded,
      * return the decoded version. Otherwise return the original.
+     *
      * @param fileString
      * @return
      */
-    static String decodeFileString(String fileString){
+    static String decodeFileString(String fileString) {
         if (needsDecode(fileString)) {
             return StringUtils.decodeURL(fileString);
-        }else{
+        } else {
             return fileString;
         }
     }
 
-    static boolean needsDecode(String fileString){
+    static boolean needsDecode(String fileString) {
         String decodedString = decodeSafe(fileString);
         return (decodedString != null && (HttpUtils.isURL(fileString) || HttpUtils.isURL(decodedString)));
     }
 
-    private static String decodeSafe(String string){
+    private static String decodeSafe(String string) {
         String tmp = null;
-        try{
+        try {
             tmp = StringUtils.decodeURL(string);
-        }catch (Exception e){
+        } catch (Exception e) {
             log.warn(e.getMessage());
         }
         return tmp;
@@ -540,28 +671,59 @@ public class CommandExecutor {
      * @return
      */
     String setSnapshotDirectory(String param1) {
+
         if (param1 == null) {
             return "ERROR: missing directory parameter";
         }
 
+        param1 = StringUtils.stripQuotes(param1);
+
+        File parentDir = null;
+        try {
+            parentDir = getFile(param1);
+        } catch (URISyntaxException e) {
+            log.error("Error parsing directory path: " + param1, e);
+            return "Error parsing directory path: " + param1;
+        }
+
         String result;
-        File parentDir = new File(param1);
         if (parentDir.exists()) {
             snapshotDirectory = parentDir;
             result = "OK";
         } else {
+            createParents(parentDir);
             parentDir.mkdir();
             if (parentDir.exists()) {
                 snapshotDirectory = parentDir;
                 result = "OK";
             } else {
-
                 result = "ERROR: directory: " + param1 + " does not exist";
             }
         }
         return result;
     }
 
+    private File getFile(String param1) throws URISyntaxException {
+
+        // Strip trailing & leading quotes
+        if (param1.startsWith("\"")) param1 = param1.substring(1);
+        if (param1.endsWith("\"")) param1 = param1.substring(0, param1.lastIndexOf('"'));
+
+        // See if file contains spaces, if not no special treatment is required
+        if (param1.indexOf(' ') < 0) {
+            return new File(param1);
+        } else {
+            // If file is absolute use a URI,
+            File f = new File(param1);
+            if (f.isAbsolute()) {
+                URI outputURI = new URI(("file://" + param1.replaceAll(" ", "%20")));
+                return new File(outputURI);
+            } else {
+                return f;
+            }
+        }
+    }
+
     private String goto1(List<String> args) {
         if (args == null || args.size() < 2) {
             return "ERROR: missing locus parameter";
@@ -637,19 +799,35 @@ public class CommandExecutor {
         igv.repaintDataPanels();
     }
 
-    private void group(String sortArg) {
-        igv.groupAlignmentTracks(getAlignmentGroupOption(sortArg));
+    private void group(String groupArg) {
+        igv.groupAlignmentTracks(getAlignmentGroupOption(groupArg));
         igv.repaintDataPanels();
     }
 
 
     private String createSnapshot(String filename, String region) {
+
         if (filename == null) {
             String locus = FrameManager.getDefaultFrame().getFormattedLocusString();
             filename = locus.replaceAll(":", "_").replace("-", "_") + ".png";
+        } else {
+            filename = StringUtils.stripQuotes(filename);
         }
 
-        File file = snapshotDirectory == null ? new File(filename) : new File(snapshotDirectory, filename);
+        File file;
+        if (snapshotDirectory == null) {
+            try {
+                file = getFile(filename);
+                if (!file.getAbsoluteFile().getParentFile().exists()) {
+                    createParents(file);
+                }
+            } catch (URISyntaxException e) {
+                log.error("Error parsing directory path: " + filename, e);
+                return "Error parsing directory path: " + filename;
+            }
+        } else {
+            file = new File(snapshotDirectory, filename);
+        }
         System.out.println("Snapshot: " + file.getAbsolutePath());
 
 
@@ -674,6 +852,15 @@ public class CommandExecutor {
         }
     }
 
+
+    private static void createParents(File outputFile) {
+        File parent = outputFile.getParentFile();
+        if (!parent.exists()) {
+            parent.mkdirs();
+        }
+
+    }
+
     private static RegionScoreType getRegionSortOption(String str) {
         if (str == null) return null;
         String option = str.toUpperCase();
@@ -691,7 +878,7 @@ public class CommandExecutor {
         } else if (str.equalsIgnoreCase("strand")) {
             return AlignmentTrack.SortOption.STRAND;
         } else if (str.equalsIgnoreCase("base")) {
-            return AlignmentTrack.SortOption.NUCELOTIDE;
+            return AlignmentTrack.SortOption.NUCLEOTIDE;
         } else if (str.equalsIgnoreCase("quality")) {
             return AlignmentTrack.SortOption.QUALITY;
         } else if (str.equalsIgnoreCase("sample")) {
@@ -705,7 +892,7 @@ public class CommandExecutor {
         } else if (str.equalsIgnoreCase("mateChr")) {
             return AlignmentTrack.SortOption.MATE_CHR;
         }
-        return AlignmentTrack.SortOption.NUCELOTIDE;
+        return AlignmentTrack.SortOption.NUCLEOTIDE;
     }
 
     private static AlignmentTrack.GroupOption getAlignmentGroupOption(String str) {
@@ -720,4 +907,26 @@ public class CommandExecutor {
         }
         return AlignmentTrack.GroupOption.NONE;
     }
+
+    private static class SortAlignmentsHandler {
+
+        private IGV igv = null;
+        private EventBus bus = null;
+        private AlignmentTrack.SortOption sortOption;
+        private String sortTag;
+
+        SortAlignmentsHandler(IGV igv, EventBus bus, AlignmentTrack.SortOption sortOption, String sortTag) {
+            this.igv = igv;
+            this.bus = bus;
+            this.sortOption = sortOption;
+            this.sortTag = sortTag;
+        }
+
+        @Subscribe
+        public void received(DataLoadedEvent event) {
+            boolean sorted = igv.sortAlignmentTracks(sortOption, sortTag);
+            if (sorted) this.bus.unregister(this);
+        }
+
+    }
 }
