@@ -11,9 +11,9 @@
 
 package org.broad.igv.bbfile;
 
-import net.sf.samtools.seekablestream.SeekableStream;
+import htsjdk.samtools.seekablestream.SeekableStream;
 import org.apache.log4j.Logger;
-import org.broad.tribble.util.LittleEndianInputStream;
+import htsjdk.tribble.util.LittleEndianInputStream;
 
 import java.io.ByteArrayInputStream;
 import java.io.DataInputStream;
