@@ -186,6 +186,13 @@ public class Genome {
                 // NCBI
                 String alias = getNCBIName(name);
                 autoAliases.put(alias, name);
+
+                // Also strip version number out, if present
+                int dotIndex = alias.lastIndexOf('.');
+                if(dotIndex > 0) {
+                    alias = alias.substring(0, dotIndex);
+                    autoAliases.put(alias, name);
+                }
             }
         }
 
@@ -518,7 +525,7 @@ public class Genome {
                 String nextLine;
                 while ((nextLine = br.readLine()) != null) {
                     if (nextLine.startsWith("#")) continue;
-                    String[] tokens = Globals.tabPattern.split(nextLine);
+                    String[] tokens = Globals.whitespacePattern.split(nextLine);
                     if (tokens.length == 2) {
                         ucscSpeciesMap.put(tokens[0], tokens[1]);
                     } else {
