@@ -18,6 +18,8 @@
  */
 package org.broad.igv.feature.genome;
 
+import com.google.common.collect.Iterables;
+import org.apache.commons.io.IOUtils;
 import org.apache.log4j.Logger;
 import org.broad.igv.DirectoryManager;
 import org.broad.igv.Globals;
@@ -29,11 +31,9 @@ import org.broad.igv.ui.IGVMenuBar;
 import org.broad.igv.ui.util.ConfirmDialog;
 import org.broad.igv.ui.util.MessageUtils;
 import org.broad.igv.ui.util.ProgressMonitor;
-import org.broad.igv.util.FileUtils;
-import org.broad.igv.util.HttpUtils;
-import org.broad.igv.util.ResourceLocator;
-import org.broad.igv.util.Utilities;
+import org.broad.igv.util.*;
 import org.broad.igv.util.collections.CI;
+import htsjdk.tribble.util.ParsingUtils;
 
 import java.awt.*;
 import java.io.*;
@@ -43,10 +43,7 @@ import java.net.URL;
 import java.net.URLDecoder;
 import java.util.*;
 import java.util.List;
-import java.util.zip.ZipEntry;
-import java.util.zip.ZipException;
-import java.util.zip.ZipFile;
-import java.util.zip.ZipInputStream;
+import java.util.zip.*;
 
 /**
  * @author jrobinso
@@ -55,7 +52,17 @@ public class GenomeManager {
 
     private static Logger log = Logger.getLogger(GenomeManager.class);
 
-    final public static String USER_DEFINED_GENOME_LIST_FILE = "user-defined-genomes.txt";
+    private static final String ACT_USER_DEFINED_GENOME_LIST_FILE = "user-defined-genomes.txt";
+    public static final String TEST_USER_DEFINED_GENOME_LIST_FILE = "test-user-defined-genomes.txt";
+
+    public static String getUserDefinedGenomeListFile() {
+        if (Globals.isTesting()) {
+            return TEST_USER_DEFINED_GENOME_LIST_FILE;
+        } else {
+            return ACT_USER_DEFINED_GENOME_LIST_FILE;
+        }
+
+    }
 
     private static GenomeManager theInstance;
 
@@ -66,7 +73,10 @@ public class GenomeManager {
     private List<GenomeListItem> cachedGenomeArchiveList;
     private Set<String> excludedArchivesUrls = new HashSet();
 
-    //ID comparison will be case insensitive
+    /**
+     * Map from genomeID -> GenomeListItem
+     * ID comparison will be case insensitive
+     */
     private Map<String, GenomeListItem> genomeItemMap = new CI.CILinkedHashMap<GenomeListItem>();
 
     public static void main(String[] args) {
@@ -100,15 +110,15 @@ public class GenomeManager {
         return serverGenomeListUnreachable;
     }
 
-    public Genome loadGenome(String genomePath, ProgressMonitor monitor) throws IOException{
+    public Genome loadGenome(String genomePath, ProgressMonitor monitor) throws IOException {
         return loadGenome(genomePath, monitor, true);
     }
 
     /**
      * Load a genome from the given path.  Could be a .genome, .gbk, chrom.sizes, or fasta file
      *
-     * @param genomePath File, http, or ftp path to the .genome or indexed fasta file
-     * @param monitor    ProgressMonitor  Monitor object, can be null
+     * @param genomePath     File, http, or ftp path to the .genome or indexed fasta file
+     * @param monitor        ProgressMonitor  Monitor object, can be null
      * @param addGenomeTrack Whether to add the genomeTrack to IGV
      * @return Genome
      * @throws FileNotFoundException
@@ -125,9 +135,12 @@ public class GenomeManager {
                 monitor.fireProgressChange(25);
             }
 
+            // Clear Feature DB
+            FeatureDB.clearFeatures();
+
             if (genomePath.endsWith(".genome")) {
                 newGenome = loadDotGenomeFile(genomePath);
-            } else if (genomePath.endsWith(".gbk")) {
+            } else if (genomePath.endsWith(".gbk") || genomePath.endsWith(".gb")) {
                 newGenome = loadGenbankFile(genomePath);
             } else if (genomePath.endsWith(".chrom.sizes")) {
                 newGenome = loadChromSizes(genomePath);
@@ -142,13 +155,13 @@ public class GenomeManager {
             // Load alias files from genome source directory, if any
             String aliasPath = FileUtils.getParent(genomePath) + "/" + newGenome.getId() + "_alias.tab";
             Collection<Collection<String>> aliases = loadChrAliases(aliasPath);
-            if(aliases != null) newGenome.addChrAliases(aliases);
+            if (aliases != null) newGenome.addChrAliases(aliases);
 
 
             // Load user-defined chr aliases, if any.  This is done last so they have priority
             aliasPath = (new File(DirectoryManager.getGenomeCacheDirectory(), newGenome.getId() + "_alias.tab")).getAbsolutePath();
             aliases = loadChrAliases(aliasPath);
-            if(aliases != null) newGenome.addChrAliases(aliases);
+            if (aliases != null) newGenome.addChrAliases(aliases);
 
             if (monitor != null) {
                 monitor.fireProgressChange(25);
@@ -196,9 +209,11 @@ public class GenomeManager {
     private Genome loadGenbankFile(String genomePath) throws IOException {
         Genome newGenome;
         GenbankParser genbankParser = new GenbankParser(genomePath);
+        genbankParser.readFeatures(true);
 
-        String chr = genbankParser.getAccession();
         String name = genbankParser.getLocusName();
+        String chr = genbankParser.getChr();
+
         if (!name.equals(chr)) {
             name = name + " (" + chr + ")";
         }
@@ -206,6 +221,18 @@ public class GenomeManager {
         byte[] seq = genbankParser.getSequence();
         Sequence sequence = new InMemorySequence(chr, seq);
         newGenome = new Genome(chr, name, sequence, true);
+
+        String[] aliases = genbankParser.getAliases();
+        if (aliases != null) {
+            List<String> aliasList = new ArrayList<String>();
+            aliasList.add(chr);
+            for (String a : aliases) {
+                aliasList.add(a);
+            }
+            newGenome.addChrAliases(Arrays.<Collection<String>>asList(aliasList));
+        }
+
+
         setCurrentGenome(newGenome);
 
         if (IGV.hasInstance() && !Globals.isHeadless()) {
@@ -213,6 +240,8 @@ public class GenomeManager {
             newGenome.setGeneTrack(geneFeatureTrack);
         }
 
+        FeatureDB.addFeatures(genbankParser.getFeatures(), newGenome);
+
         return newGenome;
     }
 
@@ -262,8 +291,8 @@ public class GenomeManager {
 
     private Collection<Collection<String>> loadChrAliases(String path) {
 
-       // String id = genome.getId();
-       // File aliasFile = new File(DirectoryManager.getGenomeCacheDirectory(), id + "_alias.tab");
+        // String id = genome.getId();
+        // File aliasFile = new File(DirectoryManager.getGenomeCacheDirectory(), id + "_alias.tab");
         File aliasFile = new File(path);
 
         if (aliasFile.exists()) {
@@ -294,7 +323,7 @@ public class GenomeManager {
     /**
      * @param path
      * @return GenomeListItem representing this path, or null
-     *         if it's a local path which doesn't exist (we don't check for existence of remote file)
+     * if it's a local path which doesn't exist (we don't check for existence of remote file)
      */
     public static GenomeListItem buildFromPath(String path) {
 
@@ -372,15 +401,23 @@ public class GenomeManager {
         }
 
         InputStream geneStream = null;
-        if (genomeDescriptor.getGeneFileName() != null) {
+        String geneFileName = genomeDescriptor.getGeneFileName();
+        if (geneFileName != null) {
             try {
                 geneStream = genomeDescriptor.getGeneStream();
-                BufferedReader reader = new BufferedReader(new InputStreamReader(geneStream));
-                FeatureTrack geneFeatureTrack = createGeneTrack(newGenome, reader,
-                        genomeDescriptor.getGeneFileName(), genomeDescriptor.getGeneTrackName(),
-                        genomeDescriptor.getUrl());
+                if (geneFileName.endsWith(".gbk")) {
+                    GenbankParser genbankParser = new GenbankParser();
+                    genbankParser.readFeatures(geneStream, false);
+                    FeatureTrack geneFeatureTrack = createGeneTrack(newGenome, genbankParser.getFeatures());
+                    newGenome.setGeneTrack(geneFeatureTrack);
+                } else {
+                    BufferedReader reader = new BufferedReader(new InputStreamReader(geneStream));
+                    FeatureTrack geneFeatureTrack = createGeneTrack(newGenome, reader,
+                            geneFileName, genomeDescriptor.getGeneTrackName(),
+                            genomeDescriptor.getUrl());
 
-                newGenome.setGeneTrack(geneFeatureTrack);
+                    newGenome.setGeneTrack(geneFeatureTrack);
+                }
             } finally {
                 if (geneStream != null) geneStream.close();
             }
@@ -495,19 +532,23 @@ public class GenomeManager {
     }
 
     /**
-     * Refresh a locally cached genome
+     * Refresh a locally cached genome if appropriate (newer one on server, user set preference to enable it)
+     * If it doesn't have a local cache, just downloaded
+     * If the cached version has a custom sequence location, that is copied over to the downloaded version
      *
      * @param cachedFile
      * @param genomeArchiveURL
      * @throws IOException
      */
-    private void refreshCache(File cachedFile, URL genomeArchiveURL) {
+    void refreshCache(File cachedFile, URL genomeArchiveURL) {
         // Look in cache first
-
-
         try {
             if (cachedFile.exists()) {
-                boolean remoteModfied = !HttpUtils.getInstance().compareResources(cachedFile, genomeArchiveURL);
+                //Check to see cached version has a custom sequence
+                GenomeDescriptor cachedDescriptor = parseGenomeArchiveFile(cachedFile);
+                //File sizes won't be the same if the local version has a different sequence location
+                boolean remoteModfied = HttpUtils.getInstance().remoteIsNewer(cachedFile, genomeArchiveURL,
+                        !cachedDescriptor.hasCustomSequenceLocation());
 
                 // Force an update of cached genome if file length does not equal remote content length
                 boolean forceUpdate = remoteModfied &&
@@ -515,12 +556,22 @@ public class GenomeManager {
                 if (forceUpdate) {
                     log.info("Refreshing genome: " + genomeArchiveURL.toString());
                     File tmpFile = new File(cachedFile.getAbsolutePath() + ".tmp");
-                    if (HttpUtils.getInstance().downloadFile(genomeArchiveURL.toExternalForm(), tmpFile)) {
-                        FileUtils.copyFile(tmpFile, cachedFile);
+                    if (HttpUtils.getInstance().downloadFile(genomeArchiveURL.toExternalForm(), tmpFile).isSuccess()) {
+
                         tmpFile.deleteOnExit();
+                        boolean success = true;
+
+                        if (cachedDescriptor.hasCustomSequenceLocation()) {
+                            success = rewriteSequenceLocation(tmpFile, cachedDescriptor.getSequenceLocation());
+                        }
+
+                        if (success) {
+                            FileUtils.copyFile(tmpFile, cachedFile);
+                        } else {
+                            log.warn("Updating genome failed: " + genomeArchiveURL.toString());
+                        }
                     }
                 }
-
             } else {
                 // Copy file directly from the server to local cache.
                 HttpUtils.getInstance().downloadFile(genomeArchiveURL.toExternalForm(), cachedFile);
@@ -535,7 +586,7 @@ public class GenomeManager {
     /**
      * Creates a genome descriptor.
      */
-    public GenomeDescriptor parseGenomeArchiveFile(File f)
+    public static GenomeDescriptor parseGenomeArchiveFile(File f)
             throws IOException {
 
 
@@ -585,6 +636,7 @@ public class GenomeManager {
                     boolean fasta = parseBooleanPropertySafe(properties, "fasta");
                     boolean fastaDirectory = parseBooleanPropertySafe(properties, "fastaDirectory");
                     boolean chromosomesAreOrdered = parseBooleanPropertySafe(properties, Globals.GENOME_ORDERED_KEY);
+                    boolean hasCustomSequenceLocation = parseBooleanPropertySafe(properties, Globals.GENOME_ARCHIVE_CUSTOM_SEQUENCE_LOCATION_KEY);
 
 
                     String fastaFileNameString = properties.getProperty("fastaFiles");
@@ -601,6 +653,7 @@ public class GenomeManager {
                             chrAliasFileName,
                             properties.getProperty(Globals.GENOME_GENETRACK_NAME, "Gene"),
                             sequenceLocation,
+                            hasCustomSequenceLocation,
                             zipFile,
                             zipEntries,
                             chromosomesAreOrdered,
@@ -627,7 +680,7 @@ public class GenomeManager {
         return genomeDescriptor;
     }
 
-    private boolean parseBooleanPropertySafe(Properties properties, String key) {
+    private static boolean parseBooleanPropertySafe(Properties properties, String key) {
         String propertyString = properties.getProperty(key);
         return Boolean.parseBoolean(propertyString);
     }
@@ -664,7 +717,7 @@ public class GenomeManager {
     public boolean loadFromArchive(String genomeId) throws IOException {
         GenomeListItem matchingItem = findGenomeListItemById(genomeId);
         if (matchingItem != null) {
-            GenomeManager.getInstance().addGenomeItems(Arrays.asList(matchingItem));
+            GenomeManager.getInstance().addGenomeItems(Arrays.asList(matchingItem), false);
         }
         return matchingItem != null;
     }
@@ -750,7 +803,7 @@ public class GenomeManager {
                 serverGenomeArchiveList = null;
                 log.error("Error fetching genome list: ", e);
                 ConfirmDialog.optionallyShowInfoDialog("Warning: could not connect to the genome server (" +
-                        genomeListURLString + ").    Only locally defined genomes will be available.",
+                                genomeListURLString + ").    Only locally defined genomes will be available.",
                         PreferenceManager.SHOW_GENOME_SERVER_WARNING);
 
             } finally {
@@ -780,7 +833,7 @@ public class GenomeManager {
     /**
      * Searches through currently loaded GenomeListItems and returns
      * that with a matching ID. null if not found. To search through
-     * all server and user defined genomes, use #findGenomeListItemByItem
+     * all server and user defined genomes, use {@link #findGenomeListItemById(String)}
      *
      * @param genomeId
      * @return
@@ -812,7 +865,8 @@ public class GenomeManager {
         return matchingItem;
     }
 
-    private GenomeListItem searchGenomeList(String genomeId, Iterable<GenomeListItem> genomeList) {
+    static GenomeListItem searchGenomeList(String genomeId, Iterable<GenomeListItem> genomeList) {
+        if (genomeList == null) return null;
         for (GenomeListItem item : genomeList) {
             if (item.getId().equals(genomeId)) {
                 return item;
@@ -827,53 +881,29 @@ public class GenomeManager {
 
     /**
      * Completely rebuild the genome drop down info.
-     * This will load genomes from server/cached/user defined (as appropriate),
-     * and only keep the ones that the user has chosen to be displayed
+     * This is based on preferences only, does not contact server
      */
     public void buildGenomeItemList() {
-
-        // Build a single available genome list from both client, server
-        // and cached information. This allows us to process
-        // everything the same way.
+        genomeItemMap.clear();
         Collection<GenomeListItem> tmpuserDefinedGenomeList = null;
-        Collection<GenomeListItem> tmpArchiveGenomeItemList = null;
+        Collection<GenomeListItem> tmpcachedGenomeArchiveList = null;
 
-        tmpArchiveGenomeItemList = getGenomeArchiveList();
         try {
+            tmpcachedGenomeArchiveList = getCachedGenomeArchiveList();
             tmpuserDefinedGenomeList = getUserDefinedGenomeArchiveList();
         } catch (IOException e) {
             MessageUtils.showErrorMessage("Cannot access user defined genome archive list", e);
         }
 
-
-        combineGenomeLists(tmpuserDefinedGenomeList, tmpArchiveGenomeItemList);
-
-    }
-
-
-    /**
-     * Combine our different lists of genomeListItems
-     *
-     * @param userDefinedGenomeItemList
-     * @param archiveGenomeItemList     Either server or cached genomes
-     */
-    private void combineGenomeLists(Collection<GenomeListItem> userDefinedGenomeItemList,
-                                    Collection<GenomeListItem> archiveGenomeItemList) {
-
-        //We use a LinkedHashMap to prevent loading duplicates
-        genomeItemMap = new LinkedHashMap<String, GenomeListItem>();
-
         String[] genomeIdArray = PreferenceManager.getInstance().getGenomeIdDisplayList();
 
         if (genomeIdArray.length == 0) {
             genomeIdArray = new String[]{PreferenceManager.getInstance().getDefaultGenome(), "hg18"};
         }
 
-        if ((userDefinedGenomeItemList != null) && !userDefinedGenomeItemList.isEmpty()) {
-            addAllInSet(userDefinedGenomeItemList, genomeIdArray, genomeItemMap);
-        }
+        Iterable<GenomeListItem> combinedTmp = Iterables.concat(tmpuserDefinedGenomeList, tmpcachedGenomeArchiveList);
+        addGenomesToMap(genomeIdArray, combinedTmp, genomeItemMap);
 
-        addAllInSet(archiveGenomeItemList, genomeIdArray, genomeItemMap);
     }
 
     /**
@@ -885,19 +915,27 @@ public class GenomeManager {
      * @param keepGenomeIds
      * @param genomeMap
      */
-    private void addAllInSet(Iterable<GenomeListItem> genomeListItems, String[] keepGenomeIds, Map<String, GenomeListItem> genomeMap) {
+    private void addGenomesToMap(String[] keepGenomeIds, Iterable<GenomeListItem> genomeListItems, Map<String, GenomeListItem> genomeMap) {
         for (String id : keepGenomeIds) {
             GenomeListItem genomeListItem = searchGenomeList(id, genomeListItems);
 
-            //if we didn't find the id, it may be a path
+            //If we didn't find the id, it may be a path
             if (genomeListItem == null) {
                 genomeListItem = buildFromPath(id);
             }
 
-            if (!genomeMap.containsKey(id) && genomeListItem != null) {
-                genomeMap.put(id, genomeListItem);
+            if (genomeListItem == null) {
+                /**
+                 * The {@code id} isn't in {@code genomeListItems},
+                 * and it's not a remote path or an existing local path
+                 * We assume it's the ID for a genome stored on the server
+                 */
+                genomeListItem = new GenomeListItem(id, null, id);
             }
 
+            if (!genomeMap.containsKey(id)) {
+                genomeMap.put(id, genomeListItem);
+            }
         }
     }
 
@@ -919,7 +957,7 @@ public class GenomeManager {
 
             userDefinedGenomeArchiveList = new LinkedHashSet<GenomeListItem>();
 
-            File listFile = new File(DirectoryManager.getGenomeCacheDirectory(), USER_DEFINED_GENOME_LIST_FILE);
+            File listFile = new File(DirectoryManager.getGenomeCacheDirectory(), getUserDefinedGenomeListFile());
 
             BufferedReader reader = null;
 
@@ -951,8 +989,12 @@ public class GenomeManager {
                         continue;
                     }
 
-                    GenomeListItem item = new GenomeListItem(fields[0], file, fields[2]);
-                    userDefinedGenomeArchiveList.add(item);
+                    try {
+                        GenomeListItem item = new GenomeListItem(fields[0], file, fields[2]);
+                        userDefinedGenomeArchiveList.add(item);
+                    } catch (Exception e) {
+                        log.error("Error updating user genome list line '" + nextLine + "'", e);
+                    }
                 }
             } catch (FileNotFoundException e) {
                 //We swallow this because the user may not have the file,
@@ -969,7 +1011,7 @@ public class GenomeManager {
     }
 
     /**
-     * Method description
+     * Delete .genome files from the cache directory
      */
     public void clearGenomeCache() {
 
@@ -1092,7 +1134,7 @@ public class GenomeManager {
             return;
         }
 
-        File listFile = new File(DirectoryManager.getGenomeCacheDirectory(), USER_DEFINED_GENOME_LIST_FILE);
+        File listFile = new File(DirectoryManager.getGenomeCacheDirectory(), getUserDefinedGenomeListFile());
         File backup = null;
         if (listFile.exists()) {
             backup = new File(listFile.getAbsolutePath() + ".bak");
@@ -1186,11 +1228,209 @@ public class GenomeManager {
         if (monitor != null) monitor.fireProgressChange(75);
 
         GenomeListItem newItem = new GenomeListItem(genomeDisplayName, genomeFile.getAbsolutePath(), genomeId);
-        addGenomeItem(newItem);
+        addGenomeItem(newItem, true);
         return newItem;
 
     }
 
+
+    /**
+     * Downloads .genome file, INCLUDING fasta file it points to,
+     * and rewrites the property.txt file to point to the local version
+     *
+     * @param srcPath
+     * @param targetDir
+     * @throws IOException
+     */
+    public RunnableResult downloadWholeGenome(String srcPath, File targetDir, Frame dialogsParent) throws IOException {
+
+        //Whether the srcGenome is simply a fasta file
+        boolean isFastaFile = FastaUtils.isFastaPath(srcPath);
+        boolean showProgressDialog = !Globals.isHeadless() && !Globals.isBatch();
+        String srcFileName = Utilities.getFileNameFromURL(srcPath);
+
+        if (isFastaFile) {
+            //Simple case, just need to copy fasta and create index
+            return downloadFasta(srcPath, targetDir, srcFileName, dialogsParent);
+
+        } else if (srcFileName.endsWith(Globals.GENOME_FILE_EXTENSION)) {
+            //Most useful case of a .genome file, which contains nearly everything in it, except the
+            //sequence which is a fasta file stored elsewhere
+            String genomeName = Utilities.getFileNameFromURL(srcPath);
+            File srcGenomeArchive = new File(targetDir, genomeName);
+            RunnableResult genomeResult = HttpUtils.getInstance().downloadFile(srcPath, srcGenomeArchive);
+            if (!genomeResult.isSuccess()) return genomeResult;
+
+            GenomeDescriptor descriptor = parseGenomeArchiveFile(srcGenomeArchive);
+
+            File targetGenomeFile = new File(targetDir, srcGenomeArchive.getName());
+            String sequencePath = descriptor.getSequenceLocation();
+
+            boolean seqIsFasta = FastaUtils.isFastaPath(sequencePath);
+            if (!seqIsFasta) {
+                String msg = ("This genome sequence is not available for download. \n" +
+                        "Please contact the igv team at https://groups.google.com/forum/#!forum/igv-help for further assistance");
+                MessageUtils.showMessage(msg);
+                return RunnableResult.CANCELLED;
+            }
+
+            String sequenceFileName = Utilities.getFileNameFromURL(sequencePath);
+            File localSequenceFile = new File(targetDir, sequenceFileName);
+
+            // Copy file directly from the server to local area
+            // Shows cancellable dialog
+            RunnableResult fastaResult = downloadFasta(sequencePath, targetDir, sequenceFileName, dialogsParent);
+            if (fastaResult.isSuccess()) {
+                //Rewrite properties file to point to local fasta
+                rewriteSequenceLocation(targetGenomeFile, localSequenceFile.getAbsolutePath());
+            }
+            return fastaResult;
+        } else {
+            throw new IllegalArgumentException("Unknown file type. Cannot download " + srcPath);
+        }
+    }
+
+
+    /**
+     * Download a fasta file. Also attempts to download the index, and create an index
+     * if that download fails
+     *
+     * @param fastaPath     Full source path of fasta
+     * @param targetDir     Destination directory for fasta file
+     * @param targetName    Destination file name for fasta file
+     * @param dialogsParent Parent of progress dialog shown. None shown if null
+     * @return
+     * @throws IOException
+     */
+    private RunnableResult downloadFasta(String fastaPath, File targetDir, String targetName, Frame dialogsParent) throws IOException {
+        //Simple case, just need to copy fasta and create index
+        File destFile = new File(targetDir, targetName);
+        //TODO PROMPT TO OVERWRITE IF FILE EXISTS
+        URLDownloader urlDownloader = HttpUtils.getInstance().downloadFile(fastaPath, destFile, dialogsParent, "Downloading genome sequence");
+        RunnableResult fastaResult = urlDownloader.getResult();
+        //If not successful for whatever reason, we don't get the index
+        if (!fastaResult.isSuccess()) return fastaResult;
+
+        File destIndexFile = new File(destFile.getAbsolutePath() + ".fai");
+        String srcIndexPath = fastaPath + ".fai";
+        RunnableResult idxResult = null;
+        if (ParsingUtils.resourceExists(srcIndexPath)) {
+            idxResult = HttpUtils.getInstance().downloadFile(srcIndexPath, destIndexFile);
+        }
+        //If remote fasta doesn't exist or download failed, we create our own index
+        if (idxResult == null || idxResult == RunnableResult.FAILURE) {
+            FastaUtils.createIndexFile(destFile.getAbsolutePath(), destIndexFile.getAbsolutePath());
+        }
+
+        return fastaResult;
+    }
+
+    /**
+     * Rewrite the {@link Globals#GENOME_ARCHIVE_SEQUENCE_FILE_LOCATION_KEY} property to equal
+     * the specified {@code newSequencePath}. Works by creating a temp file and renaming
+     *
+     * @param targetFile      A .genome file, in zip format
+     * @param newSequencePath
+     * @return boolean indicating success or failure.
+     * @throws IOException
+     */
+    static boolean rewriteSequenceLocation(File targetFile, String newSequencePath) throws IOException {
+
+        ZipFile targetZipFile = new ZipFile(targetFile);
+        boolean success = false;
+
+        File tmpZipFile = File.createTempFile("tmpGenome", ".zip");
+        ZipEntry propEntry = targetZipFile.getEntry(Globals.GENOME_ARCHIVE_PROPERTY_FILE_NAME);
+
+        InputStream propertyInputStream = null;
+        ZipOutputStream zipOutputStream = null;
+        Properties inputProperties = new Properties();
+
+
+        try {
+            propertyInputStream = targetZipFile.getInputStream(propEntry);
+            BufferedReader reader = new BufferedReader(new InputStreamReader(propertyInputStream));
+
+            //Copy over property.txt, only replacing a few properties
+            inputProperties.load(reader);
+            inputProperties.put(Globals.GENOME_ARCHIVE_SEQUENCE_FILE_LOCATION_KEY, newSequencePath);
+            inputProperties.put(Globals.GENOME_ARCHIVE_CUSTOM_SEQUENCE_LOCATION_KEY, Boolean.TRUE.toString());
+
+
+            ByteArrayOutputStream propertyBytes = new ByteArrayOutputStream();
+            PrintWriter propertyFileWriter = new PrintWriter(new OutputStreamWriter(propertyBytes));
+
+            inputProperties.store(propertyFileWriter, null);
+
+            propertyFileWriter.flush();
+            byte[] newPropertyBytes = propertyBytes.toByteArray();
+
+            Enumeration<? extends ZipEntry> entries = targetZipFile.entries();
+            zipOutputStream = new ZipOutputStream(new FileOutputStream(tmpZipFile));
+            while (entries.hasMoreElements()) {
+                ZipEntry curEntry = entries.nextElement();
+                ZipEntry writeEntry = null;
+
+                if (curEntry.getName().equals(Globals.GENOME_ARCHIVE_PROPERTY_FILE_NAME)) {
+                    writeEntry = new ZipEntry(Globals.GENOME_ARCHIVE_PROPERTY_FILE_NAME);
+                    writeEntry.setSize(newPropertyBytes.length);
+                    zipOutputStream.putNextEntry(writeEntry);
+                    zipOutputStream.write(newPropertyBytes);
+                    continue;
+                } else {
+                    //Because the compressed size can vary,
+                    //we generate a new ZipEntry and copy some attributes
+                    writeEntry = new ZipEntry(curEntry.getName());
+                    writeEntry.setSize(curEntry.getSize());
+                    writeEntry.setComment(curEntry.getComment());
+                    writeEntry.setTime(curEntry.getTime());
+                }
+
+                zipOutputStream.putNextEntry(writeEntry);
+                InputStream tmpIS = null;
+                try {
+                    tmpIS = targetZipFile.getInputStream(writeEntry);
+                    int bytes = IOUtils.copy(tmpIS, zipOutputStream);
+                    log.debug(bytes + " bytes written to " + targetFile);
+                } finally {
+                    if (tmpIS != null) tmpIS.close();
+                }
+
+            }
+        } catch (Exception e) {
+            tmpZipFile.delete();
+            throw new RuntimeException(e.getMessage(), e);
+        } finally {
+            if (propertyInputStream != null) propertyInputStream.close();
+            if (zipOutputStream != null) {
+                zipOutputStream.flush();
+                zipOutputStream.finish();
+                zipOutputStream.close();
+            }
+            zipOutputStream = null;
+            System.gc();
+            success = true;
+        }
+
+        //This is a hack. I don't know why it's necessary,
+        //but for some reason the output zip file seems to be corrupt
+        //at least when called from GenomeManager.refreshArchive
+        try {
+            Thread.sleep(1500);
+        } catch (InterruptedException e) {
+            //
+        }
+
+        //Rename tmp file
+        if (success) {
+            targetFile.delete();
+            FileUtils.copyFile(tmpZipFile, targetFile);
+            success = targetFile.exists();
+            tmpZipFile.delete();
+        }
+        return success;
+    }
+
     public String getGenomeId() {
         return currentGenome == null ? null : currentGenome.getId();
     }
@@ -1206,19 +1446,19 @@ public class GenomeManager {
         return currentGenome;
     }
 
-    public void addGenomeItems(Collection<GenomeListItem> genomeListItems) {
+    public void addGenomeItems(Collection<GenomeListItem> genomeListItems, boolean userDefined) {
         for (GenomeListItem genomeListItem : genomeListItems) {
             genomeItemMap.put(genomeListItem.getId(), genomeListItem);
-            userDefinedGenomeArchiveList.add(genomeListItem);
+            if (userDefined) userDefinedGenomeArchiveList.add(genomeListItem);
         }
         PreferenceManager.getInstance().saveGenomeIdDisplayList(genomeItemMap.values());
         updateImportedGenomePropertyFile();
     }
 
-    public void addGenomeItem(GenomeListItem genomeListItem) {
+    public void addGenomeItem(GenomeListItem genomeListItem, boolean userDefined) {
         genomeItemMap.put(genomeListItem.getId(), genomeListItem);
         PreferenceManager.getInstance().saveGenomeIdDisplayList(genomeItemMap.values());
-        userDefinedGenomeArchiveList.add(genomeListItem);
+        if (userDefined) userDefinedGenomeArchiveList.add(genomeListItem);
         updateImportedGenomePropertyFile();
     }
 
@@ -1295,7 +1535,7 @@ public class GenomeManager {
                 MessageUtils.showMessage("ERROR: Unrecognized annotation file format: " + geneFileName +
                         "<br>Annotations for genome: " + genome.getId() + " will not be loaded.");
             } else {
-                List<org.broad.tribble.Feature> genes = parser.loadFeatures(reader, genome);
+                List<htsjdk.tribble.Feature> genes = parser.loadFeatures(reader, genome);
                 String name = geneTrackName;
                 if (name == null) name = "Genes";
 
@@ -1321,7 +1561,7 @@ public class GenomeManager {
      * @param genome
      * @param features
      */
-    public FeatureTrack createGeneTrack(Genome genome, List<org.broad.tribble.Feature> features) {
+    public FeatureTrack createGeneTrack(Genome genome, List<htsjdk.tribble.Feature> features) {
 
         FeatureDB.clearFeatures();
         FeatureTrack geneFeatureTrack = null;
@@ -1342,4 +1582,31 @@ public class GenomeManager {
     }
 
 
+    /**
+     * Delete the specified .genome files and their sequences, only if they were downloaded from the
+     * server. Doesn't touch user defined genomes
+     *
+     * @param removedValuesList
+     */
+    public void deleteDownloadedGenomes(List<GenomeListItem> removedValuesList) throws IOException {
+        Collection<GenomeListItem> userDefinedGenomes = getUserDefinedGenomeArchiveList();
+        for (GenomeListItem item : removedValuesList) {
+            if (userDefinedGenomes.contains(item)) {
+                continue;
+            }
+
+            String loc = item.getLocation();
+            if (!HttpUtils.isRemoteURL(loc)) {
+                File genFile = new File(loc);
+                GenomeDescriptor descriptor = parseGenomeArchiveFile(genFile);
+                if (!HttpUtils.isRemoteURL(descriptor.getSequenceLocation())) {
+                    File seqFile = new File(descriptor.getSequenceLocation());
+                    seqFile.delete();
+                    File indexFile = new File(seqFile.getAbsolutePath() + ".fai");
+                    indexFile.delete();
+                }
+                genFile.delete();
+            }
+        }
+    }
 }
