@@ -26,12 +26,11 @@ import org.broad.igv.util.ParsingUtils;
 import org.broad.igv.util.StringUtils;
 import org.broad.igv.util.collections.CI;
 import org.broad.igv.util.collections.MultiMap;
-import org.broad.tribble.AsciiFeatureCodec;
-import org.broad.tribble.Feature;
-import org.broad.tribble.exception.CodecLineParsingException;
-import org.broad.tribble.readers.LineReader;
+import htsjdk.tribble.AsciiFeatureCodec;
+import htsjdk.tribble.Feature;
+import htsjdk.tribble.exception.CodecLineParsingException;
+import htsjdk.tribble.readers.LineIterator;
 
-import java.io.IOException;
 import java.util.*;
 
 /**
@@ -56,7 +55,6 @@ public class GFFCodec extends AsciiFeatureCodec<Feature> {
     private static Logger log = Logger.getLogger(GFFCodec.class);
 
 
-
     static HashSet<String> ignoredTypes = new HashSet();
 
     static {
@@ -64,16 +62,16 @@ public class GFFCodec extends AsciiFeatureCodec<Feature> {
         ignoredTypes.add("stop_codon");
         ignoredTypes.add("Contig");
         ignoredTypes.add("RealContig");
+        ignoredTypes.add("CDS_parts");
     }
 
 
     private TrackProperties trackProperties = null;
-    CI.CIHashSet featuresToHide = new CI.CIHashSet();
-
-
-    FeatureFileHeader header;
-    Helper helper;
-    Genome genome;
+    private CI.CIHashSet featuresToHide = new CI.CIHashSet();
+    private FeatureFileHeader header;
+    private Helper helper;
+    private Genome genome;
+    private boolean fastaSection = false;
 
     public enum Version {
         GFF2, GFF3
@@ -136,7 +134,7 @@ public class GFFCodec extends AsciiFeatureCodec<Feature> {
         }
     }
 
-    public Object readHeader(LineReader reader) {
+    public Object readActualHeader(LineIterator reader) {
 
         if (header == null) {
             header = new FeatureFileHeader();
@@ -144,11 +142,12 @@ public class GFFCodec extends AsciiFeatureCodec<Feature> {
         String line;
         int nLines = 0;
         try {
-            while ((line = reader.readLine()) != null) {
-
+            while (reader.hasNext()) {
+                line = reader.peek();
                 if (line.startsWith("#")) {
                     nLines++;
                     readHeaderLine(line);
+                    reader.next();
                 } else {
                     break;
                 }
@@ -156,8 +155,8 @@ public class GFFCodec extends AsciiFeatureCodec<Feature> {
 
             header.setTrackProperties(trackProperties);
             return header;
-        } catch (IOException e) {
-            throw new CodecLineParsingException("Error parsing header", e);
+        } catch (Exception e) {
+            throw new CodecLineParsingException("Error parsing header: " + e.getMessage(), e);
         }
     }
 
@@ -177,7 +176,7 @@ public class GFFCodec extends AsciiFeatureCodec<Feature> {
     public boolean canDecode(String path) {
         final String pathLowerCase = path.toLowerCase();
         return pathLowerCase.endsWith(".gff") || pathLowerCase.endsWith(".gff3") ||
-                pathLowerCase.endsWith(".gvf");
+                pathLowerCase.endsWith(".gvf") || pathLowerCase.endsWith(".gtf");
     }
 
     public BasicFeature decodeLoc(String line) {
@@ -186,8 +185,13 @@ public class GFFCodec extends AsciiFeatureCodec<Feature> {
 
     public BasicFeature decode(String line) {
 
+        if (fastaSection) {
+            return null;
+        }
         if (line.startsWith("#")) {
-            // This should not be possible as this line would be parsed as a header.  But just in case
+            if (line.toUpperCase().startsWith("##FASTA")) {
+                fastaSection = true;
+            }
             return null;
         }
 
@@ -231,7 +235,7 @@ public class GFFCodec extends AsciiFeatureCodec<Feature> {
 
         helper.parseAttributes(attributeString, attributes);
 
-        String id = helper.getID(attributes);
+        String id = helper.getID(attributes, featureType);
         String[] parentIds = helper.getParentIds(attributes, attributeString);
 
         BasicFeature f = new BasicFeature(chromosome, start, end, strand);
@@ -312,7 +316,7 @@ public class GFFCodec extends AsciiFeatureCodec<Feature> {
 
         void parseAttributes(String attributeString, MultiMap<String, String> map);
 
-        String getID(MultiMap<String, String> attributes);
+        String getID(MultiMap<String, String> attributes, String type);
 
         void setUrlDecoding(boolean b);
 
@@ -325,14 +329,14 @@ public class GFFCodec extends AsciiFeatureCodec<Feature> {
     public static class GFF2Helper implements Helper {
 
         //TODO Almost identical
-        static String[] DEFAULT_NAME_FIELDS = {"alias", "gene", "ID", "Locus", "locus", "Name", "name", "primary_name", "systematic_id", "transcript_id"};
+        static String[] DEFAULT_NAME_FIELDS = {"alias", "gene", "ID", "Locus", "locus", "Name", "name", "gene_name", "primary_name", "systematic_id", "transcript_id"};
         static List<String> idFields = new ArrayList<String>(Arrays.asList(DEFAULT_NAME_FIELDS));
 
         static {
             idFields.add("transcript_id");
         }
 
-        static String[] possParentNames = new String[]{"id", "mRNA", "systematic_id", "transcript_id", "gene", "transcriptId", "Parent", "proteinId"};
+        static String[] possParentNames = new String[]{"transcript_id", "id", "mRNA", "systematic_id", "gene", "transcriptId", "Parent", "proteinId"};
 
         private String[] nameFields;
 
@@ -377,28 +381,41 @@ public class GFFCodec extends AsciiFeatureCodec<Feature> {
 
         public String[] getParentIds(MultiMap<String, String> attributes, String attributeString) {
 
-            String[] parentIds = new String[1];
-            if (attributes.size() == 0) {
-                parentIds[0] = attributeString;
-            } else {
+            if (attributes.size() > 0) {
                 for (String possName : possParentNames) {
                     if (attributes.containsKey(possName)) {
-                        parentIds[0] = attributes.get(possName);
-                        break;
+                        String parent = attributes.get(possName).trim();
+                        if (parent.length() > 0) {
+                            return new String[] {parent};
+                        }
                     }
                 }
             }
-            return parentIds;
+            return null;
         }
 
 
-        public String getID(MultiMap<String, String> attributes) {
+        public String getID(MultiMap<String, String> attributes, String type) {
+
+            //Search for an attribute == type,  take this as ID
+            String id = attributes.get(type);
+            if (id != null && id.length() > 0) {
+                return id;
+            }
+
             for (String nf : idFields) {
                 if (attributes.containsKey(nf)) {
-                    return attributes.get(nf);
+                    String tmp = attributes.get(nf).trim();
+                    if(tmp.length() > 0) return tmp;
                 }
             }
-            return getName(attributes);
+
+            String tmp = getName(attributes);
+            if(tmp != null && tmp.trim().length() > 0) {
+                return tmp.trim();
+            }
+
+            return null;
         }
 
         public String getName(MultiMap<String, String> attributes) {
@@ -422,7 +439,7 @@ public class GFFCodec extends AsciiFeatureCodec<Feature> {
 
     public static class GFF3Helper implements Helper {
 
-        static String[] DEFAULT_NAME_FIELDS = {"Name", "Alias", "ID", "gene", "locus"};
+        static String[] DEFAULT_NAME_FIELDS = {"Name", "Alias", "ID", "gene", "locus", "gene_name"};
         private boolean useUrlDecoding = true;
 
         private String[] nameFields;
@@ -470,10 +487,6 @@ public class GFFCodec extends AsciiFeatureCodec<Feature> {
                     if (useUrlDecoding) {
                         key = StringUtils.decodeURL(key);
                         value = StringUtils.decodeURL(value);
-                        // Limit values to 50 characters
-                        if (value.length() > 50) {
-                            value = value.substring(0, 50) + " ...";
-                        }
                     }
                     kvalues.put(StringUtils.intern(key), value);
                 } else {
@@ -499,7 +512,7 @@ public class GFFCodec extends AsciiFeatureCodec<Feature> {
             return null;
         }
 
-        public String getID(MultiMap<String, String> attributes) {
+        public String getID(MultiMap<String, String> attributes, String ignore) {
             return attributes.get("ID");
         }
 
@@ -509,58 +522,4 @@ public class GFFCodec extends AsciiFeatureCodec<Feature> {
     }
 
 
-    /**
-     * Helper for GTF files
-     * <p/>
-     * mandatory attributes
-     * gene_id value;     A globally unique identifier for the genomic source of the transcript
-     * transcript_id value;     A globally unique identifier for the predicted transcript.
-     * <p/>
-     * Attributes must end in a semicolon which must then be separated from the start of any subsequent
-     * attribute by exactly one space character (NOT a tab character).
-     * <p/>
-     * Textual attributes should be surrounded by doublequotes.
-     */
-    public static class GTFHelper implements Helper {
-
-        @Override
-        public void parseAttributes(String description, MultiMap<String, String> kvalues) {
-
-            List<String> kvPairs = StringUtils.breakQuotedString(description.trim(), ';');
-            for (String kv : kvPairs) {
-                List<String> tokens = StringUtils.breakQuotedString(kv, ' ');
-                if (tokens.size() >= 2) {
-                    String key = tokens.get(0).trim().replaceAll("\"", "");
-                    String value = tokens.get(1).trim().replaceAll("\"", "");
-                    kvalues.put(StringUtils.intern(key), value);
-                }
-            }
-        }
-
-        @Override
-        public String[] getParentIds(MultiMap<String, String> attributes, String attributeString) {
-            return new String[0];  //To change body of implemented methods use File | Settings | File Templates.
-        }
-
-        @Override
-        public String getID(MultiMap<String, String> attributes) {
-            return attributes.get("transcript_id");  //To change body of implemented methods use File | Settings | File Templates.
-        }
-
-        @Override
-        public void setUrlDecoding(boolean b) {
-            //To change body of implemented methods use File | Settings | File Templates.
-        }
-
-        @Override
-        public String getName(MultiMap<String, String> attributes) {
-            return attributes.get("transcript_id");  //To change body of implemented methods use File | Settings | File Templates.
-        }
-
-        @Override
-        public void setNameFields(String[] fields) {
-            //To change body of implemented methods use File | Settings | File Templates.
-        }
-    }
-
 }
