@@ -31,10 +31,7 @@ import org.broad.igv.PreferenceManager;
 import org.broad.igv.data.CharArrayList;
 import org.broad.igv.feature.LocusScore;
 import org.broad.igv.feature.Strand;
-import org.broad.igv.sam.Alignment;
-import org.broad.igv.sam.AlignmentBlock;
-import org.broad.igv.sam.ReadMate;
-import org.broad.igv.sam.SamAlignment;
+import org.broad.igv.sam.*;
 import org.broad.igv.track.WindowFunction;
 
 import java.awt.*;
@@ -63,7 +60,6 @@ public class GobyAlignment implements Alignment {
     private final GobyAlignmentIterator iterator;
     protected AlignmentBlock[] block = new AlignmentBlock[1];
     protected AlignmentBlock[] insertionBlock;
-    private Color defaultColor = new Color(200, 200, 200);
     private CharArrayList gapTypes = null;
     private static final ReadMate unmappedMate = new ReadMate("*", -1, false, true);
     private Comparator<? super AlignmentBlock> blockComparator = new Comparator<AlignmentBlock>() {
@@ -227,7 +223,7 @@ public class GobyAlignment implements Alignment {
                 if (spliceHeadAlignment.gapTypes == null) {
                     spliceHeadAlignment.gapTypes = new CharArrayList(10);
                 }
-                spliceHeadAlignment.gapTypes.add(SamAlignment.SKIPPED_REGION);
+                spliceHeadAlignment.gapTypes.add(SAMAlignment.SKIPPED_REGION);
 
                 // Since the previous alignment carries this information, we clear up block and insertionBlock
                 // in this alignment, but keep any softClips:
@@ -418,10 +414,15 @@ public class GobyAlignment implements Alignment {
     /**
      * Get the reference id from the iterator, prepend "chr".
      */
-    public String getChromosome() {
+    public String getChr() {
         return getChromosome(entry.getTargetIndex());
     }
 
+    @Override
+    public String getContig() {
+        return getChr();
+    }
+
     /**
      * Get the reference id from the iterator, prepend "chr".
      *
@@ -431,11 +432,6 @@ public class GobyAlignment implements Alignment {
         return "chr" + iterator.getId(targetIndex).toString();
     }
 
-    public String getChr() {
-        //LOG.info("getChr");
-        return getChromosome();
-    }
-
     public int getAlignmentStart() {
         //     //LOG.info("getAlignmentStart");
         return entry.getPosition();
@@ -674,8 +670,8 @@ public class GobyAlignment implements Alignment {
      *
      * @return
      */
-    public Color getDefaultColor() {
-        return defaultColor;
+    public Color getColor() {
+        return null;
     }
 
     public int getStart() {
@@ -742,8 +738,8 @@ public class GobyAlignment implements Alignment {
 
         if (this.isPaired()) {
             buffer.append("----------------------" + "<br>");
-            buffer.append("Pair start = " + getMate().positionString() + "<br>");
-            buffer.append("Pair is mapped = " + (getMate().isMapped() ? "yes" : "no") + "<br>");
+            buffer.append("Mate start = " + getMate().positionString() + "<br>");
+            buffer.append("Mate is mapped = " + (getMate().isMapped() ? "yes" : "no") + "<br>");
             //buf.append("Pair is proper = " + (getProperPairFlag() ? "yes" : "no") + "<br>");
             if (getChr().equals(getMate().getChr())) {
                 buffer.append("Insert size = " + getInferredInsertSize() + "<br>");
@@ -813,4 +809,10 @@ public class GobyAlignment implements Alignment {
     public boolean isPrimary() {
         return !EntryFlagHelper.isNotPrimaryAlignment(entry);
     }
+
+    @Override
+    public boolean isSupplementary() {
+        // The SAM 0x0800 tag
+        return false;
+    }
 }
