@@ -49,8 +49,8 @@ public class IGVFeatureRenderer extends FeatureRenderer {
     static final int THIN_BLOCK_HEIGHT = 6;
     protected Color AA_COLOR_1 = new Color(92, 92, 164);
     protected Color AA_COLOR_2 = new Color(12, 12, 120);
-    static final Color DULL_BLUE = new Color(0, 0, 200);
-    static final Color DULL_RED = new Color(200, 0, 0);
+    public static final Color DULL_BLUE = new Color(0, 0, 200);
+    public static final Color DULL_RED = new Color(200, 0, 0);
     static final int NON_CODING_HEIGHT = 8;
 
     float viewLimitMin = Float.NaN;
@@ -70,11 +70,12 @@ public class IGVFeatureRenderer extends FeatureRenderer {
     //Map from Exon to y offset
     //Could use more coordinates, but they are all contained in the Exon
     private Map<IExon, Integer> exonMap = new HashMap<IExon, Integer>(100);
-    private AlternativeSpliceGraph<Integer> exonGraph = new AlternativeSpliceGraph<Integer>();
     private Set<String> drawnNames = new HashSet<String>(100);
 
     protected boolean isGenotypeRenderer = false;
 
+    private static final int MAX_NAME_LENGTH = 60;
+
 
     /**
      * Note:  assumption is that featureList is sorted by start position.
@@ -140,20 +141,21 @@ public class IGVFeatureRenderer extends FeatureRenderer {
                 double virtualPixelStart = (feature.getStart() - origin) / locScale;
                 double virtualPixelEnd = (feature.getEnd() - origin) / locScale;
 
-                int pixelEnd = (int) Math.round(Math.min(trackRectangleMaxX, virtualPixelEnd));
                 int pixelStart = (int) Math.round(Math.max(trackRectangleX, virtualPixelStart));
+                int pixelEnd = (int) Math.round(Math.min(trackRectangleMaxX, virtualPixelEnd));
 
+                final int pixelWidth = pixelEnd - pixelStart;
                 if (isGenotypeRenderer) {
-                    if ((pixelEnd - pixelStart) < 3) {
-                        double dx = 3.0 - (pixelEnd - pixelStart);
+                    if (pixelWidth < 3) {
+                        double dx = 3.0 - pixelWidth;
                         pixelStart -= dx / 2.0;
                         pixelEnd += dx / 2.0;
                     }
                 }
 
-                // Draw a maximum of "maxOcclusions" features on top of each other.
+                // Draw a maximum of "maxOcclusion" small features on top of each other.
                 if (pixelEnd <= lastPixelEnd) {
-                    if (occludedCount >= maxOcclusions) {
+                    if (occludedCount >= maxOcclusions && (pixelEnd - pixelStart) < 3) {
                         continue;
                     } else {
                         occludedCount++;
@@ -177,36 +179,58 @@ public class IGVFeatureRenderer extends FeatureRenderer {
                 boolean hasExons = false;
                 if (feature instanceof BasicFeature) {
                     BasicFeature bf = (BasicFeature) feature;
-                    pixelThickStart = (int) Math.max(trackRectangleX, (bf.getThickStart() - origin) / locScale);
-                    pixelThickEnd = (int) Math.min(trackRectangleMaxX, (bf.getThickEnd() - origin) / locScale);
+                    pixelThickStart = (int) Math.max(trackRectangleX, Math.round((bf.getThickStart() - origin) / locScale));
+                    pixelThickEnd = (int) Math.min(trackRectangleMaxX, Math.round((bf.getThickEnd() - origin) / locScale));
                     hasExons = bf.hasExons();
                 }
 
-                // Add directional arrows and exons, if there is room.
+                 // Add directional arrows and exons, if there is room.
                 int pixelYCenter = trackRectangle.y + NORMAL_STRAND_Y_OFFSET / 2;
-                if (!hasExons) {
-                    drawFeatureBlock(pixelStart, pixelEnd, pixelThickStart, pixelThickEnd, pixelYCenter, g2D);
-                }
 
-
-                if ((pixelEnd - pixelStart < 3) && hasExons) {
-                    drawFeatureBounds(pixelStart, pixelEnd, pixelYCenter, g2D);
-                } else {
-                    if (hasExons) {
+                if (hasExons) {
+                    if ((pixelWidth < 3)) {
+                        drawFeatureBounds(pixelStart, pixelEnd, pixelYCenter, g2D);
+                    } else {
                         drawExons(feature, pixelYCenter, context, g2D, trackRectangle, displayMode,
                                 alternateExonColor, track.getColor(), track.getAltColor());
-                    } else {
-                        Graphics2D arrowGraphics = context.getGraphic2DForColor(Color.WHITE);
-                        drawStrandArrows(feature.getStrand(), pixelStart, pixelEnd, pixelYCenter, 0,
-                                displayMode, arrowGraphics);
+                    }
+                } else {
+
+                    drawFeatureBlock(pixelStart, pixelEnd, pixelThickStart, pixelThickEnd, pixelYCenter, g2D);
+                    Graphics2D arrowGraphics = context.getGraphic2DForColor(Color.WHITE);
+                    drawStrandArrows(feature.getStrand(), pixelStart, pixelEnd, pixelYCenter, 0,
+                            displayMode, trackRectangle, arrowGraphics);
+
+                    // This is ugly, but alternatives are probably worse
+                    if (feature instanceof EncodePeakFeature && pixelWidth > 5) {
+                        int peakPosition = ((EncodePeakFeature) feature).getPeakPosition();
+                        if (peakPosition > 0) {
+                            Color c = g2D.getColor();
+                            int peakPixelPosition = (int) ((peakPosition - origin) / locScale);
+                            Color peakColor = c == Color.black ? Color.red : Color.black;
+                            //if (track.isUseScore()) {
+                            //    float alpha = getAlpha(0, 500, feature.getScore());
+                            //    peakColor = ColorUtilities.getCompositeColor(DULL_RED, alpha);
+                            //}
+                            g2D.setColor(peakColor);
+                            int pw = Math.min(3, pixelWidth / 5);
+                            g2D.fillRect(peakPixelPosition - pw/2, pixelYCenter - thinBlockHeight/2 - 1, pw, thinBlockHeight + 2);
+                            g2D.setColor(c);
+                        }
                     }
 
                 }
 
+
                 // Draw name , if there is room
                 if (displayMode != Track.DisplayMode.SQUISHED) {
                     String name = feature.getName();
                     if (name != null) {
+                        // Limit name display length
+                        if (name.length() >= MAX_NAME_LENGTH) {
+                            name = name.substring(0, MAX_NAME_LENGTH - 3) + " ...";
+                        }
+
                         LineMetrics lineMetrics = font.getLineMetrics(name, g2D.getFontRenderContext());
                         int fontHeight = (int) Math.ceil(lineMetrics.getHeight());
 
@@ -216,7 +240,7 @@ public class IGVFeatureRenderer extends FeatureRenderer {
                         // the track rectangle.
                         int nameStart = Math.max(0, pixelStart);
                         int nameEnd = Math.min(pixelEnd, (int) trackRectangle.getWidth());
-                        int textBaselineY = pixelYCenter + blockHeight;
+                        int textBaselineY = trackRectangle.y + trackRectangle.height - 3;
 
                         // Calculate the minimum amount of vertical track
                         // space required be we  draw the
@@ -234,7 +258,7 @@ public class IGVFeatureRenderer extends FeatureRenderer {
                 if (getHighlightFeature() == feature) {
                     int yStart = pixelYCenter - blockHeight / 2 - 1;
                     Graphics2D highlightGraphics = context.getGraphic2DForColor(Color.cyan);
-                    highlightGraphics.drawRect(pixelStart - 1, yStart, (pixelEnd - pixelStart + 2), blockHeight + 2);
+                    highlightGraphics.drawRect(pixelStart - 1, yStart, (pixelWidth + 2), blockHeight + 2);
 
 
                 }
@@ -268,7 +292,7 @@ public class IGVFeatureRenderer extends FeatureRenderer {
      * @param g
      */
     private void drawFeatureBlock(int pixelStart, int pixelEnd, int pixelThickStart, int pixelThickEnd,
-                                        int yOffset, Graphics2D g) {
+                                  int yOffset, Graphics2D g) {
 
         Graphics2D g2D = (Graphics2D) g.create();
 
@@ -290,10 +314,11 @@ public class IGVFeatureRenderer extends FeatureRenderer {
     final private void drawConnectingLine(int startX, int startY, int endX, int endY, Strand strand, Graphics2D g) {
 
         Graphics2D g2D = (Graphics2D) g.create();
-
-        float lineThickness = ((BasicStroke) g.getStroke()).getLineWidth();
         if (strand == null) {
 
+            BasicStroke befStroke = (BasicStroke) g.getStroke();
+            float lineThickness = befStroke.getLineWidth();
+
             // Double the line thickness
             lineThickness *= NO_STRAND_THICKNESS;
             Stroke stroke = new BasicStroke(lineThickness);
@@ -324,7 +349,7 @@ public class IGVFeatureRenderer extends FeatureRenderer {
                              Graphics2D g2D, Rectangle trackRectangle, Track.DisplayMode mode,
                              boolean alternateExonColor, Color color1, Color color2) {
 
-        Graphics2D exonNumberGraphics = (Graphics2D)g2D.create();
+        Graphics2D exonNumberGraphics = (Graphics2D) g2D.create();
 
 
         exonNumberGraphics.setColor(Color.BLACK);
@@ -354,8 +379,11 @@ public class IGVFeatureRenderer extends FeatureRenderer {
         int maxLineEndX = Integer.MIN_VALUE;
         IExon lastExon = null;
 
-        exonGraph.startFeature();
-        for (Exon exon : gene.getExons()) {
+        int exonCount = gene.getExons().size();
+
+        for (int idx = 0; idx < exonCount; idx++) {
+
+            Exon exon = gene.getExons().get(idx);
 
             // Parse expression from tags, if available
             //Credit Michael Poidinger and Solomonraj Wilson, Singapore Immunology Network.
@@ -389,18 +417,6 @@ public class IGVFeatureRenderer extends FeatureRenderer {
             int curYOffset = yOffset;
             boolean drawConnectingLine = true;
 
-            if (mode == Track.DisplayMode.ALTERNATIVE_SPLICE) {
-
-                IExon eProx = Exon.getExonProxy(exon);
-                drawConnectingLine = !exonGraph.containsEdge(lastExon, eProx);
-                if (exonGraph.hasParameter(eProx)) {
-                    curYOffset = exonGraph.getParameter(eProx);
-                } else {
-                    exonGraph.put(eProx, curYOffset);
-                }
-                lastExon = eProx;
-            }
-
             int pStart = getPixelFromChromosomeLocation(exon.getChr(), exon.getStart(), theOrigin, locationScale);
             int pEnd = getPixelFromChromosomeLocation(exon.getChr(), exon.getEnd(), theOrigin, locationScale);
 
@@ -412,7 +428,7 @@ public class IGVFeatureRenderer extends FeatureRenderer {
                     && lastExonEndX >= maxLineEndX) {
                 drawConnectingLine(lastExonEndX, lastY, pStart, curYOffset, exon.getStrand(), blockGraphics);
                 double angle = Math.atan(-(curYOffset - lastY) / ((pStart - lastExonEndX) + 1e-12));
-                drawStrandArrows(gene.getStrand(), lastExonEndX, pStart, lastY, angle, mode, arrowGraphics);
+                drawStrandArrows(gene.getStrand(), lastExonEndX, pStart, lastY, angle, mode, trackRectangle, arrowGraphics);
                 maxLineEndX = Math.max(maxLineEndX, pStart);
             }
             lastExonEndX = pEnd;
@@ -462,11 +478,11 @@ public class IGVFeatureRenderer extends FeatureRenderer {
 
                 Graphics2D whiteArrowGraphics = context.getGraphic2DForColor(Color.white);
                 drawStrandArrows(gene.getStrand(), pStart + ARROW_SPACING / 2, pEnd, curYOffset, 0, mode,
-                        whiteArrowGraphics);
+                        trackRectangle, whiteArrowGraphics);
 
                 if (locationScale < 0.25) {
                     labelAminoAcids(pStart, fontGraphics, theOrigin, context, gene, locationScale,
-                            curYOffset, exon, trackRectangle);
+                            curYOffset, trackRectangle, idx);
                 }
 
                 if (FrameManager.isExomeMode()) {
@@ -486,12 +502,13 @@ public class IGVFeatureRenderer extends FeatureRenderer {
 
     /**
      * Draw an individual rectangle representing an individual exon
+     *
      * @param x
      * @param y
      * @param width
      * @param height
      */
-    protected void drawExonRect(Graphics blockGraphics, Exon exon, int x, int y, int width, int height){
+    protected void drawExonRect(Graphics blockGraphics, Exon exon, int x, int y, int width, int height) {
         blockGraphics.fillRect(x, y, width, height);
     }
 
@@ -504,37 +521,47 @@ public class IGVFeatureRenderer extends FeatureRenderer {
      * @param mode
      * @param g2D
      */
-    protected void drawStrandArrows(Strand strand, int startX, int endX, int startY, double angle, Track.DisplayMode mode,
+    protected void drawStrandArrows(Strand strand,
+                                    int startX,
+                                    int endX,
+                                    int startY,
+                                    double angle,
+                                    Track.DisplayMode mode,
+                                    Rectangle trackRectangle,
                                     Graphics2D g2D) {
 
+
         //Don't draw arrows if we don't have a strand
         if (!strand.equals(Strand.POSITIVE) && !strand.equals(Strand.NEGATIVE)) {
             return;
         }
 
         // Don't draw strand arrows for very small regions
-        int distance = endX - startX;
-        if ((distance < 6)) {
+        // Limit drawing to visible region, we don't really know the viewport pEnd,
+        if ((endX - startX)  < 6) {
             return;
         }
 
         Graphics2D g = (Graphics2D) g2D.create();
 
-        // Limit drawing to visible region, we don't really know the viewport pEnd,
-        int vStart = 0;
-        int vEnd = 10000;
 
         // Draw the directional arrows on the feature
         int sz = mode == Track.DisplayMode.EXPANDED ? 3 : 2;
         sz = strand.equals(Strand.POSITIVE) ? -sz : sz;
-
         final int asz = Math.abs(sz);
 
         /*
          We draw arrows in a translated and rotated frame.
          This is to deal with alternative splice
          */
-        g.translate(startX, startY);
+
+        int delta = startX < trackRectangle.x ? (int) ((trackRectangle.x - startX) % ARROW_SPACING) : 0;
+        startX = Math.max(startX, trackRectangle.x);
+        endX = Math.min(endX, trackRectangle.x + trackRectangle.width);
+
+        int distance = endX - startX;
+
+        g.translate(startX - delta, startY);
         g.rotate(-angle);
         double endXInFrame = distance / Math.cos(angle);
 
@@ -551,7 +578,7 @@ public class IGVFeatureRenderer extends FeatureRenderer {
                                       int textBaselineY) {
 
         String name = feature.getName();
-        if (name == null || (drawnNames.contains(name) && mode == Track.DisplayMode.ALTERNATIVE_SPLICE)) {
+        if (name == null) {
             return lastFeatureEndedAtPixelX;
         }
 
@@ -585,52 +612,37 @@ public class IGVFeatureRenderer extends FeatureRenderer {
      * @param gene
      * @param locationScale
      * @param yOffset
-     * @param exon
      * @param trackRectangle
+     * @param idx  exon index
      */
     public void labelAminoAcids(int pStart, Graphics2D fontGraphics, double theOrigin,
                                 RenderContext context, IGVFeature gene, double locationScale,
-                                int yOffset, Exon exon, Rectangle trackRectangle) {
+                                int yOffset, Rectangle trackRectangle, int idx) {
 
         Genome genome = GenomeManager.getInstance().getCurrentGenome();
-        AminoAcidSequence aaSequence = exon.getAminoAcidSequence(genome);
+        Exon exon = gene.getExons().get(idx);
+
+        Exon prevExon = idx == 0 ? null : gene.getExons().get(idx-1);
+        Exon nextExon = (idx+1) < gene.getExons().size() ?gene.getExons().get(idx+1) : null;
+
+        AminoAcidSequence aaSequence = exon.getAminoAcidSequence(genome, prevExon, nextExon);
+
         if ((aaSequence != null) && aaSequence.hasNonNullSequence()) {
             Rectangle aaRect = new Rectangle(pStart, yOffset - blockHeight / 2, 1, blockHeight);
 
             int aaSeqStartPosition = aaSequence.getStartPosition();
-            int firstFullAcidIndex = (int) Math.floor((aaSeqStartPosition - exon.getReadingFrame()) / 3);
-            //calculated oddness or evenness of first amino acid. This is also done independently in SequenceRenderer
-            boolean odd = (firstFullAcidIndex % 2) == 1;
-
-            if (aaSeqStartPosition > exon.getStart()) {
-
-                // The codon for the first amino acid is split between this and the previous exon
-                // TODO -- get base from previous exon and compute aaSequence.  For now skipping drawing
-                // the AA label
-                int cdStart = Math.max(exon.getCdStart(), exon.getStart());
-                aaRect.x = getPixelFromChromosomeLocation(exon.getChr(), cdStart, theOrigin,
-                        locationScale);
-                aaRect.width = getPixelFromChromosomeLocation(exon.getChr(),
-                        aaSeqStartPosition, theOrigin, locationScale) - aaRect.x;
-
-                if (trackRectangle.intersects(aaRect)) {
-                    //use opposite color from first AA color
-                    Graphics2D bgGraphics = context.getGraphic2DForColor(!odd ? AA_COLOR_1 : AA_COLOR_2);
-                    bgGraphics.fill(aaRect);
-                }
-            }
-
+            boolean odd =  exon.getAminoAcidNumber(exon.getCdStart()) % 2 == 1;
 
             for (AminoAcid acid : aaSequence.getSequence()) {
                 if (acid != null) {
 
-                    int px = getPixelFromChromosomeLocation(exon.getChr(), aaSeqStartPosition, theOrigin, locationScale);
-                    int px2 = getPixelFromChromosomeLocation(exon.getChr(), aaSeqStartPosition + 3, theOrigin, locationScale);
+                    int start = Math.max(exon.getStart(), aaSeqStartPosition);
+                    int end = Math.min(exon.getEnd(), aaSeqStartPosition + 3);
+                    int px = getPixelFromChromosomeLocation(exon.getChr(), start, theOrigin, locationScale);
+                    int px2 = getPixelFromChromosomeLocation(exon.getChr(), end, theOrigin, locationScale);
 
                     if ((px <= trackRectangle.getMaxX()) && (px2 >= trackRectangle.getX())) {
 
-                        // {
-
                         aaRect.x = px;
                         aaRect.width = px2 - px;
 
@@ -738,7 +750,6 @@ public class IGVFeatureRenderer extends FeatureRenderer {
     @Override
     public void reset() {
         exonMap.clear();
-        exonGraph = new AlternativeSpliceGraph<Integer>();
         drawnNames.clear();
     }
 }
