@@ -22,6 +22,7 @@ package org.broad.igv.renderer;
 
 import org.apache.log4j.Logger;
 import org.broad.igv.Globals;
+import org.broad.igv.PreferenceManager;
 import org.broad.igv.feature.IExon;
 import org.broad.igv.feature.IGVFeature;
 import org.broad.igv.feature.SpliceJunctionFeature;
@@ -31,7 +32,7 @@ import org.broad.igv.sam.IAlignmentDataManager;
 import org.broad.igv.track.RenderContext;
 import org.broad.igv.track.Track;
 import org.broad.igv.ui.FontManager;
-import org.broad.tribble.Feature;
+import htsjdk.tribble.Feature;
 
 import java.awt.*;
 import java.awt.geom.Ellipse2D;
@@ -51,6 +52,18 @@ public class SashimiJunctionRenderer extends IGVFeatureRenderer {
 
     private static Logger log = Logger.getLogger(SashimiJunctionRenderer.class);
 
+    public enum ShapeType{
+        CIRCLE,
+        ELLIPSE,
+        TEXT,
+        NONE
+    }
+
+
+    // Static -- property is shared by all instances
+    private static ShapeType shapeType = ShapeType.TEXT;
+
+
     Color ARC_COLOR_POS = new Color(150, 50, 50, 140); //transparent dull red
 
     private Color color = ARC_COLOR_POS;
@@ -62,7 +75,6 @@ public class SashimiJunctionRenderer extends IGVFeatureRenderer {
     //this depth and deeper will all look the same
     protected int DEFAULT_MAX_DEPTH = 50;
     protected int maxDepth = DEFAULT_MAX_DEPTH;
-    private ShapeType shapeType = ShapeType.TEXT;
     private Set<IExon> selectedExons;
 
     private CoverageTrack coverageTrack = null;
@@ -89,7 +101,7 @@ public class SashimiJunctionRenderer extends IGVFeatureRenderer {
     }
 
     /**
-     * Set the data manager and coverage track.
+     * Set the data manager
      * @param dataManager
      */
     public void setDataManager(IAlignmentDataManager dataManager) {
@@ -108,12 +120,6 @@ public class SashimiJunctionRenderer extends IGVFeatureRenderer {
         return coverageTrack;
     }
 
-    public enum ShapeType{
-        CIRCLE,
-        ELLIPSE,
-        TEXT
-    }
-
     public int getMaxDepth() {
         return maxDepth;
     }
@@ -122,11 +128,11 @@ public class SashimiJunctionRenderer extends IGVFeatureRenderer {
         this.maxDepth = maxDepth;
     }
 
-    public void setShapeType(ShapeType shapeType){
-        this.shapeType = shapeType;
+    public static void setShapeType(ShapeType st){
+        shapeType = st;
     }
 
-    public ShapeType getShapeType() {
+    public static ShapeType getShapeType() {
         return shapeType;
     }
 
@@ -142,10 +148,11 @@ public class SashimiJunctionRenderer extends IGVFeatureRenderer {
     public void setCoverageTrack(CoverageTrack covTrack) {
         this.coverageTrack = new CoverageTrack(covTrack);
         this.coverageTrack.setColor(color);
-        //Don't want to color SNPs
+        //Don't want to color SNPs, so we just set an impossibly high threshold
         coverageTrack.setSnpThreshold(2.0f);
+
         coverageTrack.setAutoScale(true);
-        coverageTrack.rescale();
+        coverageTrack.setGlobalAutoScale(false);
     }
 
 
@@ -168,7 +175,9 @@ public class SashimiJunctionRenderer extends IGVFeatureRenderer {
 
         Rectangle coverageRectangle = new Rectangle(trackRectangle);
 
-        if(coverageTrack != null){
+        final boolean showCoverage = PreferenceManager.getInstance().getAsBoolean(PreferenceManager.SASHIMI_SHOW_COVERAGE);
+
+        if(coverageTrack != null && showCoverage){
             //Only want the coverage track to go so high so that the arcs still have room
             int newHeight = coverageRectangle.height / 2;
             int newY = coverageRectangle.y + coverageRectangle.height / 2 - newHeight;
@@ -259,7 +268,7 @@ public class SashimiJunctionRenderer extends IGVFeatureRenderer {
                     int pixelYstart = 0;
                     int pixelYend = 0;
 
-                    if(coverageTrack != null){
+                    if(coverageTrack != null && showCoverage){
                         pixelYstart = getYOffset(coverageRectangle, coverageTrack.getDataRange() ,getCoverage(junctionStart));
                         pixelYend = getYOffset(coverageRectangle, coverageTrack.getDataRange() ,getCoverage(junctionEnd));
                     }
@@ -293,7 +302,7 @@ public class SashimiJunctionRenderer extends IGVFeatureRenderer {
 //        if(yOffset != null) return yOffset;
 
         if(dataManager == null) return 0;
-        Collection<AlignmentInterval> intervals = dataManager.getAllLoadedIntervals();
+        Collection<AlignmentInterval> intervals = dataManager.getLoadedIntervals();
         if(intervals == null) return 0;
 
         int buffer = 4;
@@ -412,15 +421,18 @@ public class SashimiJunctionRenderer extends IGVFeatureRenderer {
 //        strGraphics.drawString("" + depth, midX, arcPeakY);
 
         double actArcPeakY = arcBeginY + yPosModifier * Math.pow(0.5, 3) * (6) * arcHeight;
-
+        int maxPossibleArcHeight = (trackRectangle.height - 1) / 4;
+        float depthProportionOfMax = Math.min(1, depth / maxDepth);
+        float maxPossibleShapeHeight = maxPossibleArcHeight / 2;
         Shape shape = null;
+
         switch(shapeType){
-//            case CIRCLE:
-//                shape = createDepthCircle(maxPossibleShapeHeight, depthProportionOfMax, midX, actArcPeakY);
-//                break;
-//            case ELLIPSE:
-//                shape = createDepthEllipse(maxPossibleShapeHeight, depthProportionOfMax, midX, actArcPeakY);
-//                break;
+            case CIRCLE:
+                shape = createDepthCircle(maxPossibleShapeHeight, depthProportionOfMax, midX, actArcPeakY);
+                break;
+            case ELLIPSE:
+                shape = createDepthEllipse(maxPossibleShapeHeight, depthProportionOfMax, midX, actArcPeakY);
+                break;
             case TEXT:
                 String text = "" + depth;
                 Rectangle2D textBounds = g2D.getFontMetrics().getStringBounds(text, g2D);
