@@ -77,6 +77,7 @@ public class AlignmentRenderer implements FeatureRenderer {
     private final Color LL_COLOR = new Color(0, 150, 150);
 
     private final Color OUTLINE_COLOR = new Color(185, 185, 185);
+    private static final Color SUPPLEMENTARY_OUTLINE_COLOR = Color.blue;
 
     private static Map<String, AlignmentTrack.OrientationType> frOrientationTypes;
     private static Map<String, AlignmentTrack.OrientationType> f1f2OrientationTypes;
@@ -84,8 +85,11 @@ public class AlignmentRenderer implements FeatureRenderer {
     private static Map<String, AlignmentTrack.OrientationType> rfOrientationTypes;
     private Map<AlignmentTrack.OrientationType, Color> typeToColorMap;
 
+    public static HashMap<Character, Color> nucleotideColors;
+
     static {
         initializeTagTypes();
+        setNucleotideColors();
     }
 
     PreferenceManager prefs;
@@ -96,6 +100,31 @@ public class AlignmentRenderer implements FeatureRenderer {
     private TreeSet<Shape> arcsByEnd;
     private HashMap<Shape, Alignment> curveMap;
 
+    private static void setNucleotideColors() {
+
+        PreferenceManager prefs = PreferenceManager.getInstance();
+
+        nucleotideColors = new HashMap();
+
+        Color a = ColorUtilities.stringToColor(prefs.get(PreferenceManager.SAM_COLOR_A), Color.green);
+        Color c = ColorUtilities.stringToColor(prefs.get(PreferenceManager.SAM_COLOR_C), Color.blue);
+        Color t = ColorUtilities.stringToColor(prefs.get(PreferenceManager.SAM_COLOR_T), Color.red);
+        Color g = ColorUtilities.stringToColor(prefs.get(PreferenceManager.SAM_COLOR_G), Color.gray);
+        Color n = ColorUtilities.stringToColor(prefs.get(PreferenceManager.SAM_COLOR_N), Color.gray);
+
+        nucleotideColors.put('A', a);
+        nucleotideColors.put('a', a);
+        nucleotideColors.put('C', c);
+        nucleotideColors.put('c', c);
+        nucleotideColors.put('T', t);
+        nucleotideColors.put('t', t);
+        nucleotideColors.put('G', g);
+        nucleotideColors.put('g', g);
+        nucleotideColors.put('N', n);
+        nucleotideColors.put('n', n);
+
+    }
+
     public static AlignmentRenderer getInstance() {
         if (instance == null) {
             instance = new AlignmentRenderer();
@@ -238,6 +267,7 @@ public class AlignmentRenderer implements FeatureRenderer {
         double origin = context.getOrigin();
         double locScale = context.getScale();
         Font font = FontManager.getFont(10);
+        boolean completeReadsOnly = PreferenceManager.getInstance().getAsBoolean(PreferenceManager.SAM_COMPLETE_READS_ONLY);
 
         if ((alignments != null) && (alignments.size() > 0)) {
 
@@ -248,13 +278,17 @@ public class AlignmentRenderer implements FeatureRenderer {
                 double pixelStart = ((alignment.getStart() - origin) / locScale);
                 double pixelEnd = ((alignment.getEnd() - origin) / locScale);
 
-                // If the any part of the feature fits in the track rectangle draw  it
-                if (pixelEnd < rowRect.x) {
+                // If any any part of the feature fits in the track rectangle draw  it
+                if (pixelEnd < rowRect.x || pixelStart > rowRect.getMaxX()) {
                     continue;
-                } else if (pixelStart > rowRect.getMaxX()) {
-                    break;
                 }
 
+                // Optionally only draw alignments that are completely in view
+                if (completeReadsOnly) {
+                    if (pixelStart < rowRect.x || pixelEnd > rowRect.getMaxX()) {
+                        continue;
+                    }
+                }
 
                 // If the alignment is 3 pixels or less,  draw alignment as a single block,
                 // further detail would not be seen and just add to drawing overhead
@@ -266,7 +300,7 @@ public class AlignmentRenderer implements FeatureRenderer {
 
                     // Optimization for really zoomed out views.  If this alignment occupies screen space already taken,
                     // and it is the default color, skip drawing.
-                    if (pixelEnd <= lastPixelDrawn && alignmentColor == alignment.getDefaultColor()) {
+                    if (pixelEnd <= lastPixelDrawn && alignmentColor == grey1) {
                         continue;
                     }
 
@@ -292,7 +326,7 @@ public class AlignmentRenderer implements FeatureRenderer {
             // Optionally draw a border around the center base
             boolean showCenterLine = prefs.getAsBoolean(PreferenceManager.SAM_SHOW_CENTER_LINE);
             final int bottom = rowRect.y + rowRect.height;
-            if (locScale < 5 && showCenterLine) {
+            if (showCenterLine) {
                 // Calculate center lines
                 double center = (int) (context.getReferenceFrame().getCenter() - origin);
                 int centerLeftP = (int) (center / locScale);
@@ -596,6 +630,10 @@ public class AlignmentRenderer implements FeatureRenderer {
                     cRed.draw(blockShape);
                 }
 
+                if (alignment.isSupplementary()) {
+                    context.getGraphic2DForColor(SUPPLEMENTARY_OUTLINE_COLOR).draw(blockShape);
+                }
+
                 if (selectedReadNames.containsKey(alignment.getReadName())) {
                     Color c = selectedReadNames.get(alignment.getReadName());
                     if (c == null) {
@@ -623,7 +661,7 @@ public class AlignmentRenderer implements FeatureRenderer {
                 Stroke stroke;
                 int gapIdx = blockNumber - 1;
                 Color gapLineColor = deletionColor;
-                if (gapTypes != null && gapIdx < gapTypes.length && gapTypes[gapIdx] == SamAlignment.SKIPPED_REGION) {
+                if (gapTypes != null && gapIdx < gapTypes.length && gapTypes[gapIdx] == SAMAlignment.SKIPPED_REGION) {
                     gLine = context.getGraphic2DForColor(skippedColor);
                     stroke = gLine.getStroke();
                 } else {
@@ -690,7 +728,6 @@ public class AlignmentRenderer implements FeatureRenderer {
         boolean isSoftClipped = block.isSoftClipped();
 
 
-
         // Get the base qualities, start/end,  and reference sequence
         String chr = context.getChr();
         final int start = block.getStart();
@@ -713,9 +750,9 @@ public class AlignmentRenderer implements FeatureRenderer {
         }
 
         byte[] read;
-        if(haveBases){
+        if (haveBases) {
             read = block.getBases();
-        }else{
+        } else {
             read = reference;
         }
 
@@ -755,7 +792,7 @@ public class AlignmentRenderer implements FeatureRenderer {
                     (bisulfiteMode && (!DisplayStatus.NOTHING.equals(bisinfo.getDisplayStatus(idx))))) {
                 char c = (char) read[idx];
 
-                Color color = Globals.nucleotideColors.get(c);
+                Color color = nucleotideColors.get(c);
                 if (bisulfiteMode) color = bisinfo.getDisplayColor(idx);
                 if (color == null) {
                     color = Color.black;
@@ -965,8 +1002,11 @@ public class AlignmentRenderer implements FeatureRenderer {
         // center line.  Also restorePersistentState row "score" if alignment intersects center line
 
 
-        Color defaultColor = alignment.getDefaultColor();
-        Color c = defaultColor;
+        Color color = alignment.getColor();
+        if (color != null) return color;   // Color has been explicitly set
+
+        Color c = alignment.isSupplementary() ? Color.darkGray : grey1;
+
         ColorOption colorOption = renderOptions.getColorOption();
         switch (colorOption) {
             case BISULFITE:
@@ -987,7 +1027,7 @@ public class AlignmentRenderer implements FeatureRenderer {
             case UNEXPECTED_PAIR:
             case PAIR_ORIENTATION:
                 c = getOrientationColor(alignment, getPEStats(alignment, renderOptions));
-                if (colorOption == ColorOption.PAIR_ORIENTATION || c != defaultColor) {
+                if (colorOption == ColorOption.PAIR_ORIENTATION) {
                     break;
                 }
             case INSERT_SIZE:
@@ -1097,7 +1137,7 @@ public class AlignmentRenderer implements FeatureRenderer {
      * @param alignment
      * @param renderOptions
      * @return -1 if unknown (stats not computed), 0 if not
-     *         an outlier, 1 if outlier
+     * an outlier, 1 if outlier
      */
     private int getOutlierStatus(Alignment alignment, RenderOptions renderOptions) {
         PEStats peStats = getPEStats(alignment, renderOptions);
