@@ -33,7 +33,6 @@ import org.broad.igv.tdf.TDFDataSource;
 import org.broad.igv.tdf.TDFReader;
 import org.broad.igv.track.*;
 import org.broad.igv.ui.DataRangeDialog;
-import org.broad.igv.ui.FontManager;
 import org.broad.igv.ui.IGV;
 import org.broad.igv.ui.color.ColorUtilities;
 import org.broad.igv.ui.event.DataLoadedEvent;
@@ -54,7 +53,6 @@ import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
 import java.io.File;
 import java.util.ArrayList;
-import java.util.Arrays;
 import java.util.Collection;
 import java.util.List;
 
@@ -90,16 +88,19 @@ public class CoverageTrack extends AbstractTrack {
     JMenuItem autoscaleItem;
     Genome genome;
 
-    public void setRenderOptions(AlignmentTrack.RenderOptions renderOptions) {
-        this.renderOptions = renderOptions;
-    }
-
-    AlignmentTrack.RenderOptions getRenderOptions(){
-        return this.renderOptions;
-    }
+    /**
+     * Whether to autoscale across all ReferenceFrames
+     * Default is true because we usually do, SashimiPlot does not
+     */
+    private boolean globalAutoScale = true;
 
     private AlignmentTrack.RenderOptions renderOptions = null;
 
+    /**
+     * Copy constructor.  Used for Sashimi plot.
+     *
+     * @param track
+     */
     public CoverageTrack(CoverageTrack track){
         this(track.getResourceLocator(), track.getName(), track.genome);
         if(track.dataManager != null) this.setDataManager(track.dataManager);
@@ -136,14 +137,6 @@ public class CoverageTrack extends AbstractTrack {
 
     }
 
-
-    @Override
-    public void setDataRange(DataRange axisDefinition) {
-        // Explicitly setting a data range turns off auto-scale
-        autoScale = false;
-        super.setDataRange(axisDefinition);
-    }
-
     public void setSnpThreshold(float snpThreshold){
         this.snpThreshold = snpThreshold;
     }
@@ -156,104 +149,137 @@ public class CoverageTrack extends AbstractTrack {
         return showReference;
     }
 
+
+    public void setRenderOptions(AlignmentTrack.RenderOptions renderOptions) {
+        this.renderOptions = renderOptions;
+    }
+
+    AlignmentTrack.RenderOptions getRenderOptions(){
+        return this.renderOptions;
+    }
+
     /**
      * Rescale as necessary, and tell components to repaint
      * @param e
      */
     @Subscribe
     public void receiveDataLoaded(DataLoadedEvent e){
-        rescale(e.context.getReferenceFrame());
-        e.context.getReferenceFrame().getEventBus().post(new ViewChange.Result());
+        ReferenceFrame frame = e.context.getReferenceFrame();
+        rescale(frame);
+        frame.getEventBus().post(new ViewChange.Result());
     }
 
-    public void rescale() {
-        if (autoScale & dataManager != null) {
-            List<ReferenceFrame> frameList = FrameManager.getFrames();
-            List<AlignmentInterval> intervals = new ArrayList<AlignmentInterval>(frameList.size());
-            for(ReferenceFrame frame: frameList){
-                intervals.add(dataManager.getLoadedInterval(frame.getName()));
+    public void rescale(ReferenceFrame iframe) {
+        List<ReferenceFrame> frameList = new ArrayList<ReferenceFrame>();
+        if(iframe != null) frameList.add(iframe);
+        if(globalAutoScale){
+            frameList.addAll(FrameManager.getFrames());
+        }
+
+        if (autoScale && dataManager != null) {
+
+            int max = 10;
+            for (ReferenceFrame frame : frameList) {
+                AlignmentInterval interval = dataManager.getLoadedInterval(frame.getCurrentRange());
+                if (interval == null) continue;
+
+                int origin = (int) frame.getOrigin();
+                int end = (int) frame.getEnd() + 1;
+
+                int intervalMax = interval.getMaxCount(origin, end);
+                max = intervalMax > max ? intervalMax : max;
             }
-            rescaleIntervals(intervals);
+
+            DataRange newRange = new DataRange(0, max);
+            newRange.setType(getDataRange().getType());
+            super.setDataRange(newRange);
+
         }
     }
 
-    public void rescale(ReferenceFrame frame) {
-        if (autoScale & dataManager != null) {
-            rescaleIntervals(Arrays.asList(dataManager.getLoadedInterval(frame.getName())));
+    public void render(RenderContext context, Rectangle rect){
+
+        overlay(context, rect);
+
+        drawBorder(context, rect);
+        List<LocusScore> scores = getSummaryScores(context);
+        if (scores != null) {
+            dataSourceRenderer.renderBorder(this, context, rect);
         }
+
+        if(!isRepeatY(rect)){
+            lastRenderY = rect.y;
+            if(dataSourceRenderer != null){
+                dataSourceRenderer.renderAxis(this, context, rect);
+            }
+            if(FrameManager.isExomeMode()){
+                int x = context.getGraphics().getClipBounds().x;
+                Rectangle scaleRect = new Rectangle(x, rect.y, rect.width, rect.height);
+                drawScale(context, scaleRect);
+            }
+
+        }
+
     }
 
-    private void rescaleIntervals(List<AlignmentInterval> intervals) {
-        if (intervals == null || intervals.size() == 0) return;
-
-        int max = 10;
-        for(AlignmentInterval interval: intervals){
-            if(interval == null) continue;
-            max = Math.max(max, interval.getMaxCount());
+    private List<LocusScore> getSummaryScores(RenderContext context){
+        List<LocusScore> scores = null;
+        if(dataSource != null){
+            String chr = context.getChr();
+            int start = (int) context.getOrigin();
+            int end = (int) context.getEndLocation();
+            int zoom = context.getZoom();
+            scores = dataSource.getSummaryScoresForRange(chr, start, end, zoom);
         }
-        DataRange.Type type = getDataRange().getType();
-        super.setDataRange(new DataRange(0, 0, max));
-        getDataRange().setType(type);
+        return scores;
     }
 
-
-    public void render(RenderContext context, Rectangle rect) {
+    public void overlay(RenderContext context, Rectangle rect) {
 
         float maxRange = PreferenceManager.getInstance().getAsFloat(PreferenceManager.SAM_MAX_VISIBLE_RANGE);
         float minVisibleScale = (maxRange * 1000) / 700;
 
         if (context.getScale() < minVisibleScale && !context.getChr().equals(Globals.CHR_ALL)) {
-            //
+            //Show coverage calculated from intervals if zoomed in enough
             AlignmentInterval interval = null;
             if (dataManager != null) {
-                dataManager.preload(context, renderOptions, true);
-                interval = dataManager.getLoadedInterval(context.getReferenceFrame().getName());
+                dataManager.load(context, renderOptions, true);
+                interval = dataManager.getLoadedInterval(context.getReferenceFrame().getCurrentRange());
             }
             if (interval != null) {
                 if (interval.contains(context.getChr(), (int) context.getOrigin(), (int) context.getEndLocation())) {
+                    if(autoScale) rescale(context.getReferenceFrame());
                     intervalRenderer.paint(context, rect, interval.getCounts());
+                    return;
                 }
             }
-        } else if (dataSource != null) {
-            // Use precomputed data source, if any
-            String chr = context.getChr();
-            int start = (int) context.getOrigin();
-            int end = (int) context.getEndLocation();
-            int zoom = context.getZoom();
-            List<LocusScore> scores = dataSource.getSummaryScoresForRange(chr, start, end, zoom);
-            if (scores != null) {
-                dataSourceRenderer.render(scores, context, rect, this);
-            }
-
         }
-        drawBorder(context, rect);
+
+        //Use precomputed scores, if available
+        List<LocusScore> scores = getSummaryScores(context);
+        if (scores != null) {
+            dataSourceRenderer.renderScores(this, scores, context, rect);
+        }
+
     }
 
+    /**
+     * Draw border and scale
+     * @param context
+     * @param rect
+     */
     private void drawBorder(RenderContext context, Rectangle rect) {
-        // Draw border
         context.getGraphic2DForColor(Color.gray).drawLine(
                 rect.x, rect.y + rect.height,
                 rect.x + rect.width, rect.y + rect.height);
 
-        // Draw scale
-        if (!FrameManager.isExomeMode())
+        if(!FrameManager.isExomeMode()){
             drawScale(context, rect);
+        }
     }
 
     public void drawScale(RenderContext context, Rectangle rect) {
-        DataRange range = getDataRange();
-        if (range != null) {
-            Graphics2D g = context.getGraphic2DForColor(Color.black);
-            Font font = g.getFont();
-            Font smallFont = FontManager.getFont(8);
-            g.setFont(smallFont);
-            String scale = "[" + (int) range.getMinimum() + " - " +
-                    (int) range.getMaximum() + "]";
-
-            g.drawString(scale, rect.x + 5, rect.y + 10);
-
-            g.setFont(font);
-        }
+        DataRenderer.drawScale(getDataRange(), context, rect);
     }
 
     public boolean isLogNormalized() {
@@ -265,7 +291,7 @@ public class CoverageTrack extends AbstractTrack {
         float maxRange = PreferenceManager.getInstance().getAsFloat(PreferenceManager.SAM_MAX_VISIBLE_RANGE);
         float minVisibleScale = (maxRange * 1000) / 700;
         if (frame.getScale() < minVisibleScale) {
-            AlignmentInterval interval = dataManager.getLoadedInterval(frame.getName());
+            AlignmentInterval interval = dataManager.getLoadedInterval(frame.getCurrentRange());
             if (interval == null) return null;
 
             if (interval.contains(chr, (int) position, (int) position)) {
@@ -314,12 +340,11 @@ public class CoverageTrack extends AbstractTrack {
 
         private void paint(RenderContext context, Rectangle rect, AlignmentCounts alignmentCounts) {
 
-
             Color color = getColor();
             Graphics2D graphics = context.getGraphic2DForColor(color);
 
             DataRange range = getDataRange();
-            double maxRange = range.isLog() ? Math.log10(range.getMaximum()) : range.getMaximum();
+            double maxRange = range.isLog() ? Math.log10(range.getMaximum() + 1) : range.getMaximum();
 
             final double rectX = rect.getX();
             final double rectMaxX = rect.getMaxX();
@@ -354,7 +379,7 @@ public class CoverageTrack extends AbstractTrack {
                 if (pX + dX > lastpX) {
                     int pY = (int) rectMaxY - 1;
                     int totalCount = alignmentCounts.getTotalCount(pos);
-                    double tmp = range.isLog() ? Math.log10(totalCount) / maxRange : totalCount / maxRange;
+                    double tmp = range.isLog() ? Math.log10(totalCount + 1) / maxRange : totalCount / maxRange;
                     int height = (int) (tmp * rectHeight);
 
                     height = Math.min(height, rect.height - 1);
@@ -434,7 +459,7 @@ public class CoverageTrack extends AbstractTrack {
                     int pY = (int) rectMaxY - 1;
 
                     int totalCount = alignmentCounts.getTotalCount(pos);
-                    double tmp = range.isLog() ? Math.log10(totalCount) / maxRange : totalCount / maxRange;
+                    double tmp = range.isLog() ? Math.log10(totalCount + 1) / maxRange : totalCount / maxRange;
                     int height = (int) (tmp * rectHeight);
 
                     height = Math.min(height, rect.height - 1);
@@ -493,12 +518,12 @@ public class CoverageTrack extends AbstractTrack {
 
             int count = interval.getCount(pos, (byte) nucleotide);
 
-            Color c = Globals.nucleotideColors.get(nucleotide);
+            Color c = AlignmentRenderer.nucleotideColors.get(nucleotide);
 
             Graphics2D tGraphics = context.getGraphic2DForColor(c);
 
             double tmp = isLog ?
-                    (count / totalCount) * Math.log10(totalCount) / max :
+                    (count / totalCount) * Math.log10(totalCount + 1) / max :
                     count / max;
             int height = (int) (tmp * rect.getHeight());
 
@@ -547,7 +572,7 @@ public class CoverageTrack extends AbstractTrack {
         // unMethylated *= mult;
 
         double tmp = isLog ?
-                (nMethylated / totalCount) * Math.log10(totalCount) / maxRange :
+                (nMethylated / totalCount) * Math.log10(totalCount + 1) / maxRange :
                 nMethylated / maxRange;
         int height = (int) (tmp * rect.getHeight());
 
@@ -562,7 +587,7 @@ public class CoverageTrack extends AbstractTrack {
         tGraphics = context.getGraphic2DForColor(c);
 
         tmp = isLog ?
-                (unMethylated / totalCount) * Math.log10(totalCount) / maxRange :
+                (unMethylated / totalCount) * Math.log10(totalCount + 1) / maxRange :
                 unMethylated / maxRange;
         height = (int) (tmp * rect.getHeight());
 
@@ -603,7 +628,7 @@ public class CoverageTrack extends AbstractTrack {
 
         for (char nucleotide : nucleotides) {
 
-            Color c = Globals.nucleotideColors.get(nucleotide);
+            Color c = AlignmentRenderer.nucleotideColors.get(nucleotide);
             Graphics2D tGraphics = context.getGraphic2DForColor(c);
 
             int count = isPositive ? interval.getPosCount(pos, (byte) nucleotide) :
@@ -671,7 +696,7 @@ public class CoverageTrack extends AbstractTrack {
         popupMenu.add(TrackMenuUtils.getTrackRenameItem(tmp));
         addCopyDetailsItem(popupMenu, te);
 
-        addAutoscaleItem(popupMenu);
+        addAutoscaleItem(popupMenu, te.getFrame());
         addLogScaleItem(popupMenu);
         dataRangeItem = addDataRangeItem(IGV.getMainFrame(), popupMenu, tmp);
 
@@ -689,7 +714,7 @@ public class CoverageTrack extends AbstractTrack {
     private void addCopyDetailsItem(IGVPopupMenu popupMenu, TrackClickEvent te) {
         JMenuItem copyDetails = new JMenuItem("Copy Details to Clipboard");
         copyDetails.setEnabled(false);
-        if(te.getFrame() != null){
+        if (te.getFrame() != null) {
             final String details = getValueStringAt(te.getFrame().getChrName(), te.getChromosomePosition(), te.getMouseEvent().getY(), te.getFrame());
             copyDetails.addActionListener(new ActionListener() {
                 @Override
@@ -729,6 +754,7 @@ public class CoverageTrack extends AbstractTrack {
 
                         for (Track track : selectedTracks) {
                             track.setDataRange(dataRange);
+                            track.setAutoScale(false);
                         }
                         parentFrame.repaint();
                     }
@@ -736,7 +762,7 @@ public class CoverageTrack extends AbstractTrack {
 
             }
         });
-        if(menu != null) menu.add(maxValItem);
+        if (menu != null) menu.add(maxValItem);
 
         return maxValItem;
     }
@@ -800,7 +826,7 @@ public class CoverageTrack extends AbstractTrack {
 
     }
 
-    public void addAutoscaleItem(JPopupMenu menu) {
+    public void addAutoscaleItem(JPopupMenu menu,final ReferenceFrame frame) {
         // Change track height by attribute
         autoscaleItem = new JCheckBoxMenuItem("Autoscale");
         autoscaleItem.setSelected(autoScale);
@@ -811,7 +837,7 @@ public class CoverageTrack extends AbstractTrack {
                 autoScale = autoscaleItem.isSelected();
                 dataRangeItem.setEnabled(!autoScale);
                 if (autoScale) {
-                    rescale();
+                    rescale(frame);
                 }
                 IGV.getInstance().repaintDataPanels();
 
@@ -843,9 +869,12 @@ public class CoverageTrack extends AbstractTrack {
     }
 
     @SubtlyImportant
-    private static CoverageTrack getNextTrack(){
+    private static CoverageTrack getNextTrack() {
         return (CoverageTrack) IGVSessionReader.getNextTrack();
     }
 
+    public void setGlobalAutoScale(boolean globalAutoScale) {
+        this.globalAutoScale = globalAutoScale;
+    }
 
 }
