@@ -71,6 +71,11 @@ public class FeatureWrappedAlignment implements Alignment {
         return chr;
     }
 
+    @Override
+    public String getContig() {
+        return chr;
+    }
+
     public int getAlignmentStart() {
         return start;
     }
@@ -211,8 +216,8 @@ public class FeatureWrappedAlignment implements Alignment {
         return false;  //To change body of implemented methods use File | Settings | File Templates.
     }
 
-    public Color getDefaultColor() {
-        return AlignmentRenderer.grey1;
+    public Color getColor() {
+        return null;
     }
 
     public char[] getGapTypes() {
@@ -247,4 +252,9 @@ public class FeatureWrappedAlignment implements Alignment {
     public boolean isPrimary() {
         return true;
     }
+
+    @Override
+    public boolean isSupplementary() {
+        return false;
+    }
 }
