@@ -23,13 +23,13 @@
 package org.broad.igv.sam;
 
 import org.apache.log4j.Logger;
-import org.broad.igv.Globals;
 import org.broad.igv.PreferenceManager;
 import org.broad.igv.feature.SpliceJunctionFeature;
 import org.broad.igv.renderer.DataRange;
 import org.broad.igv.renderer.SpliceJunctionRenderer;
 import org.broad.igv.track.*;
 import org.broad.igv.ui.IGV;
+import org.broad.igv.ui.SashimiPlot;
 import org.broad.igv.ui.event.AlignmentTrackEvent;
 import org.broad.igv.ui.event.AlignmentTrackEventListener;
 import org.broad.igv.ui.panel.IGVPopupMenu;
@@ -40,7 +40,6 @@ import java.awt.*;
 import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
 import java.util.ArrayList;
-import java.util.Collection;
 import java.util.Collections;
 import java.util.List;
 
@@ -48,31 +47,32 @@ import java.util.List;
  * @author dhmay
  *         Finds splice junctions in real time and renders them as Features
  */
-public class SpliceJunctionFinderTrack extends FeatureTrack implements AlignmentTrackEventListener {
+public class SpliceJunctionFinderTrack extends FeatureTrack {
 
     private static Logger log = Logger.getLogger(SpliceJunctionFinderTrack.class);
 
+    public enum StrandOption {COMBINE, FORWARD, REVERSE, BOTH}
+
+    // Strand option is shared by all tracks
+    private static StrandOption strandOption;
 
     IAlignmentDataManager dataManager;
     PreferenceManager prefs;
-    RenderContext context;
 
     // The "parent" of the track (a DataPanel).  This release of IGV does not support owner-track relationships
     // directory,  so this field might be null at any given time.  It is updated each repaint.
     JComponent parent;
 
-    boolean ignoreStrand;
 
-    public SpliceJunctionFinderTrack(ResourceLocator locator, String name, IAlignmentDataManager dataManager, boolean ignoreStrand) {
+    public SpliceJunctionFinderTrack(ResourceLocator locator, String name, IAlignmentDataManager dataManager, StrandOption ignoreStrand) {
         super(locator, locator.getPath() + "_junctions", name);
 
         super.setDataRange(new DataRange(0, 0, 60));
         setRendererClass(SpliceJunctionRenderer.class);
         this.dataManager = dataManager;
         prefs = PreferenceManager.getInstance();
-        this.ignoreStrand = ignoreStrand;
+        this.strandOption = ignoreStrand;
         // Register track
-        IGV.getInstance().addAlignmentTrackEventListener(this);
     }
 
     @Override
@@ -83,6 +83,20 @@ public class SpliceJunctionFinderTrack extends FeatureTrack implements Alignment
     }
 
 
+    public static void setStrandOption(StrandOption so) {
+        strandOption = so;
+    }
+
+
+    public static StrandOption getStrandOption() {
+        return strandOption;
+    }
+
+    public void clear() {
+        this.packedFeaturesMap.clear();
+    }
+
+
     /**
      * Override to return a specialized popup menu
      *
@@ -109,16 +123,20 @@ public class SpliceJunctionFinderTrack extends FeatureTrack implements Alignment
         popupMenu.addSeparator();
         popupMenu.add(getChangeAutoScale());
 
+
+        popupMenu.addSeparator();
+        JMenuItem sashimi = new JMenuItem("Sashimi Plot");
+        sashimi.addActionListener(new ActionListener() {
+            @Override
+            public void actionPerformed(ActionEvent e) {
+                SashimiPlot.getSashimiPlot(null);
+            }
+        });
+        popupMenu.add(sashimi);
+
         return popupMenu;
     }
 
-    @Override
-    public void setDataRange(DataRange axisDefinition) {
-        // Explicitly setting a data range turns off auto-scale
-        super.setDataRange(axisDefinition);
-    }
-
-
     public boolean isLogNormalized() {
         return false;
     }
@@ -130,11 +148,11 @@ public class SpliceJunctionFinderTrack extends FeatureTrack implements Alignment
     @Override
     protected void loadFeatures(String chr, int start, int end, RenderContext context) {
         parent = context.getPanel();
-        AlignmentInterval loadedInterval = dataManager.getLoadedInterval(context.getReferenceFrame().getName());
+        AlignmentInterval loadedInterval = dataManager.getLoadedInterval(context.getReferenceFrame().getCurrentRange());
         if (loadedInterval == null) return;
 
         SpliceJunctionHelper helper = loadedInterval.getSpliceJunctionHelper();
-        List<SpliceJunctionFeature> features = ignoreStrand ? helper.getFilteredJunctionsIgnoreStrand() : helper.getFilteredJunctions();
+        List<SpliceJunctionFeature> features = helper.getFilteredJunctions(strandOption);
         if (features == null) {
             features = Collections.emptyList();
         }
@@ -154,15 +172,6 @@ public class SpliceJunctionFinderTrack extends FeatureTrack implements Alignment
         return result;
     }
 
-    public void onAlignmentTrackEvent(AlignmentTrackEvent e) {
-        AlignmentTrackEvent.Type type = e.getType();
-        switch (type) {
-            case SPLICE_JUNCTION:
-                packedFeaturesMap.clear();
-        }
-
-    }
-
     // Start of Roche-Tessella modification
     private JMenuItem getChangeAutoScale() {
 
