@@ -19,7 +19,7 @@ import org.broad.igv.ui.util.CancellableProgressDialog;
 import org.broad.igv.ui.util.IndefiniteProgressMonitor;
 import org.broad.igv.ui.util.ProgressMonitor;
 import org.broad.igv.util.LongRunningTask;
-import org.broad.tribble.Feature;
+import htsjdk.tribble.Feature;
 
 import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
@@ -33,6 +33,8 @@ import java.util.Iterator;
  */
 class FeatureTrackUtils {
 
+    private static volatile boolean isSearching;
+
     /**
      * Find the next/previous feature which lies outside chr:initStart-initEnd
      * @param source
@@ -120,6 +122,12 @@ class FeatureTrackUtils {
      */
     public static void nextFeatureSearch(FeatureSource source, String chr, int initStart, int initEnd, boolean forward,
                                          final FeatureSearcher.IFeatureFound foundHandler) throws IOException{
+
+        //Only allow one to be shown at a time
+        if(isSearching()){
+            return;
+        }
+
         Genome genome = GenomeManager.getInstance().getCurrentGenome();
 
         //We search backwards by setting a negative searchIncrement
@@ -134,23 +142,31 @@ class FeatureTrackUtils {
         final ActionListener cancelListener = new ActionListener() {
             @Override
             public void actionPerformed(ActionEvent e) {
+                FeatureTrackUtils.isSearching = false;
                 searcher.cancel();
             }
         };
 
-        final CancellableProgressDialog dialog = CancellableProgressDialog.showCancellableProgressDialog(IGV.getMainFrame(), "Searching...", cancelListener, monitor);
+        final CancellableProgressDialog dialog = CancellableProgressDialog.showCancellableProgressDialog(IGV.getMainFrame(), "Searching...", cancelListener, true, monitor);
         dialog.getProgressBar().setIndeterminate(true);
 
         monitor.addPropertyChangeListener(new PropertyChangeListener() {
             @Override
             public void propertyChange(PropertyChangeEvent evt) {
                 if(evt.getPropertyName().equals(ProgressMonitor.PROGRESS_PROPERTY) &&  (Integer) evt.getNewValue() >= 100){
+                    FeatureTrackUtils.isSearching = false;
                     Iterator<? extends Feature> result = searcher.getResult();
                     if(result != null) foundHandler.processResult(result);
                 }
             }
         });
 
-        LongRunningTask.getThreadExecutor().execute(searcher);
+        FeatureTrackUtils.isSearching = true;
+        LongRunningTask.submit(searcher);
     }
+
+    private static boolean isSearching() {
+        return isSearching;
+    }
+
 }
