@@ -15,7 +15,6 @@
 package org.broad.igv.track;
 
 
-import org.broad.igv.feature.genome.Genome;
 import org.broad.igv.renderer.ContinuousColorScale;
 import org.broad.igv.renderer.DataRange;
 import org.broad.igv.renderer.Renderer;
@@ -24,7 +23,7 @@ import org.broad.igv.ui.panel.IGVPopupMenu;
 import org.broad.igv.ui.panel.MouseableRegion;
 import org.broad.igv.ui.panel.ReferenceFrame;
 import org.broad.igv.util.ResourceLocator;
-import org.broad.tribble.Feature;
+import htsjdk.tribble.Feature;
 
 import java.awt.*;
 import java.awt.event.MouseEvent;
@@ -36,10 +35,8 @@ import java.util.List;
  */
 public interface Track extends Persistable{
 
-
-
     enum DisplayMode {
-        COLLAPSED, SQUISHED, EXPANDED, ALTERNATIVE_SPLICE
+        COLLAPSED, SQUISHED, EXPANDED
     }
 
     /**
@@ -52,7 +49,7 @@ public interface Track extends Persistable{
     String getId();
 
 
-    void preload(RenderContext context);
+    void load(RenderContext context);
 
     /**
      * Return true if a track can be filtered by sample annotation.
@@ -79,6 +76,12 @@ public interface Track extends Persistable{
      */
     void overlay(RenderContext context, Rectangle rect);
 
+    /**
+     * We store the y-coordinate at which this track was last rendered,
+     * to avoid repeating borders/scales/etc. This resets that value
+     */
+    void resetLastY();
+
     /**
      * Render the name of the track. Both the track and visible rectangles are supplied so the implementor
      * can adjust the placing of the name based on the current viewport.  This is used to center track names
@@ -106,6 +109,13 @@ public interface Track extends Persistable{
 
     ResourceLocator getResourceLocator();
 
+    /**
+     * Return ALL ResourceLocators associated with this track.
+     * For most tracks, this will just be a collection of size 1
+     * @return
+     */
+    Collection<ResourceLocator> getResourceLocators();
+
     void setAttributeValue(String key, String value);
 
     String getAttributeValue(String attributeKey);
@@ -134,6 +144,11 @@ public interface Track extends Persistable{
 
     int getMinimumHeight();
 
+    /**
+     * Manually specify the data range.
+     * {@code autoScale} must be turned off elsewhere, if applicable
+     * @param axisDefinition
+     */
     void setDataRange(DataRange axisDefinition);
 
     boolean hasDataRange();
@@ -216,4 +231,6 @@ public interface Track extends Persistable{
     
     boolean getAutoScale();
 
+    void setAutoScale(boolean autoScale);
+
 }
