@@ -11,33 +11,50 @@
 
 package org.broad.igv.track;
 
+import com.google.common.base.Predicate;
 import com.google.common.collect.Iterables;
 import com.google.common.collect.Lists;
 import org.apache.commons.math.stat.StatUtils;
 import org.apache.log4j.Logger;
 import org.broad.igv.Globals;
 import org.broad.igv.PreferenceManager;
-import org.broad.igv.blat.BlatClient;
 import org.broad.igv.data.CombinedDataSource;
 import org.broad.igv.feature.Exon;
+import org.broad.igv.feature.FeatureUtils;
 import org.broad.igv.feature.IGVFeature;
+import org.broad.igv.feature.Range;
 import org.broad.igv.feature.genome.Genome;
 import org.broad.igv.feature.genome.GenomeManager;
+import org.broad.igv.feature.tribble.IGVBEDCodec;
 import org.broad.igv.renderer.*;
+import org.broad.igv.sam.AlignmentDataManager;
+import org.broad.igv.sam.AlignmentTrack;
+import org.broad.igv.sam.SAMWriter;
 import org.broad.igv.ui.*;
+import org.broad.igv.ui.color.ColorUtilities;
 import org.broad.igv.ui.panel.FrameManager;
 import org.broad.igv.ui.panel.IGVPopupMenu;
 import org.broad.igv.ui.panel.ReferenceFrame;
+import org.broad.igv.ui.panel.TrackPanel;
+import org.broad.igv.ui.util.FileDialogUtils;
 import org.broad.igv.ui.util.MessageUtils;
 import org.broad.igv.ui.util.UIUtilities;
+import org.broad.igv.util.LongRunningTask;
+import org.broad.igv.util.ResourceLocator;
 import org.broad.igv.util.StringUtils;
+import org.broad.igv.util.blat.BlatClient;
+import org.broad.igv.util.collections.CollUtils;
 import org.broad.igv.util.stats.KMPlotFrame;
-import org.broad.tribble.Feature;
+import htsjdk.tribble.Feature;
 
 import javax.swing.*;
 import java.awt.*;
 import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
+import java.io.File;
+import java.io.FileNotFoundException;
+import java.io.IOException;
+import java.io.PrintWriter;
 import java.util.*;
 import java.util.List;
 
@@ -65,10 +82,11 @@ public class TrackMenuUtils {
     /**
      * Called by plugins to add a listener, which is then called when TrackMenus are created
      * to generate menu entries.
+     *
      * @param builder
      * @api
      */
-    public static void addTrackMenuItemBuilder(TrackMenuItemBuilder builder){
+    public static void addTrackMenuItemBuilder(TrackMenuItemBuilder builder) {
         trackMenuItems.add(builder);
     }
 
@@ -102,22 +120,23 @@ public class TrackMenuUtils {
 
     /**
      * Add menu items which have been added through the api, not known until runtime
+     *
      * @param menu
      * @param tracks
      * @param te
      */
     public static void addPluginItems(JPopupMenu menu, Collection<Track> tracks, TrackClickEvent te) {
         List<JMenuItem> items = new ArrayList<JMenuItem>(0);
-        for(TrackMenuItemBuilder builder: trackMenuItems){
+        for (TrackMenuItemBuilder builder : trackMenuItems) {
             JMenuItem item = builder.build(tracks, te);
-            if(item != null){
+            if (item != null) {
                 items.add(item);
             }
         }
 
-        if(items.size() > 0){
+        if (items.size() > 0) {
             menu.addSeparator();
-            for(JMenuItem item: items){
+            for (JMenuItem item : items) {
                 menu.add(item);
             }
         }
@@ -202,8 +221,8 @@ public class TrackMenuUtils {
      */
     public static void addDataItems(JPopupMenu menu, final Collection<Track> tracks) {
 
-        if (log.isDebugEnabled()) {
-            log.debug("enter getDataPopupMenu");
+        if (log.isTraceEnabled()) {
+            log.trace("enter getDataPopupMenu");
         }
 
         final String[] labels = {"Heatmap", "Bar Chart", "Points", "Line Plot"};
@@ -294,17 +313,61 @@ public class TrackMenuUtils {
 
         menu.add(getAutoscaleItem(tracks));
 
-
         menu.add(getShowDataRangeItem(tracks));
 
+        //Add overlay track option
+        menu.addSeparator();
+        final List<DataTrack> dataTrackList = Lists.newArrayList(Iterables.filter(tracks, DataTrack.class));
+        final JMenuItem overlayGroups = new JMenuItem("Create Overlay Track");
+        overlayGroups.addActionListener(new ActionListener() {
+            @Override
+            public void actionPerformed(ActionEvent e) {
+                MergedTracks mergedTracks = new MergedTracks(UUID.randomUUID().toString(), "Overlay", dataTrackList);
+                Track firstTrack = tracks.iterator().next();
+                TrackPanel panel = TrackPanel.getParentPanel(firstTrack);
+                panel.addTrack(mergedTracks);
+                panel.moveSelectedTracksTo(Arrays.asList(mergedTracks), firstTrack, false);
+                panel.removeTracks(tracks);
+            }
+        });
+
+        int numDataTracks = dataTrackList.size();
+        overlayGroups.setEnabled(numDataTracks >= 2 && numDataTracks == tracks.size());
+        menu.add(overlayGroups);
+
+        // Enable "separateTracks" menu if selection is a single track, and that track is merged.
+
+        JMenuItem unmergeItem = new JMenuItem("Separate Tracks");
+        menu.add(unmergeItem);
+
+        Track firstTrack = tracks.iterator().next();
+        if (tracks.size() == 1 && firstTrack instanceof MergedTracks) {
+
+            unmergeItem.addActionListener(new ActionListener() {
+                @Override
+                public void actionPerformed(ActionEvent e) {
+                    Track firstTrack = tracks.iterator().next();
+                    TrackPanel panel = TrackPanel.getParentPanel(firstTrack);
+                    final MergedTracks mergedTracks = (MergedTracks) firstTrack;
+                    mergedTracks.setTrackAlphas(255);
+                    panel.addTracks(mergedTracks.getMemberTracks());
+                    panel.moveSelectedTracksTo(mergedTracks.getMemberTracks(), mergedTracks, true);
+                    IGV.getInstance().removeTracks(Arrays.asList(mergedTracks));
+                }
+            });
+        } else {
+            unmergeItem.setEnabled(false);
+        }
+
+
         menu.addSeparator();
         menu.add(getChangeKMPlotItem(tracks));
 
-//        if(!Globals.isProduction()){
-//            for(JMenuItem item: getCombinedDataSourceItems(tracks)){
-//                menu.add(item);
-//            }
-//        }
+        if (Globals.isDevelopment() && FrameManager.isGeneListMode() && tracks.size() == 1) {
+            menu.addSeparator();
+            menu.add(getShowSortFramesItem(tracks.iterator().next()));
+        }
+
 
     }
 
@@ -336,9 +399,9 @@ public class TrackMenuUtils {
         return Arrays.asList(addItem, subItem);
     }
 
-    private static void addCombinedDataTrack(List<DataTrack> dataTracks, CombinedDataSource.Operation op){
+    private static void addCombinedDataTrack(List<DataTrack> dataTracks, CombinedDataSource.Operation op) {
         String text = "";
-        switch (op){
+        switch (op) {
             case ADD:
                 text = "Sum";
                 break;
@@ -391,15 +454,152 @@ public class TrackMenuUtils {
 
                 featurePopupMenu.add(getCopyDetailsItem(f, te));
                 featurePopupMenu.add(getCopySequenceItem(f));
+                featurePopupMenu.add(getBlatItem(f));
+
 
-                if (!Globals.isProduction()) {
-                    featurePopupMenu.add(getBlatItem(f));
-                }
             }
+            if (Globals.isDevelopment()) {
+                featurePopupMenu.addSeparator();
+                featurePopupMenu.add(getFeatureToGeneListItem(t));
+            }
+            if (Globals.isDevelopment() && FrameManager.isGeneListMode() && tracks.size() == 1) {
+                featurePopupMenu.addSeparator();
+                featurePopupMenu.add(getShowSortFramesItem(tracks.iterator().next()));
+            }
+
         }
 
         featurePopupMenu.addSeparator();
         featurePopupMenu.add(getChangeFeatureWindow(tracks));
+
+    }
+
+    private static JMenuItem getFeatureToGeneListItem(final Track t) {
+        JMenuItem mi = new JMenuItem("Use as loci list");
+
+        mi.addActionListener(new ActionListener() {
+            @Override
+            public void actionPerformed(ActionEvent e) {
+                // Current chromosome only for now
+
+
+            }
+        });
+
+        return mi;
+    }
+
+    /**
+     * Return a menu item which will export visible features
+     * If {@code tracks} is not a single {@code FeatureTrack}, {@code null}
+     * is returned (there should be no menu entry)
+     *
+     * @param tracks
+     * @return
+     */
+    public static JMenuItem getExportFeatures(final Collection<Track> tracks, final ReferenceFrame frame) {
+        Track ft = tracks.iterator().next();
+        if (tracks.size() != 1) {
+            return null;
+        }
+        JMenuItem exportData = null;
+
+        if (ft instanceof FeatureTrack) {
+            exportData = new JMenuItem("Export Features...");
+            exportData.addActionListener(new ActionListener() {
+                @Override
+                public void actionPerformed(ActionEvent e) {
+                    File outFile = FileDialogUtils.chooseFile("Save Visible Data",
+                            PreferenceManager.getInstance().getLastTrackDirectory(),
+                            new File("visibleData.bed"),
+                            FileDialogUtils.SAVE);
+
+                    exportVisibleFeatures(outFile.getAbsolutePath(), tracks, frame.getCurrentRange());
+                }
+            });
+        } else if (ft instanceof AlignmentTrack) {
+            exportData = new JMenuItem("Export Alignments...");
+            exportData.addActionListener(new ActionListener() {
+                @Override
+                public void actionPerformed(ActionEvent e) {
+                    File outFile = FileDialogUtils.chooseFile("Save Visible Data",
+                            PreferenceManager.getInstance().getLastTrackDirectory(),
+                            new File("visibleData.sam"),
+                            FileDialogUtils.SAVE);
+
+                    int countExp = exportVisibleAlignments(outFile.getAbsolutePath(), tracks, frame);
+                    String msg = String.format("%d reads written", countExp);
+                    MessageUtils.setStatusBarMessage(msg);
+                }
+            });
+        }
+
+        return exportData;
+    }
+
+    static int exportVisibleAlignments(String outPath, Collection<Track> tracks, ReferenceFrame frame) {
+        AlignmentTrack alignmentTrack = null;
+        for (Track track : tracks) {
+            if (track instanceof AlignmentTrack) {
+                alignmentTrack = (AlignmentTrack) track;
+                break;
+            }
+        }
+
+        if (alignmentTrack == null) return -1;
+
+
+        File outFile = new File(outPath);
+        try {
+            AlignmentDataManager dataManager = alignmentTrack.getDataManager();
+            ResourceLocator inlocator = dataManager.getLocator();
+            Range range = frame.getCurrentRange();
+
+            //Read directly from file
+            //return SAMWriter.writeAlignmentFilePicard(inlocator, outPath, range.getChr(), range.getStart(), range.getEnd());
+
+            //Export those in memory, overlapping current view
+            return SAMWriter.writeAlignmentFilePicard(dataManager, outFile, range.getChr(), range.getStart(), range.getEnd());
+        } catch (IOException e) {
+            log.error(e.getMessage(), e);
+            throw new RuntimeException(e);
+        }
+
+    }
+
+    /**
+     * Write features in {@code track} found in {@code range} to {@code outPath},
+     * BED format
+     * TODO Move somewhere else? run on separate thread?  Probably shouldn't be here
+     *
+     * @param outPath
+     * @param tracks
+     * @param range
+     */
+    static void exportVisibleFeatures(String outPath, Collection<Track> tracks, Range range) {
+        PrintWriter writer;
+        try {
+            writer = new PrintWriter(outPath);
+        } catch (FileNotFoundException e) {
+            throw new RuntimeException(e);
+        }
+
+        for (Track track : tracks) {
+            if (track instanceof FeatureTrack) {
+                FeatureTrack fTrack = (FeatureTrack) track;
+                //Can't trust FeatureTrack.getFeatures to limit itself, so we filter
+                List<Feature> features = fTrack.getFeatures(range.getChr(), range.getStart(), range.getEnd());
+                Predicate<Feature> pred = FeatureUtils.getOverlapPredicate(range.getChr(), range.getStart(), range.getEnd());
+                features = CollUtils.filter(features, pred);
+                IGVBEDCodec codec = new IGVBEDCodec();
+                for (Feature feat : features) {
+                    String featString = codec.encode(feat);
+                    writer.println(featString);
+                }
+            }
+        }
+        writer.flush();
+        writer.close();
     }
 
 
@@ -510,44 +710,23 @@ public class TrackMenuUtils {
 
             public void actionPerformed(ActionEvent evt) {
                 if (selectedTracks.size() > 0) {
-                    // Create a datarange that spans the extent of prev tracks range
-                    float mid = 0;
-                    float min = Float.MAX_VALUE;
-                    float max = Float.MIN_VALUE;
-                    boolean drawBaseline = true;
-                    boolean isLog = true;
-                    for (Track t : selectedTracks) {
-                        DataRange dr = t.getDataRange();
-                        min = Math.min(min, dr.getMinimum());
-                        max = Math.max(max, dr.getMaximum());
-                        mid += dr.getBaseline();
-                        drawBaseline &= dr.isDrawBaseline();
-                        isLog &= dr.isLog();
-                    }
-                    mid /= selectedTracks.size();
-                    if (mid < min) {
-                        mid = min;
-                    } else if (mid > max) {
-                        min = max;
-                    }
 
-                    DataRange prevAxisDefinition = new DataRange(min, mid, max, drawBaseline, isLog);
+                    // Create a datarange that spans the extent of prev tracks range
+                    DataRange prevAxisDefinition = DataRange.getFromTracks(selectedTracks);
                     DataRangeDialog dlg = new DataRangeDialog(IGV.getMainFrame(), prevAxisDefinition);
                     dlg.setVisible(true);
                     if (!dlg.isCanceled()) {
-                        min = Math.min(dlg.getMax(), dlg.getMin());
-                        max = Math.max(dlg.getMin(), dlg.getMax());
-                        mid = dlg.getBase();
+                        float min = Math.min(dlg.getMax(), dlg.getMin());
+                        float max = Math.max(dlg.getMin(), dlg.getMax());
+                        float mid = dlg.getBase();
                         mid = Math.max(min, Math.min(mid, max));
 
                         DataRange axisDefinition = new DataRange(dlg.getMin(), mid, dlg.getMax(),
-                                drawBaseline, dlg.isLog());
+                                prevAxisDefinition.isDrawBaseline(), dlg.isLog());
 
                         for (Track track : selectedTracks) {
                             track.setDataRange(axisDefinition);
-                            if (track instanceof DataTrack) {
-                                ((DataTrack) track).setAutoScale(false);
-                            }
+                            track.setAutoScale(false);
                         }
                         IGV.getInstance().repaint();
                     }
@@ -616,9 +795,7 @@ public class TrackMenuUtils {
 
                     boolean autoScale = autoscaleItem.isSelected();
                     for (Track t : selectedTracks) {
-                        if (t instanceof DataTrack) {
-                            ((DataTrack) t).setAutoScale(autoScale);
-                        }
+                        t.setAutoScale(autoScale);
                     }
                     IGV.getInstance().repaintDataPanels();
                 }
@@ -630,7 +807,7 @@ public class TrackMenuUtils {
     private static boolean checkAutoscale(Collection<Track> selectedTracks) {
         boolean autoScale = false;
         for (Track t : selectedTracks) {
-            if (t instanceof DataTrack && ((DataTrack) t).isAutoScale()) {
+            if (t.getAutoScale()) {
                 autoScale = true;
                 break;
             }
@@ -700,10 +877,7 @@ public class TrackMenuUtils {
         modes.put("Collapsed", Track.DisplayMode.COLLAPSED);
         modes.put("Expanded", Track.DisplayMode.EXPANDED);
         modes.put("Squished", Track.DisplayMode.SQUISHED);
-        boolean showAS = Boolean.parseBoolean(System.getProperty("showAS", "false"));
-        if (showAS) {
-            modes.put("Alternative Splice", Track.DisplayMode.ALTERNATIVE_SPLICE);
-        }
+
         for (final Map.Entry<String, Track.DisplayMode> entry : modes.entrySet()) {
             JRadioButtonMenuItem mm = new JRadioButtonMenuItem(entry.getKey());
             mm.setSelected(currentMode == entry.getValue());
@@ -736,41 +910,51 @@ public class TrackMenuUtils {
         item.addActionListener(new ActionListener() {
 
             public void actionPerformed(ActionEvent evt) {
-                if (selectedTracks.isEmpty()) {
-                    return;
-                }
-
-                StringBuffer buffer = new StringBuffer();
-                for (Track track : selectedTracks) {
-                    buffer.append("\n\t");
-                    buffer.append(track.getName());
-                }
-                String deleteItems = buffer.toString();
-
-                JTextArea textArea = new JTextArea();
-                textArea.setEditable(false);
-                JScrollPane scrollPane = new JScrollPane(textArea);
-                textArea.setText(deleteItems);
-
-                JOptionPane optionPane = new JOptionPane(scrollPane,
-                        JOptionPane.PLAIN_MESSAGE,
-                        JOptionPane.YES_NO_OPTION);
-                optionPane.setPreferredSize(new Dimension(550, 500));
-                JDialog dialog = optionPane.createDialog(IGV.getMainFrame(), "Remove The Following Tracks");
-                dialog.setVisible(true);
-
-                Object choice = optionPane.getValue();
-                if ((choice == null) || (JOptionPane.YES_OPTION != ((Integer) choice).intValue())) {
-                    return;
-                }
-
-                IGV.getInstance().removeTracks(selectedTracks);
-                IGV.getInstance().doRefresh();
+                removeTracksAction(selectedTracks);
             }
         });
         return item;
     }
 
+    /**
+     * Display a dialog to the user asking to confirm if they want to remove the
+     * selected tracks
+     *
+     * @param selectedTracks
+     */
+    public static void removeTracksAction(final Collection<Track> selectedTracks) {
+        if (selectedTracks.isEmpty()) {
+            return;
+        }
+
+        StringBuffer buffer = new StringBuffer();
+        for (Track track : selectedTracks) {
+            buffer.append("\n\t");
+            buffer.append(track.getName());
+        }
+        String deleteItems = buffer.toString();
+
+        JTextArea textArea = new JTextArea();
+        textArea.setEditable(false);
+        JScrollPane scrollPane = new JScrollPane(textArea);
+        textArea.setText(deleteItems);
+
+        JOptionPane optionPane = new JOptionPane(scrollPane,
+                JOptionPane.PLAIN_MESSAGE,
+                JOptionPane.YES_NO_OPTION);
+        optionPane.setPreferredSize(new Dimension(550, 500));
+        JDialog dialog = optionPane.createDialog(IGV.getMainFrame(), "Remove The Following Tracks");
+        dialog.setVisible(true);
+
+        Object choice = optionPane.getValue();
+        if ((choice == null) || (JOptionPane.YES_OPTION != ((Integer) choice).intValue())) {
+            return;
+        }
+
+        IGV.getInstance().removeTracks(selectedTracks);
+        IGV.getInstance().doRefresh();
+    }
+
 
     public static void changeRenderer(final Collection<Track> selectedTracks, Class rendererClass) {
         for (Track track : selectedTracks) {
@@ -804,26 +988,20 @@ public class TrackMenuUtils {
     }
 
     public static void changeTrackHeight(final Collection<Track> selectedTracks) {
-
-
         if (selectedTracks.isEmpty()) {
             return;
         }
 
         final String parameter = "Track height";
-        int value = getIntValue(parameter, getRepresentativeTrackHeight(selectedTracks));
-        if (value == Integer.MIN_VALUE) {
+        Integer value = getIntegerInput(parameter, getRepresentativeTrackHeight(selectedTracks));
+        if (value == null) {
             return;
         }
 
         value = Math.max(0, value);
-
         for (Track track : selectedTracks) {
-
             track.setHeight(value, true);
-
         }
-
         refresh();
     }
 
@@ -843,8 +1021,8 @@ public class TrackMenuUtils {
 
         int origValue = featureTracks.iterator().next().getVisibilityWindow();
         double origValueKB = (origValue / 1000.0);
-        double value = getNumericValue("Visibility window (kb)", origValueKB);
-        if (value == Integer.MIN_VALUE) {
+        Double value = getDoubleInput("Enter visibility window in kilo-bases.  To load all data enter zero.", origValueKB);
+        if (value == null) {
             return;
         }
 
@@ -864,8 +1042,8 @@ public class TrackMenuUtils {
 
         final String parameter = "Font size";
         int defaultValue = selectedTracks.iterator().next().getFontSize();
-        int value = getIntValue(parameter, defaultValue);
-        if (value == Integer.MIN_VALUE) {
+        Integer value = getIntegerInput(parameter, defaultValue);
+        if (value == null) {
             return;
         }
 
@@ -877,20 +1055,21 @@ public class TrackMenuUtils {
     }
 
 
-    public static int getIntValue(String parameter, int value) {
+    public static Integer getIntegerInput(String parameter, int value) {
 
         while (true) {
 
-            String height = JOptionPane.showInputDialog(
+            String strValue = JOptionPane.showInputDialog(
                     IGV.getMainFrame(), parameter + ": ",
                     String.valueOf(value));
 
-            if ((height == null) || height.trim().equals("")) {
-                return Integer.MIN_VALUE;   // <= the logical "null" value
+            //strValue will be null if dialog cancelled
+            if ((strValue == null) || strValue.trim().equals("")) {
+                return null;
             }
 
             try {
-                value = Integer.parseInt(height);
+                value = Integer.parseInt(strValue);
                 return value;
             } catch (NumberFormatException numberFormatException) {
                 JOptionPane.showMessageDialog(IGV.getMainFrame(),
@@ -899,20 +1078,21 @@ public class TrackMenuUtils {
         }
     }
 
-    public static double getNumericValue(String parameter, double value) {
+    public static Double getDoubleInput(String parameter, double value) {
 
         while (true) {
 
-            String height = JOptionPane.showInputDialog(
+            String strValue = JOptionPane.showInputDialog(
                     IGV.getMainFrame(), parameter + ": ",
                     String.valueOf(value));
 
-            if ((height == null) || height.trim().equals("")) {
-                return Double.MIN_VALUE;   // <= the logical "null" value
+            //strValue will be null if dialog cancelled
+            if ((strValue == null) || strValue.trim().equals("")) {
+                return null;
             }
 
             try {
-                value = Double.parseDouble(height);
+                value = Double.parseDouble(strValue);
                 return value;
             } catch (NumberFormatException numberFormatException) {
                 MessageUtils.showMessage(parameter + " must be a number.");
@@ -932,13 +1112,16 @@ public class TrackMenuUtils {
                 "Select Track Color (Positive Values)",
                 currentSelection);
 
-        if (color != null) {
-            for (Track track : selectedTracks) {
-                track.setColor(color);
-            }
-            refresh();
+        if (color == null) {
+            return;
         }
 
+        for (Track track : selectedTracks) {
+            //We preserve the alpha value. This is motivated by MergedTracks
+            track.setColor(ColorUtilities.modifyAlpha(color, currentSelection.getAlpha()));
+        }
+        refresh();
+
     }
 
     public static void changeAltTrackColor(final Collection<Track> selectedTracks) {
@@ -958,8 +1141,7 @@ public class TrackMenuUtils {
         }
 
         for (Track track : selectedTracks) {
-
-            track.setAltColor(color);
+            track.setAltColor(ColorUtilities.modifyAlpha(color, currentSelection.getAlpha()));
         }
         refresh();
 
@@ -1105,5 +1287,29 @@ public class TrackMenuUtils {
         return item;
     }
 
+
+    // Experimental methods follow
+
+    public static JMenuItem getShowSortFramesItem(final Track track) {
+
+        final JCheckBoxMenuItem item = new JCheckBoxMenuItem("Sort frames");
+
+        item.addActionListener(new ActionListener() {
+
+            @Override
+            public void actionPerformed(ActionEvent e) {
+                Runnable runnable = new Runnable() {
+                    public void run() {
+                        FrameManager.sortFrames(track);
+                        IGV.getInstance().resetFrames();
+                    }
+                };
+                LongRunningTask.submit(runnable);
+            }
+
+        });
+        return item;
+    }
+
 }
 
