@@ -24,7 +24,7 @@ import com.google.common.collect.Iterables;
 import com.google.common.collect.Lists;
 import com.google.common.eventbus.Subscribe;
 import com.jidesoft.swing.JideSplitPane;
-import net.sf.samtools.seekablestream.SeekableFileStream;
+import htsjdk.samtools.seekablestream.SeekableFileStream;
 import org.apache.log4j.Logger;
 import org.broad.igv.DirectoryManager;
 import org.broad.igv.Globals;
@@ -33,6 +33,8 @@ import org.broad.igv.annotations.ForTesting;
 import org.broad.igv.batch.BatchRunner;
 import org.broad.igv.batch.CommandListener;
 import org.broad.igv.dev.api.IGVPlugin;
+import org.broad.igv.exceptions.DataLoadException;
+import org.broad.igv.feature.Locus;
 import org.broad.igv.feature.MaximumContigGenomeException;
 import org.broad.igv.feature.RegionOfInterest;
 import org.broad.igv.feature.genome.*;
@@ -52,7 +54,6 @@ import org.broad.igv.ui.panel.*;
 import org.broad.igv.ui.util.*;
 import org.broad.igv.ui.util.ProgressMonitor;
 import org.broad.igv.util.*;
-import org.broad.igv.util.collections.LRUCache;
 import org.broad.igv.variant.VariantTrack;
 
 import javax.swing.*;
@@ -130,6 +131,7 @@ public class IGV {
             Collections.synchronizedCollection(new ArrayList<SoftReference<AlignmentTrackEventListener>>());
 
     private List<JComponent> otherToolMenus = new ArrayList<JComponent>();
+    private boolean rulerEnabled;
 
     /**
      * Add an entry to the "Tools" menu
@@ -282,7 +284,7 @@ public class IGV {
             @Override
             public void mouseClicked(MouseEvent e) {
                 super.mouseClicked(e);
-                if (!Globals.isProduction()) {
+                if (Globals.isDevelopment()) {
                     Component cButton = contentPane.getStatusBar().getCancelButton();
                     int tX = -(contentPane.getStatusBar().getX() + cButton.getX());
                     int tY = -(contentPane.getY() + contentPane.getStatusBar().getY());
@@ -315,8 +317,6 @@ public class IGV {
 
         mainFrame.setExtendedState(state);
         mainFrame.setBounds(applicationBounds);
-
-        BAMHttpReader.cleanTempDir(BAMHttpReader.oneDay * 5);
     }
 
 
@@ -378,10 +378,10 @@ public class IGV {
         contentPane.chromosomeChanged(chrName);
         repaintDataAndHeaderPanels(updateCommandBar);
         contentPane.getCommandBar().updateComponentStates();
-
     }
 
     public void repaintStatusAndZoomSlider() {
+        contentPane.getCommandBar().updateComponentStates();
         contentPane.getCommandBar().repaint();
     }
 
@@ -485,14 +485,14 @@ public class IGV {
             }
 
             String cytobandFileName = genomeBuilderDialog.getCytobandFileName();
-            String refFlatFileName = genomeBuilderDialog.getRefFlatFileName();
+            String geneAnnotFileName = genomeBuilderDialog.getGeneAnnotFileName();
             String fastaFileName = genomeBuilderDialog.getFastaFileName();
             String chrAliasFile = genomeBuilderDialog.getChrAliasFileName();
             String genomeDisplayName = genomeBuilderDialog.getGenomeDisplayName();
             String genomeId = genomeBuilderDialog.getGenomeId();
 
             GenomeListItem genomeListItem = getGenomeManager().defineGenome(
-                    genomeZipFile, cytobandFileName, refFlatFileName,
+                    genomeZipFile, cytobandFileName, geneAnnotFileName,
                     fastaFileName, chrAliasFile, genomeDisplayName,
                     genomeId, monitor);
 
@@ -552,24 +552,39 @@ public class IGV {
         Runnable showDialog = new Runnable() {
             @Override
             public void run() {
-                if (GenomeManager.getInstance().getServerGenomeArchiveList() == null) {
-                    waitForNotify(10000);
-                }
-
                 Collection<GenomeListItem> inputListItems = GenomeManager.getInstance().getServerGenomeArchiveList();
                 if (inputListItems == null) {
-                    IOException exc = new IOException("Unable to reach genome server");
-                    MessageUtils.showErrorMessage(exc.getMessage(), exc);
+                    //Not necessary to display a message, getServerGenomeArchiveList does it already
+                    //IOException exc = new IOException("Unable to reach genome server");
+                    //MessageUtils.showErrorMessage(exc.getMessage(), exc);
                     return;
                 }
 
                 GenomeSelectionDialog dialog = new GenomeSelectionDialog(IGV.getMainFrame(), inputListItems, ListSelectionModel.SINGLE_SELECTION);
                 dialog.setVisible(true);
                 List<GenomeListItem> selectedValues = dialog.getSelectedValuesList();
+
                 if (selectedValues != null && selectedValues.size() >= 1) {
-                    GenomeManager.getInstance().addGenomeItems(selectedValues);
-                    getContentPane().getCommandBar().refreshGenomeListComboBox();
-                    selectGenomeFromList(selectedValues.get(0).getId());
+
+                    if (selectedValues.size() == 1 && dialog.downloadSequence()) {
+                        GenomeListItem oldItem = selectedValues.get(0);
+                        GenomeSelectionDialog.downloadGenome(getMainFrame(), oldItem);
+
+                        File newLocation = new File(DirectoryManager.getGenomeCacheDirectory().getAbsolutePath(), Utilities.getFileNameFromURL(oldItem.getLocation()));
+
+                        GenomeListItem newItem = new GenomeListItem(oldItem.getDisplayableName(), newLocation.getAbsolutePath(), oldItem.getId());
+                        //Checking to see if it has a downloaded sequence might seem redundant,
+                        //but if the user cancels a download we want to use the oldItem
+                        if(newItem.hasDownloadedSequence()){
+                            selectedValues = Arrays.asList(newItem);
+                        }
+                    }
+
+                    if(selectedValues.size() > 0){
+                        GenomeManager.getInstance().addGenomeItems(selectedValues, false);
+                        getContentPane().getCommandBar().refreshGenomeListComboBox();
+                        selectGenomeFromList(selectedValues.get(0).getId());
+                    }
                 }
             }
         };
@@ -602,7 +617,7 @@ public class IGV {
                     progressDialog = ProgressBar.showProgressDialog(mainFrame, "Loading Genome...", monitor, false);
                 }
 
-                loadGenome(file.getAbsolutePath(), monitor);
+                loadGenome(file.getAbsolutePath(), monitor, true);
 
             }
         } catch (IOException e) {
@@ -621,8 +636,20 @@ public class IGV {
 
     }
 
-    public void loadGenome(String path, ProgressMonitor monitor) throws IOException {
-        loadGenome(path, monitor, true);
+    public void loadGenomeById(String genomeId) {
+        if (ParsingUtils.pathExists(genomeId)) {
+            try {
+                IGV.getInstance().loadGenome(genomeId, null, false);
+            } catch (IOException e) {
+                log.error("Error loading genome file: " + genomeId, e);
+            }
+        } else {
+            contentPane.getCommandBar().selectGenome(genomeId);
+        }
+    }
+
+    public void loadGenome(String path, ProgressMonitor monitor, boolean userDefined) throws IOException {
+        loadGenome(path, monitor, true, userDefined);
     }
 
     /**
@@ -631,11 +658,11 @@ public class IGV {
      * @param addGenomeTrack Whether to display the gene track as well
      * @throws IOException
      */
-    public void loadGenome(String path, ProgressMonitor monitor, boolean addGenomeTrack) throws IOException {
+    public void loadGenome(String path, ProgressMonitor monitor, boolean addGenomeTrack, boolean userDefined) throws IOException {
 
         File file = new File(path);
         if (file.exists()) {
-            File directory = file.getParentFile();
+            File directory = file.getAbsoluteFile().getParentFile();
             PreferenceManager.getInstance().setLastGenomeImportDirectory(directory);
         }
 
@@ -649,14 +676,14 @@ public class IGV {
         final String id = genome.getId();
 
         GenomeListItem genomeListItem = new GenomeListItem(name, path, id);
-        getGenomeManager().addGenomeItem(genomeListItem);
+        getGenomeManager().addGenomeItem(genomeListItem, userDefined);
 
         IGVCommandBar cmdBar = contentPane.getCommandBar();
         cmdBar.refreshGenomeListComboBox();
         cmdBar.selectGenome(genomeListItem.getId());
         cmdBar.updateChromosFromGenome(genome);
 
-        FrameManager.getDefaultFrame().setChromosomeName(Globals.CHR_ALL, true);
+        FrameManager.getDefaultFrame().setChromosomeName(genome.getHomeChromosome(), true);
 
         //TODO Should use EventBus/events for changing genome, clean this up a lot
         menuBar.createFileMenu();
@@ -675,13 +702,13 @@ public class IGV {
      *
      * @param locators
      */
-    public void loadTracks(final Collection<ResourceLocator> locators) {
+    public Future loadTracks(final Collection<ResourceLocator> locators) {
 
         contentPane.getStatusBar().setMessage("Loading ...");
 
         log.debug("Run loadTracks");
 
-
+        Future toRet = null;
         if (locators != null && !locators.isEmpty()) {
 
             // NOTE:  this work CANNOT be done on the dispatch thread, it will potentially cause deadlock if
@@ -705,11 +732,10 @@ public class IGV {
                 }
             };
 
-            LongRunningTask.submit(runnable);
+            toRet = LongRunningTask.submit(runnable);
         }
-
         log.debug("Finish loadTracks");
-
+        return toRet;
     }
 
     /**
@@ -717,7 +743,7 @@ public class IGV {
      * changed.  Also record panel sizes
      *
      * @return A 2 element list: 0th element is a map from scrollpane -> number of tracks,
-     *         1st element is a map from scrollpane -> track height (in pixels)
+     * 1st element is a map from scrollpane -> track height (in pixels)
      */
     public List<Map<TrackPanelScrollPane, Integer>> getTrackPanelAttrs() {
         Map<TrackPanelScrollPane, Integer> trackCountMap = new HashMap();
@@ -953,6 +979,14 @@ public class IGV {
         dlg.setVisible(true);
 
         if (!dlg.isCanceled()) {
+            // If any "default" attributes are checked turn off hide default option
+            Set<String> selections = dlg.getSelections();
+            for (String att : AttributeManager.defaultTrackAttributes) {
+                if (selections.contains(att)) {
+                    PreferenceManager.getInstance().put(PreferenceManager.SHOW_DEFAULT_TRACK_ATTRIBUTES, true);
+                    break;
+                }
+            }
             IGV.getInstance().getSession().setHiddenAttributes(dlg.getNonSelections());
             doRefresh();
         }
@@ -984,7 +1018,10 @@ public class IGV {
         try {
             token = WaitCursorManager.showWaitCursor();
             contentPane.getStatusBar().setMessage("Exporting image: " + defaultFile.getAbsolutePath());
-            createSnapshotNonInteractive(target, file, false);
+            String msg = createSnapshotNonInteractive(target, file, false);
+            if(msg != null && msg.toLowerCase().startsWith("error")){
+                MessageUtils.showMessage(msg);
+            }
         } catch (IOException e) {
             log.error("Error creating exporting image ", e);
             MessageUtils.showMessage(("Error creating the image file: " + defaultFile + "<br> "
@@ -1018,11 +1055,15 @@ public class IGV {
 
         String message;
         IOException exc = null;
-        // If valid extension
+
         if (type == SnapshotFileChooser.SnapshotFileType.NULL) {
             message = "ERROR: Unknown file extension " + extension;
             log.error(message);
             return message;
+        } else if (type == SnapshotFileChooser.SnapshotFileType.EPS && !SnapshotUtilities.canExportScreenshotEps()) {
+            message = "ERROR: EPS output requires EPSGraphics library. See https://www.broadinstitute.org/software/igv/third_party_tools#epsgraphics";
+            log.error(message);
+            return message;
         }
 
         //boolean doubleBuffered = RepaintManager.currentManager(contentPane).isDoubleBufferingEnabled();
@@ -1150,8 +1191,7 @@ public class IGV {
      * @param sessionPath
      */
     public void resetSession(String sessionPath) {
-
-        LRUCache.clearCaches();
+        System.gc();
 
         AttributeManager.getInstance().clearAllAttributes();
 
@@ -1178,6 +1218,7 @@ public class IGV {
 
         doRefresh();
 
+        System.gc();
     }
 
     /**
@@ -1363,7 +1404,7 @@ public class IGV {
             setStatusBarMessage("Opening session...");
             inputStream = new BufferedInputStream(ParsingUtils.openInputStreamGZ(new ResourceLocator(sessionPath)));
 
-            boolean isUCSC = sessionPath.endsWith(".session");
+            boolean isUCSC = sessionPath.endsWith(".session") || sessionPath.endsWith(".session.txt");
             final SessionReader sessionReader = isUCSC ?
                     new UCSCSessionReader(this) :
                     new IGVSessionReader(this);
@@ -1380,7 +1421,7 @@ public class IGV {
 
 
             mainFrame.setTitle(UIConstants.APPLICATION_NAME + " - Session: " + sessionPath);
-            LRUCache.clearCaches();
+            System.gc();
 
 
             double[] dividerFractions = session.getDividerFractions();
@@ -1477,6 +1518,28 @@ public class IGV {
         contentPane.getCommandBar().searchByLocus(locus);
     }
 
+    /**
+     * To to multiple loci,  creating a new gene list if required.  This method is provided to support control of
+     * multiple panels from a command or external program.
+     *
+     * @param loci
+     */
+    public void goToLociList(List<String> loci) {
+
+        List<ReferenceFrame> frames = FrameManager.getFrames();
+        if (frames.size() == loci.size()) {
+            for (int i = 0; i < loci.size(); i++) {
+                frames.get(i).jumpTo(new Locus(loci.get(i)));
+            }
+            repaint();
+        } else {
+            GeneList geneList = new GeneList("", loci, false);
+            getSession().setCurrentGeneList(geneList);
+            resetFrames();
+        }
+
+    }
+
     public void tweakPanelDivider() {
         contentPane.getMainPanel().tweakPanelDivider();
     }
@@ -1677,7 +1740,7 @@ public class IGV {
      * @param locator
      * @return A list of loaded tracks
      */
-    public List<Track> load(ResourceLocator locator) {
+    public List<Track> load(ResourceLocator locator) throws DataLoadException {
 
         TrackLoader loader = new TrackLoader();
         List<Track> newTracks = loader.load(locator, this);
@@ -1691,9 +1754,9 @@ public class IGV {
                 if (lastSlashIdx > 0) {
                     fn = fn.substring(lastSlashIdx + 1);
                 }
-                track.setAttributeValue("NAME", track.getName());
-                track.setAttributeValue("DATA FILE", fn);
-                track.setAttributeValue("DATA TYPE", track.getTrackType().toString());
+                track.setAttributeValue(Globals.TRACK_NAME_ATTRIBUTE, track.getName());
+                track.setAttributeValue(Globals.TRACK_DATA_FILE_ATTRIBUTE, fn);
+                track.setAttributeValue(Globals.TRACK_DATA_TYPE_ATTRIBUTE, track.getTrackType().toString());
 
                 if (track instanceof TrackGroupEventListener) {
                     addGroupEventListener((TrackGroupEventListener) track);
@@ -1708,7 +1771,7 @@ public class IGV {
     /**
      * Load the data file into the specified panel.   Triggered via drag and drop.
      */
-    public void load(ResourceLocator locator, TrackPanel panel) {
+    public void load(ResourceLocator locator, TrackPanel panel) throws DataLoadException {
         // If this is a session  TODO -- need better "is a session?" test
         if (locator.getPath().endsWith(".xml") || locator.getPath().endsWith(("session"))) {
             boolean merge = false;  // TODO -- ask user?
@@ -1735,7 +1798,7 @@ public class IGV {
             return getTrackPanel(DATA_PANEL_NAME);
         } else if (path.endsWith(".sam") || path.endsWith(".bam") ||
                 path.endsWith(".sam.list") || path.endsWith(".bam.list") ||
-                path.endsWith(".aligned") || path.endsWith(".sorted.txt")) {
+                path.endsWith(".aligned") || "ga4gh".equals(locator.getType())) {
 
             String newPanelName = "Panel" + System.currentTimeMillis();
             return addDataPanel(newPanelName).getTrackPanel();
@@ -1812,20 +1875,22 @@ public class IGV {
     }
 
 
-    public void sortAlignmentTracks(AlignmentTrack.SortOption option, String tag) {
-        sortAlignmentTracks(option, null, tag);
+    public boolean sortAlignmentTracks(AlignmentTrack.SortOption option, String tag) {
+        return sortAlignmentTracks(option, null, tag);
     }
 
-    public void sortAlignmentTracks(AlignmentTrack.SortOption option, Double location, String tag) {
+    public boolean sortAlignmentTracks(AlignmentTrack.SortOption option, Double location, String tag) {
         double actloc;
+        boolean toRet = true;
         for (Track t : getAllTracks()) {
             if (t instanceof AlignmentTrack) {
                 for (ReferenceFrame frame : FrameManager.getFrames()) {
                     actloc = location != null ? location : frame.getCenter();
-                    ((AlignmentTrack) t).sortRows(option, frame, actloc, tag);
+                    toRet &= ((AlignmentTrack) t).sortRows(option, frame, actloc, tag);
                 }
             }
         }
+        return toRet;
     }
 
     /**
@@ -1837,9 +1902,7 @@ public class IGV {
     public void groupAlignmentTracks(AlignmentTrack.GroupOption option) {
         for (Track t : getAllTracks()) {
             if (t instanceof AlignmentTrack) {
-                for (ReferenceFrame frame : FrameManager.getFrames()) {
-                    ((AlignmentTrack) t).groupAlignments(option, frame);
-                }
+                ((AlignmentTrack) t).groupAlignments(option, FrameManager.getFrames());
             }
         }
     }
@@ -1847,9 +1910,7 @@ public class IGV {
     public void packAlignmentTracks() {
         for (Track t : getAllTracks()) {
             if (t instanceof AlignmentTrack) {
-                for (ReferenceFrame frame : FrameManager.getFrames()) {
-                    ((AlignmentTrack) t).packAlignments(frame);
-                }
+                ((AlignmentTrack) t).packAlignments();
             }
         }
     }
@@ -2032,13 +2093,13 @@ public class IGV {
         HashSet<ResourceLocator> locators = new HashSet();
 
         for (Track track : getAllTracks()) {
-            ResourceLocator locator = track.getResourceLocator();
+            Collection<ResourceLocator> tlocators = track.getResourceLocators();
 
-            if (locator != null) {
-                locators.add(locator);
+            if (tlocators != null) {
+                locators.addAll(tlocators);
             }
         }
-
+        locators.remove(null);
         return locators;
 
     }
@@ -2052,7 +2113,7 @@ public class IGV {
     }
 
 
-    public void removeTracks(Collection<Track> tracksToRemove) {
+    public void removeTracks(Collection<? extends Track> tracksToRemove) {
 
         // Make copy of list as we will be modifying the original in the loop
         List<TrackPanel> panels = getTrackPanels();
@@ -2090,7 +2151,6 @@ public class IGV {
                 getTrackPanel(DATA_PANEL_NAME) : getTrackPanel(FEATURE_PANEL_NAME);
         SequenceTrack newSeqTrack = new SequenceTrack("Reference sequence");
 
-
         panel.addTrack(newSeqTrack);
         if (newGeneTrack != null) {
             panel.addTrack(newGeneTrack);
@@ -2101,12 +2161,26 @@ public class IGV {
     public boolean hasGeneTrack() {
         FeatureTrack geneTrack = GenomeManager.getInstance().getCurrentGenome().getGeneTrack();
         if (geneTrack == null) return false;
-        for (FeatureTrack ft : getFeatureTracks()) {
-            if (geneTrack == ft) return true;
+        for (Track t : getFeatureTracks()) {
+            if (geneTrack == t) return true;
         }
         return false;
     }
 
+    public boolean hasSequenceTrack() {
+        return getSequenceTrack() != null;
+    }
+
+    /**
+     * @return First SequenceTrack found, or null if none
+     */
+    public SequenceTrack getSequenceTrack() {
+        for (Track t : getAllTracks()) {
+            if (t instanceof SequenceTrack) return (SequenceTrack) t;
+        }
+        return null;
+    }
+
     /////////////////////////////////////////////////////////////////////////////////////////
     // Sorting
 
@@ -2314,6 +2388,14 @@ public class IGV {
         return LongRunningTask.submit(new StartupRunnable(igvArgs));
     }
 
+    public void setRulerEnabled(boolean rulerEnabled) {
+        this.rulerEnabled = rulerEnabled;
+    }
+
+    public boolean isRulerEnabled() {
+        return rulerEnabled;
+    }
+
     /**
      * Swing worker class to startup IGV
      */
@@ -2328,6 +2410,9 @@ public class IGV {
         @Override
         public void run() {
 
+            final boolean runningBatch = igvArgs.getBatchFile() != null;
+            BatchRunner.setIsBatchMode(runningBatch);
+
             final ProgressMonitor monitor = new ProgressMonitor();
             final ProgressBar.ProgressDialog progressDialog = ProgressBar.showProgressDialog(mainFrame, "Initializing...", monitor, false);
             progressDialog.getProgressBar().setIndeterminate(true);
@@ -2354,20 +2439,15 @@ public class IGV {
             }
 
             if (igvArgs.getGenomeId() != null) {
-                if (ParsingUtils.pathExists(igvArgs.getGenomeId())) {
-                    try {
-                        IGV.getInstance().loadGenome(igvArgs.getGenomeId(), null);
-                    } catch (IOException e) {
-                        log.error("Error loading genome file: " + igvArgs.getGenomeId());
-                    }
-                } else {
-                    contentPane.getCommandBar().selectGenome(igvArgs.getGenomeId());
-                }
+                IGV.getInstance().loadGenomeById(igvArgs.getGenomeId());
             } else if (igvArgs.getSessionFile() == null) {
                 String genomeId = preferenceManager.getDefaultGenome();
                 contentPane.getCommandBar().selectGenome(genomeId);
             }
 
+            //Load plugins
+            //Do this before loading files, hooks might need to be inserted
+            initIGVPlugins();
 
             //If there is an argument assume it is a session file or url
             if (igvArgs.getSessionFile() != null || igvArgs.getDataFileString() != null) {
@@ -2404,16 +2484,27 @@ public class IGV {
                     }
                 } else if (igvArgs.getDataFileString() != null) {
                     // Not an xml file, assume its a list of data files
-                    String[] tokens = igvArgs.getDataFileString().split(",");
+                    String[] dataFiles = igvArgs.getDataFileString().split(",");
                     String[] names = null;
                     if (igvArgs.getName() != null) {
                         names = igvArgs.getName().split(",");
                     }
+                    String[] indexFiles = null;
+                    if (igvArgs.getIndexFile() != null) {
+                        indexFiles = igvArgs.getIndexFile().split(",");
+                    }
+                    String[] coverageFiles = null;
+                    if (igvArgs.getCoverageFile() != null) {
+                        coverageFiles = igvArgs.getCoverageFile().split(",");
+                    }
+
 
-                    String indexFile = igvArgs.getIndexFile();
                     List<ResourceLocator> locators = new ArrayList();
-                    int idx = 0;
-                    for (String p : tokens) {
+
+
+                    for (int i = 0; i < dataFiles.length; i++) {
+
+                        String p = dataFiles[i].trim();
 
                         // Decode local file paths
                         if (HttpUtils.isURL(p) && !FileUtils.isRemote(p)) {
@@ -2422,15 +2513,36 @@ public class IGV {
 
                         ResourceLocator rl = new ResourceLocator(p);
 
-                        if (names != null && idx < names.length) {
-                            String name = names[idx];
+                        if (names != null && i < names.length) {
+                            String name = names[i];
                             rl.setName(name);
                         }
-                        rl.setIndexPath(indexFile);
+
+                        //Set index file, iff one was passed
+                        if(indexFiles != null && i < indexFiles.length) {
+                            String idxP = indexFiles[i];
+                            if (HttpUtils.isURL(idxP) && !FileUtils.isRemote(idxP)) {
+                                idxP = StringUtils.decodeURL(idxP);
+                            }
+                            if (idxP.length() > 0) {
+                                rl.setIndexPath(idxP);
+                            }
+                        }
+
+                        //Set coverage file, iff one was passed
+                        if(coverageFiles != null && i < coverageFiles.length) {
+                            String covP = coverageFiles[i];
+                            if (HttpUtils.isURL(covP) && !FileUtils.isRemote(covP)) {
+                                covP = StringUtils.decodeURL(covP);
+                            }
+                            if (covP.length() > 0) {
+                                rl.setCoverage(covP);
+                            }
+                        }
+
                         locators.add(rl);
-                        idx++;
                     }
-                    loadResources(locators);
+                    loadTracks(locators);
                 }
 
 
@@ -2458,19 +2570,16 @@ public class IGV {
                     if (igvArgs.getLocusString() != null) {
                         goToLocus(igvArgs.getLocusString());
                     }
-                    if (igvArgs.getBatchFile() != null) {
+                    if (runningBatch) {
                         LongRunningTask.submit(new BatchRunner(igvArgs.getBatchFile()));
                     }
 
                 }
             });
 
-            initIGVPlugins();
-
             synchronized (IGV.getInstance()) {
                 IGV.getInstance().notifyAll();
             }
-
         }
 
         private void setAppleDockIcon() {
@@ -2507,6 +2616,7 @@ public class IGV {
             }
             pluginClassNames.addAll(Arrays.asList(PreferenceManager.getInstance().getIGVPluginList()));
             for (String classname : pluginClassNames) {
+                if(classname.startsWith("#")) continue;
                 try {
                     Class clazz = Class.forName(classname);
                     IGVPlugin plugin = (IGVPlugin) clazz.newInstance();
