@@ -24,7 +24,8 @@ import org.broad.igv.cli_plugin.ui.SetPluginPathDialog;
 import org.broad.igv.dev.db.DBProfileEditor;
 import org.broad.igv.feature.genome.GenomeListItem;
 import org.broad.igv.feature.genome.GenomeManager;
-import org.broad.igv.feature.tribble.IGVBEDCodec;
+import org.broad.igv.ga4gh.Ga4ghAPIHelper;
+import org.broad.igv.ga4gh.OAuthUtils;
 import org.broad.igv.gs.GSOpenSessionMenuAction;
 import org.broad.igv.gs.GSSaveSessionMenuAction;
 import org.broad.igv.gs.GSUtils;
@@ -32,8 +33,6 @@ import org.broad.igv.lists.GeneListManagerUI;
 import org.broad.igv.lists.VariantListManager;
 import org.broad.igv.tools.IgvToolsGui;
 import org.broad.igv.track.CombinedDataSourceDialog;
-import org.broad.igv.track.FeatureTrack;
-import org.broad.igv.track.Track;
 import org.broad.igv.ui.action.*;
 import org.broad.igv.ui.legend.LegendDialog;
 import org.broad.igv.ui.panel.FrameManager;
@@ -41,22 +40,24 @@ import org.broad.igv.ui.panel.MainPanel;
 import org.broad.igv.ui.panel.ReferenceFrame;
 import org.broad.igv.ui.panel.ReorderPanelsDialog;
 import org.broad.igv.ui.util.*;
-import org.broad.igv.util.BrowserLauncher;
-import org.broad.tribble.Feature;
+import org.broad.igv.util.*;
+import org.broad.igv.util.encode.EncodeFileBrowser;
 
 import javax.swing.*;
+import javax.swing.event.MenuEvent;
+import javax.swing.event.MenuListener;
 import javax.swing.plaf.basic.BasicBorders;
 import java.awt.*;
 import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
 import java.awt.event.KeyEvent;
 import java.io.File;
-import java.io.FileNotFoundException;
 import java.io.IOException;
-import java.io.PrintWriter;
 import java.net.URI;
+import java.net.URL;
 import java.util.ArrayList;
-import java.util.Collection;
+import java.util.Arrays;
+import java.util.HashSet;
 import java.util.List;
 
 import static org.broad.igv.ui.UIConstants.*;
@@ -92,6 +93,7 @@ public class IGVMenuBar extends JMenuBar {
     private static final String CANNOT_LOAD_GENOME_SERVER_TOOLTIP = "Could not reach genome server";
 
     private static IGVMenuBar instance;
+    private JMenu googleMenu;
 
     public void notifyGenomeServerReachable(boolean reachable) {
         if (loadFromServerMenuItem != null) {
@@ -160,6 +162,12 @@ public class IGVMenuBar extends JMenuBar {
         //extrasMenu.setVisible(false);
         menus.add(extrasMenu);
 
+
+        googleMenu = createGoogleMenu();
+        googleMenu.setVisible(PreferenceManager.getInstance().get(PreferenceManager.GOOGLE_API_KEY) != null ||
+                PreferenceManager.getInstance().getAsBoolean(PreferenceManager.ENABLE_GOOGLE_MENU));
+        menus.add(googleMenu);
+
         menus.add(createHelpMenu());
 
         // Experimental -- remove for production release
@@ -221,21 +229,6 @@ public class IGVMenuBar extends JMenuBar {
 
         //-------------------------------------//
 
-        JMenuItem exportData = new JMenuItem("Export Features");
-        exportData.addActionListener(new ActionListener() {
-            @Override
-            public void actionPerformed(ActionEvent e) {
-                File outFile = FileDialogUtils.chooseFile("Save Visible Data",
-                        PreferenceManager.getInstance().getLastTrackDirectory(),
-                        new File("visibleData.bed"),
-                        FileDialogUtils.SAVE);
-                IGVMenuBar.exportVisibleData(outFile.getAbsolutePath(), IGV.getInstance().getAllTracks());
-            }
-        });
-        //menuItems.add(exportData);
-
-        //-------------------------------------//
-
         for (final PluginSpecReader pluginSpecReader : PluginSpecReader.getPlugins()) {
             for (final PluginSpecReader.Tool tool : pluginSpecReader.getTools()) {
                 final String toolName = tool.name;
@@ -270,7 +263,14 @@ public class IGVMenuBar extends JMenuBar {
                             cmdItem.addActionListener(new ActionListener() {
                                 @Override
                                 public void actionPerformed(ActionEvent e) {
-                                    (new RunPlugin(IGV.getMainFrame(), pluginSpecReader, tool, command)).setVisible(true);
+                                    RunPlugin runPlugin = null;
+                                    try {
+                                        runPlugin = new RunPlugin(IGV.getMainFrame(), pluginSpecReader, tool, command);
+                                    } catch (IllegalStateException e1) {
+                                        MessageUtils.showErrorMessage(e1.getMessage(), e1);
+                                        return;
+                                    }
+                                    runPlugin.setVisible(true);
                                 }
                             });
                             cmdItem.setEnabled(true);
@@ -298,7 +298,7 @@ public class IGVMenuBar extends JMenuBar {
         //-------------------------------------//
 
         //-----------SQL DB Tools--------------//
-        boolean showDBEditor = Boolean.parseBoolean(System.getProperty(DBProfileEditor.ENABLE_EDITOR_PROPERTY, "false"));
+        boolean showDBEditor = Globals.isDevelopment();
         if (showDBEditor) {
             JMenu sqlDBProfileEditor = new JMenu("SQL DB Profile Editor");
             JMenuItem createNewProfile = new JMenuItem("Create New Profile");
@@ -306,7 +306,7 @@ public class IGVMenuBar extends JMenuBar {
                 @Override
                 public void actionPerformed(ActionEvent e) {
                     File file = FileDialogUtils.chooseFile("Save DB Profile", DirectoryManager.getUserDirectory(), FileDialogUtils.SAVE);
-                    if(file != null){
+                    if (file != null) {
                         DBProfileEditor editor = new DBProfileEditor(IGV.getMainFrame(), file.getAbsolutePath());
                         editor.setVisible(true);
                     }
@@ -317,8 +317,8 @@ public class IGVMenuBar extends JMenuBar {
                 @Override
                 public void actionPerformed(ActionEvent e) {
                     File file = FileDialogUtils.chooseFile("Select .dbxml database profile");
-                    if(file != null){
-                        if(!file.exists()){
+                    if (file != null) {
+                        if (!file.exists()) {
 
                         }
                         DBProfileEditor editor = new DBProfileEditor(IGV.getMainFrame(), file.getAbsolutePath());
@@ -336,17 +336,17 @@ public class IGVMenuBar extends JMenuBar {
 
 
         //DataTrack Math------------------------//
-        if(!Globals.isProduction()){
-            JMenuItem combineDataItem = new JMenuItem("Combine Data Tracks");
-            combineDataItem.addActionListener(new ActionListener() {
-                @Override
-                public void actionPerformed(ActionEvent e) {
-                    CombinedDataSourceDialog dialog = new CombinedDataSourceDialog(IGV.getMainFrame());
-                    dialog.setVisible(true);
-                }
-            });
-            menuItems.add(combineDataItem);
-        }
+
+        JMenuItem combineDataItem = new JMenuItem("Combine Data Tracks");
+        combineDataItem.addActionListener(new ActionListener() {
+            @Override
+            public void actionPerformed(ActionEvent e) {
+                CombinedDataSourceDialog dialog = new CombinedDataSourceDialog(IGV.getMainFrame());
+                dialog.setVisible(true);
+            }
+        });
+        menuItems.add(combineDataItem);
+
 
         //-------------------------------------//
 
@@ -354,6 +354,7 @@ public class IGVMenuBar extends JMenuBar {
         MenuAction toolsMenuAction = new MenuAction("Tools", null);
         if (toolsMenu == null) {
             toolsMenu = MenuAndToolbarUtils.createMenu(menuItems, toolsMenuAction);
+            toolsMenu.setName("Tools");
         } else {
             toolsMenu.removeAll();
             for (JComponent item : menuItems) {
@@ -398,6 +399,18 @@ public class IGVMenuBar extends JMenuBar {
             menuItems.add(MenuAndToolbarUtils.createMenuItem(menuAction));
         }
 
+        String genomeId = IGV.getInstance().getGenomeManager().getGenomeId();
+        if (EncodeFileBrowser.genomeSupported(genomeId)) {
+            menuAction = new BrowseEncodeAction("Load from ENCODE...", KeyEvent.VK_E, igv);
+            menuItems.add(MenuAndToolbarUtils.createMenuItem(menuAction));
+        }
+
+        if (PreferenceManager.getInstance().get(PreferenceManager.GOOGLE_API_KEY) != null ||
+                PreferenceManager.getInstance().getAsBoolean(PreferenceManager.ENABLE_GOOGLE_MENU)) {
+            menuAction = new BrowseGa4ghAction("Load from Ga4gh...", KeyEvent.VK_G, igv);
+            menuItems.add(MenuAndToolbarUtils.createMenuItem(menuAction));
+        }
+
         //Disable loading if no genome loaded. Something of an edge case
         if (!genomeLoaded) {
             for (JComponent menuItem : menuItems) {
@@ -576,6 +589,11 @@ public class IGVMenuBar extends JMenuBar {
                     GenomeManager.getInstance().buildGenomeItemList();
                     igv.getContentPane().getCommandBar().refreshGenomeListComboBox();
                     if (removedValuesList != null && !removedValuesList.isEmpty()) {
+                        try {
+                            GenomeManager.getInstance().deleteDownloadedGenomes(removedValuesList);
+                        } catch (IOException e) {
+                            MessageUtils.showErrorMessage("Error deleting genome files", e);
+                        }
                         GenomeManager.getInstance().updateImportedGenomePropertyFile();
                         notifyGenomesAddedRemoved(removedValuesList, false);
                     }
@@ -860,6 +878,16 @@ public class IGVMenuBar extends JMenuBar {
             }
         }
 
+        menuAction =
+                new MenuAction("Check for Updates...") {
+
+                    @Override
+                    public void actionPerformed(ActionEvent e) {
+                        checkVersion();
+                    }
+                };
+        menuItems.add(MenuAndToolbarUtils.createMenuItem(menuAction));
+
         menuAction =
                 new MenuAction("About IGV ") {
 
@@ -872,9 +900,55 @@ public class IGVMenuBar extends JMenuBar {
         menuItems.add(MenuAndToolbarUtils.createMenuItem(menuAction));
 
         MenuAction helpMenuAction = new MenuAction("Help");
+
+
         return MenuAndToolbarUtils.createMenu(menuItems, helpMenuAction);
     }
 
+    private void checkVersion() {
+
+        int readTimeout = Globals.READ_TIMEOUT;
+        int connectTimeout = Globals.CONNECT_TIMEOUT;
+
+        try {
+            Main.Version thisVersion = Main.Version.getVersion(Globals.VERSION);
+            if (thisVersion == null) return;  // Can't compare
+
+            Globals.CONNECT_TIMEOUT = 5000;
+            Globals.READ_TIMEOUT = 1000;
+            final String serverVersionString = HttpUtils.getInstance().getContentsAsString(new URL(Globals.getVersionURL())).trim();
+            // See if user has specified to skip this update
+
+            final String skipString = PreferenceManager.getInstance().get(PreferenceManager.SKIP_VERSION);
+            HashSet<String> skipVersion = new HashSet<String>(Arrays.asList(skipString.split(",")));
+            if (skipVersion.contains(serverVersionString)) return;
+
+            Main.Version serverVersion = Main.Version.getVersion(serverVersionString.trim());
+            if (serverVersion == null) return;
+
+            if (thisVersion.lessThan(serverVersion)) {
+
+                log.info("A later version of IGV is available (" + serverVersionString + ")");
+                final VersionUpdateDialog dlg = new VersionUpdateDialog(serverVersionString);
+
+                dlg.setVisible(true);
+                if (dlg.isSkipVersion()) {
+                    String newSkipString = skipString + "," + serverVersionString;
+                    PreferenceManager.getInstance().put(PreferenceManager.SKIP_VERSION, newSkipString);
+                }
+
+            } else {
+                MessageUtils.showMessage("IGV is up to date");
+            }
+
+        } catch (Exception e) {
+            log.error("Error checking version", e);
+        } finally {
+            Globals.CONNECT_TIMEOUT = connectTimeout;
+            Globals.READ_TIMEOUT = readTimeout;
+        }
+    }
+
     private JMenu createGenomeSpaceMenu() {
 
         JMenu menu = new JMenu("GenomeSpace");
@@ -1055,6 +1129,88 @@ public class IGVMenuBar extends JMenuBar {
         return menu;
     }
 
+    private JMenu createGoogleMenu() {
+
+        final JMenu menu = new JMenu("Google");
+
+        final JMenuItem login = new JMenuItem("Login ... ");
+        login.addActionListener(new ActionListener() {
+            @Override
+            public void actionPerformed(ActionEvent e) {
+                try {
+                    OAuthUtils.getInstance().openAuthorizationPage();
+                } catch (Exception ex) {
+                    MessageUtils.showErrorMessage("Error fetching oAuth tokens.  See log for details", ex);
+                    log.error("Error fetching oAuth tokens", ex);
+                }
+
+            }
+        });
+        login.setEnabled(false);
+        menu.add(login);
+
+
+        final JMenuItem logout = new JMenuItem("Logout ");
+        logout.addActionListener(new ActionListener() {
+            @Override
+            public void actionPerformed(ActionEvent e) {
+                OAuthUtils.getInstance().logout();
+            }
+        });
+        logout.setEnabled(false);
+        menu.add(logout);
+
+        final JMenuItem loadReadset = new JMenuItem("Load Genomics ReadGroupSet... ");
+        loadReadset.addActionListener(new ActionListener() {
+            @Override
+            public void actionPerformed(ActionEvent e) {
+                String readsetId = MessageUtils.showInputDialog("Enter ReadGroupSet ID (e.g. CMvnhpKTFhCjz9_25e_lCw): ");
+                if (readsetId != null) {
+                    ResourceLocator locator = new ResourceLocator(readsetId);
+                    locator.setName(readsetId);
+                    locator.setType(Ga4ghAPIHelper.RESOURCE_TYPE);
+                    locator.setAttribute("provider", Ga4ghAPIHelper.GA4GH_GOOGLE_PROVIDER);
+                    IGV.getInstance().loadTracks(Arrays.asList(locator));
+                }
+            }
+        });
+        loadReadset.setEnabled(false);
+        menu.add(loadReadset);
+
+        menu.addMenuListener(new MenuListener() {
+            @Override
+            public void menuSelected(MenuEvent e) {
+                Runnable runnable = new Runnable() {
+                    @Override
+                    public void run() {
+                        boolean loggedIn = OAuthUtils.getInstance().isLoggedIn();
+                        if (loggedIn) {
+                            login.setText(OAuthUtils.getInstance().getCurrentUserName());
+                        } else {
+                            login.setText("Login ...");
+                        }
+                        login.setEnabled(!loggedIn);
+                        logout.setEnabled(loggedIn);
+                        loadReadset.setEnabled(loggedIn);
+                    }
+                };
+                LongRunningTask.submit(runnable);
+            }
+
+            @Override
+            public void menuDeselected(MenuEvent e) {
+
+            }
+
+            @Override
+            public void menuCanceled(MenuEvent e) {
+
+            }
+        });
+
+
+        return menu;
+    }
 
 //    public void enableRemoveGenomes() {
 //        if (removeImportedGenomeAction != null) {
@@ -1119,35 +1275,12 @@ public class IGVMenuBar extends JMenuBar {
 
     }
 
-    /**
-     * Write visible data to a file
-     * TODO Move to own action class, thread
-     */
-    private static final void exportVisibleData(String outPath, Collection<Track> tracks) {
-        PrintWriter writer;
-        try {
-            writer = new PrintWriter(outPath);
-        } catch (FileNotFoundException e) {
-            throw new RuntimeException(e);
-        }
-        ReferenceFrame.Range range = FrameManager.getDefaultFrame().getCurrentRange();
-        for (Track track : tracks) {
-            if (track instanceof FeatureTrack) {
-                FeatureTrack fTrack = (FeatureTrack) track;
-                List<Feature> features = fTrack.getFeatures(range.getChr(), range.getStart(), range.getEnd());
-                IGVBEDCodec codec = new IGVBEDCodec();
-                for (Feature feat : features) {
-                    String featString = codec.encode(feat);
-                    writer.println(featString);
-                }
-            }
-        }
-        writer.flush();
-        writer.close();
-    }
-
     @ForTesting
     static void destroyInstance() {
         instance = null;
     }
+
+    public void enableGoogleMenu(boolean aBoolean) {
+        googleMenu.setVisible(aBoolean);
+    }
 }
