@@ -46,7 +46,6 @@ public class SashimiPlot extends JFrame {
 
     private List<SpliceJunctionFinderTrack> spliceJunctionTracks;
     private ReferenceFrame frame;
-    private AlignmentDataManager dataManager;  // <= retained to release self from event bus
 
     /**
      * The minimum allowed origin of the frame. We set scrolling
@@ -61,7 +60,6 @@ public class SashimiPlot extends JFrame {
     private final double maxEnd;
 
 
-
     private static final List<Color> plotColors;
 
     static {
@@ -78,7 +76,6 @@ public class SashimiPlot extends JFrame {
 
         minOrigin = this.frame.getOrigin();
         maxEnd = this.frame.getEnd();
-        int minZoom = this.frame.getZoom();
 
         initSize(frame.getWidthInPixels());
 
@@ -86,32 +83,21 @@ public class SashimiPlot extends JFrame {
         getContentPane().setLayout(boxLayout);
 
         //Add control elements to the top
-        ZoomSliderPanel controlPanel = new ZoomSliderPanel(this.frame);
-        controlPanel.setMinZoomLevel(minZoom);
-
-        Dimension dimSize = new Dimension(200, 30);
-        controlPanel.setPreferredSize(dimSize);
-        controlPanel.setMinimumSize(dimSize);
-        controlPanel.setMaximumSize(dimSize);
-
-        getContentPane().add(controlPanel);
-        controlPanel.addMouseListener(new MouseAdapter() {
-            @Override
-            public void mouseClicked(MouseEvent e) {
-                SashimiPlot.this.repaint();
-            }
-        });
+        getContentPane().add(generateControlPanel(this.frame));
 
         spliceJunctionTracks = new ArrayList<SpliceJunctionFinderTrack>(alignmentTracks.size());
         int colorInd = 0;
 
-        for(AlignmentTrack alignmentTrack: alignmentTracks){
+        for (AlignmentTrack alignmentTrack : alignmentTracks) {
 
 
             AlignmentDataManager oldDataManager = alignmentTrack.getDataManager();
             MemoryAlignmentDataManager dataManager = new MemoryAlignmentDataManager(oldDataManager, oldDataManager.getSpliceJunctionLoadOptions());
 
-            SpliceJunctionFinderTrack spliceJunctionTrack = new SpliceJunctionFinderTrack(alignmentTrack.getResourceLocator(), alignmentTrack.getName(), dataManager, true);
+            SpliceJunctionFinderTrack spliceJunctionTrack =
+                    new SpliceJunctionFinderTrack(alignmentTrack.getResourceLocator(), alignmentTrack.getName(), dataManager, SpliceJunctionFinderTrack.StrandOption.COMBINE);
+            // Override expand/collpase setting -- expanded sashimi plots make no sense
+            spliceJunctionTrack.setDisplayMode(Track.DisplayMode.COLLAPSED);
 
             spliceJunctionTrack.setRendererClass(SashimiJunctionRenderer.class);
 
@@ -133,13 +119,50 @@ public class SashimiPlot extends JFrame {
         SelectableFeatureTrack geneTrackClone = new SelectableFeatureTrack(geneTrack);
         TrackComponent<SelectableFeatureTrack> geneComponent = new TrackComponent<SelectableFeatureTrack>(frame, geneTrackClone);
 
-
         getContentPane().add(geneComponent);
 
         initGeneComponent(frame.getWidthInPixels(), geneComponent, geneTrackClone);
         validate();
     }
 
+    private Component generateControlPanel(ReferenceFrame frame) {
+        JPanel controlPanel = new JPanel();
+
+        ZoomSliderPanel zoomSliderPanel = new ZoomSliderPanel(frame);
+        zoomSliderPanel.setMinZoomLevel(frame.getZoom());
+
+        zoomSliderPanel.addMouseListener(new MouseAdapter() {
+            @Override
+            public void mouseClicked(MouseEvent e) {
+                SashimiPlot.this.repaint();
+            }
+        });
+
+        Dimension controlSize = new Dimension(200, 30);
+
+        //JSlider scaleSlider = new JSlider(JSlider.HORIZONTAL);
+        //setFixedSize(scaleSlider, controlSize);
+        //controlPanel.add(scaleSlider);
+
+        controlPanel.add(zoomSliderPanel);
+        setFixedSize(zoomSliderPanel, controlSize);
+
+        Dimension panelSize = controlSize;
+        setFixedSize(controlPanel, panelSize);
+
+
+        BoxLayout layout = new BoxLayout(controlPanel, BoxLayout.X_AXIS);
+        controlPanel.setLayout(layout);
+
+        return controlPanel;
+    }
+
+    private static void setFixedSize(Component component, Dimension dimension) {
+        component.setPreferredSize(dimension);
+        component.setMinimumSize(dimension);
+        component.setMaximumSize(dimension);
+    }
+
     private void initSize(int width) {
         setSize(width, 500);
     }
@@ -160,10 +183,10 @@ public class SashimiPlot extends JFrame {
 
         geneTrack.setDisplayMode(Track.DisplayMode.SQUISHED);
 
-        //Hacky way of clearing packed features
-        geneTrack.setVisibilityWindow(geneTrack.getVisibilityWindow());
+        geneTrack.clearPackedFeatures();
         RenderContext context = new RenderContextImpl(geneComponent, null, frame, null);
-        geneTrack.preload(context);
+        geneTrack.setForceLoadSync(true);
+        geneTrack.load(context);
 
 
         Dimension maxGeneDim = new Dimension(Integer.MAX_VALUE, geneTrack.getNumberOfFeatureLevels() * geneTrack.getSquishedRowHeight() + 10);
@@ -184,6 +207,7 @@ public class SashimiPlot extends JFrame {
 
         getRenderer(trackComponent.track).setDataManager(dataManager);
         getRenderer(trackComponent.track).setCoverageTrack(coverageTrack);
+        getRenderer(trackComponent.track).getCoverageTrack().rescale(trackComponent.frame);
 
         dataManager.setMinJunctionCoverage(minJunctionCoverage);
 
@@ -202,12 +226,18 @@ public class SashimiPlot extends JFrame {
 
         private T track;
         private ReferenceFrame frame;
+        private String toolTipText = null;
 
         public TrackComponent(ReferenceFrame frame, T track) {
             this.frame = frame;
             this.track = track;
         }
 
+        public void updateToolTipText(TrackClickEvent tce) {
+            toolTipText = track.getValueStringAt(tce.getFrame().getChrName(), tce.getChromosomePosition(), tce.getMouseEvent().getY(), tce.getFrame());
+            toolTipText = "<html>" + toolTipText;
+            setToolTipText(toolTipText);
+        }
 
         @Override
         public void paintComponent(Graphics g) {
@@ -219,6 +249,35 @@ public class SashimiPlot extends JFrame {
 
     }
 
+    /**
+     * Set the minimum junction coverage, per trac,k and is not persistent
+     * <p/>
+     * Our "Set Max Junction Coverage Range" just changes the view scaling, it doesn't
+     * filter anything, which is different behavior than the minimum. This might be confusing.
+     *
+     * @param trackComponent
+     * @param newMinJunctionCoverage
+     */
+    private void setMinJunctionCoverage(TrackComponent<SpliceJunctionFinderTrack> trackComponent, int newMinJunctionCoverage) {
+        IAlignmentDataManager dataManager = getRenderer(trackComponent.track).getDataManager();
+        dataManager.setMinJunctionCoverage(newMinJunctionCoverage);
+        trackComponent.track.clear();
+        trackComponent.repaint();
+    }
+
+
+    /**
+     * Set the max coverage depth, which is a graphical scaling parameter for determining how
+     * thick the junction arcs will be
+     *
+     * @param trackComponent
+     * @param newMaxDepth
+     */
+    private void setMaxCoverageDepth(TrackComponent<SpliceJunctionFinderTrack> trackComponent, int newMaxDepth) {
+        getRenderer(trackComponent.track).setMaxDepth(newMaxDepth);
+        repaint();
+    }
+
     private class JunctionTrackMouseAdapter extends TrackComponentMouseAdapter<SpliceJunctionFinderTrack> {
 
         JunctionTrackMouseAdapter(TrackComponent<SpliceJunctionFinderTrack> trackComponent) {
@@ -234,46 +293,21 @@ public class SashimiPlot extends JFrame {
         protected IGVPopupMenu getPopupMenu(MouseEvent e) {
             IGVPopupMenu menu = new IGVPopupMenu();
 
-            CoverageTrack covTrack = getRenderer(this.trackComponent.track).getCoverageTrack();
-            JMenuItem setCoverageDataRange = CoverageTrack.addDataRangeItem(SashimiPlot.this, null, Arrays.asList(covTrack));
-            setCoverageDataRange.setText("Set Coverage Data Range");
-            menu.add(setCoverageDataRange);
-
-            JMenuItem minJunctionCoverage = new JMenuItem("Set Min Junction Coverage");
-            minJunctionCoverage.setToolTipText("Junctions below this threshold will be removed from view");
-            minJunctionCoverage.addActionListener(new ActionListener() {
+            final JCheckBoxMenuItem showCoverageData = new JCheckBoxMenuItem("Show Exon Coverage Data");
+            showCoverageData.setSelected(PreferenceManager.getInstance().getAsBoolean(PreferenceManager.SASHIMI_SHOW_COVERAGE));
+            showCoverageData.addActionListener(new ActionListener() {
                 @Override
                 public void actionPerformed(ActionEvent e) {
-                    /** The popup sets it per track and is not persistent
-                     *
-                     * On top of this, our "Set Max Junction Coverage Range" just changes the view scaling, it doesn't
-                     * filter anything, which is different behavior than the minimum. This might be confusing.
-                     *
-                     * Did some refactoring, going to set it for the track and clear that tracks data, but the setting
-                     * will not persist at all. May be weird for users. Still has the problem that max/min do very different
-                     * things.
-                     */
-
-                    IAlignmentDataManager dataManager = getRenderer(trackComponent.track).getDataManager();
-                    SpliceJunctionHelper.LoadOptions loadOptions = dataManager.getSpliceJunctionLoadOptions();
-
-                    String input = JOptionPane.showInputDialog("Set Minimum Junction Coverage", loadOptions.minJunctionCoverage);
-                    if (input == null || input.length() == 0) return;
-                    try {
-                        int newMinJunctionCoverage = Integer.parseInt(input);
-                        dataManager.setMinJunctionCoverage(newMinJunctionCoverage);
-
-                        trackComponent.track.onAlignmentTrackEvent(new AlignmentTrackEvent(this, AlignmentTrackEvent.Type.SPLICE_JUNCTION));
-                        trackComponent.repaint();
-                    } catch (NumberFormatException ex) {
-                        JOptionPane.showMessageDialog(SashimiPlot.this, input + " is not an integer");
-                    }
-
-
+                    PreferenceManager.getInstance().put(PreferenceManager.SASHIMI_SHOW_COVERAGE, showCoverageData.isSelected());
+                    SashimiPlot.this.repaint();
                 }
             });
 
-            JMenuItem maxJunctionCoverageRange = new JMenuItem("Set Max Junction Coverage Range");
+            CoverageTrack covTrack = getRenderer(this.trackComponent.track).getCoverageTrack();
+            JMenuItem setCoverageDataRange = CoverageTrack.addDataRangeItem(SashimiPlot.this, null, Arrays.asList(covTrack));
+            setCoverageDataRange.setText("Set Exon Coverage Max");
+
+            JMenuItem maxJunctionCoverageRange = new JMenuItem("Set Junction Coverage Max");
             maxJunctionCoverageRange.setToolTipText("The thickness of each line will be proportional to the coverage, up until this value");
 
             maxJunctionCoverageRange.addActionListener(new ActionListener() {
@@ -283,13 +317,32 @@ public class SashimiPlot extends JFrame {
                     if (input == null || input.length() == 0) return;
                     try {
                         int newMaxDepth = Integer.parseInt(input);
-                        getRenderer(trackComponent.track).setMaxDepth(newMaxDepth);
-                        repaint();
+                        setMaxCoverageDepth(JunctionTrackMouseAdapter.this.trackComponent, newMaxDepth);
                     } catch (NumberFormatException ex) {
                         JOptionPane.showMessageDialog(SashimiPlot.this, input + " is not an integer");
                     }
                 }
             });
+            JMenuItem minJunctionCoverage = new JMenuItem("Set Junction Coverage Min");
+            minJunctionCoverage.setToolTipText("Junctions below this threshold will be removed from view");
+            minJunctionCoverage.addActionListener(new ActionListener() {
+                @Override
+                public void actionPerformed(ActionEvent e) {
+                    IAlignmentDataManager dataManager = getRenderer(trackComponent.track).getDataManager();
+                    SpliceJunctionHelper.LoadOptions loadOptions = dataManager.getSpliceJunctionLoadOptions();
+
+                    String input = JOptionPane.showInputDialog("Set Minimum Junction Coverage", loadOptions.minJunctionCoverage);
+                    if (input == null || input.length() == 0) return;
+                    try {
+                        int newMinJunctionCoverage = Integer.parseInt(input);
+                        setMinJunctionCoverage(JunctionTrackMouseAdapter.this.trackComponent, newMinJunctionCoverage);
+
+                    } catch (NumberFormatException ex) {
+                        JOptionPane.showMessageDialog(SashimiPlot.this, input + " is not an integer");
+                    }
+                }
+            });
+
 
             JMenuItem colorItem = new JMenuItem("Set Color");
             colorItem.addActionListener(new ActionListener() {
@@ -304,6 +357,44 @@ public class SashimiPlot extends JFrame {
                 }
             });
 
+            ButtonGroup shapeGroup = new ButtonGroup();
+
+            JRadioButtonMenuItem textShape = getJunctionCoverageRadioButton("Text", SashimiJunctionRenderer.ShapeType.TEXT);
+            textShape.setToolTipText("Show junction coverage as text");
+            shapeGroup.add(textShape);
+
+            JRadioButtonMenuItem circleShape = getJunctionCoverageRadioButton("Circle", SashimiJunctionRenderer.ShapeType.CIRCLE);
+            circleShape.setToolTipText("Show junction coverage as a circle");
+            shapeGroup.add(circleShape);
+
+            JRadioButtonMenuItem ellipseShape = getJunctionCoverageRadioButton("Ellipse", SashimiJunctionRenderer.ShapeType.ELLIPSE);
+            ellipseShape.setToolTipText("Show junction coverage as an ellipse");
+            shapeGroup.add(ellipseShape);
+
+            JRadioButtonMenuItem noShape = getJunctionCoverageRadioButton("None", SashimiJunctionRenderer.ShapeType.NONE);
+            ellipseShape.setToolTipText("Show junction coverage as an ellipse");
+            shapeGroup.add(ellipseShape);
+
+
+            ButtonGroup strandGroup = new ButtonGroup();
+
+            JRadioButtonMenuItem combineStrands = getStrandRadioButton("Combine Strands", SpliceJunctionFinderTrack.StrandOption.COMBINE);
+            combineStrands.setToolTipText("Combine junctions from both strands -- best for non-strand preserving libraries.");
+            strandGroup.add(combineStrands);
+
+            //  JRadioButtonMenuItem bothStrands = getStrandRadioButton("Both Strands", SpliceJunctionFinderTrack.StrandOption.BOTH);
+            //  strandGroup.add(bothStrands);
+            //  menu.add(bothStrands);
+
+            JRadioButtonMenuItem plusStrand = getStrandRadioButton("Forward Strand", SpliceJunctionFinderTrack.StrandOption.FORWARD);
+            plusStrand.setToolTipText("Show only junctions on the forward read strand  (of first-in-pair for paired reads)");
+            strandGroup.add(plusStrand);
+
+            JRadioButtonMenuItem minusStrand = getStrandRadioButton("Reverse Strand", SpliceJunctionFinderTrack.StrandOption.REVERSE);
+            plusStrand.setToolTipText("Show only junctions on the reverse read strand  (of first-in-pair for paired reads)");
+            strandGroup.add(minusStrand);
+
+
             JMenuItem saveImageItem = new JMenuItem("Save Image...");
             saveImageItem.addActionListener(new ActionListener() {
                 @Override
@@ -313,15 +404,69 @@ public class SashimiPlot extends JFrame {
                 }
             });
 
+            // Coverage ranges -- these apply to current plot only
+            menu.add(new JLabel("Junction Coverage Display"));
+            menu.add(setCoverageDataRange);
             menu.add(minJunctionCoverage);
             menu.add(maxJunctionCoverageRange);
             menu.add(colorItem);
+
+            // Coverage data  -- applies to all plots
+            menu.add(showCoverageData);
+
+            // Shape options -- all plots
+            menu.addSeparator();
+            menu.add(textShape);
+            menu.add(circleShape);
+           // menu.add(ellipseShape);
+            menu.add(noShape);
+
+            // Strand options -- applies to all plots
+            menu.addSeparator();
+            menu.add(combineStrands);
+            menu.add(plusStrand);
+            menu.add(minusStrand);
+            menu.addSeparator();
+
             menu.add(saveImageItem);
 
             return menu;
         }
+
+        private JRadioButtonMenuItem getJunctionCoverageRadioButton(String label, final SashimiJunctionRenderer.ShapeType shapeType) {
+
+
+            JRadioButtonMenuItem item = new JRadioButtonMenuItem(label);
+            item.setSelected(SashimiJunctionRenderer.getShapeType() == shapeType);
+            item.addActionListener(new ActionListener() {
+                @Override
+                public void actionPerformed(ActionEvent e) {
+                    SashimiJunctionRenderer.setShapeType(shapeType);
+                    SashimiPlot.this.repaint();
+                }
+            });
+            return item;
+        }
+
+        private JRadioButtonMenuItem getStrandRadioButton(String label, final SpliceJunctionFinderTrack.StrandOption option) {
+
+            JRadioButtonMenuItem item = new JRadioButtonMenuItem(label);
+            item.setSelected(SpliceJunctionFinderTrack.getStrandOption() == option);
+            item.addActionListener(new ActionListener() {
+                @Override
+                public void actionPerformed(ActionEvent e) {
+                    SpliceJunctionFinderTrack.setStrandOption(option);
+                    for (SpliceJunctionFinderTrack t : spliceJunctionTracks) {
+                        t.clear();
+                    }
+                    repaint();
+                }
+            });
+            return item;
+        }
     }
 
+
     private class GeneTrackMouseAdapter extends TrackComponentMouseAdapter<SelectableFeatureTrack> {
 
         GeneTrackMouseAdapter(TrackComponent<SelectableFeatureTrack> trackComponent) {
@@ -338,6 +483,11 @@ public class SashimiPlot extends JFrame {
             repaint();
         }
 
+        @Override
+        public void mouseMoved(MouseEvent e) {
+            trackComponent.updateToolTipText(createTrackClickEvent(e));
+        }
+
         @Override
         protected IGVPopupMenu getPopupMenu(MouseEvent e) {
             IGVPopupMenu menu = new IGVPopupMenu();
@@ -439,10 +589,17 @@ public class SashimiPlot extends JFrame {
     public static void getSashimiPlot(SashimiPlot sashimiPlot) {
         if (sashimiPlot == null) {
             FeatureTrack geneTrack = null;
-            if (IGV.getInstance().getFeatureTracks().size() == 1) {
-                geneTrack = IGV.getInstance().getFeatureTracks().get(0);
+
+            List<FeatureTrack> nonJunctionTracks = new ArrayList(IGV.getInstance().getFeatureTracks());
+            Iterator iter = nonJunctionTracks.iterator();
+            while(iter.hasNext()) {
+                if(iter.next() instanceof SpliceJunctionFinderTrack) iter.remove();
+            }
+
+            if (nonJunctionTracks.size() == 1) {
+                geneTrack = nonJunctionTracks.get(0);
             } else {
-                FeatureTrackSelectionDialog dlg = new FeatureTrackSelectionDialog(IGV.getMainFrame());
+                FeatureTrackSelectionDialog dlg = new FeatureTrackSelectionDialog(IGV.getMainFrame(), nonJunctionTracks);
                 dlg.setTitle("Select Gene Track");
                 dlg.setVisible(true);
                 if (dlg.getIsCancelled()) return;
@@ -457,7 +614,8 @@ public class SashimiPlot extends JFrame {
             }
 
             if (alignmentTracks.size() > 1) {
-                TrackSelectionDialog<AlignmentTrack> alDlg = new TrackSelectionDialog<AlignmentTrack>(IGV.getMainFrame(), TrackSelectionDialog.SelectionMode.MULTIPLE, alignmentTracks);
+                TrackSelectionDialog<AlignmentTrack> alDlg =
+                        new TrackSelectionDialog<AlignmentTrack>(IGV.getMainFrame(), TrackSelectionDialog.SelectionMode.MULTIPLE, alignmentTracks);
                 alDlg.setTitle("Select Alignment Tracks");
                 alDlg.setVisible(true);
                 if (alDlg.getIsCancelled()) return;
