@@ -17,6 +17,8 @@ package org.broad.igv.ui.action;
 import org.apache.log4j.Logger;
 import org.broad.igv.Globals;
 import org.broad.igv.PreferenceManager;
+import org.broad.igv.annotations.ForTesting;
+import org.broad.igv.dev.api.NamedFeatureSearcher;
 import org.broad.igv.feature.*;
 import org.broad.igv.feature.genome.Genome;
 import org.broad.igv.feature.genome.GenomeManager;
@@ -25,7 +27,7 @@ import org.broad.igv.ui.IGV;
 import org.broad.igv.ui.event.ViewChange;
 import org.broad.igv.ui.panel.ReferenceFrame;
 import org.broad.igv.ui.util.MessageUtils;
-import org.broad.tribble.Feature;
+import htsjdk.tribble.Feature;
 
 import javax.swing.*;
 import java.awt.*;
@@ -41,17 +43,8 @@ import java.util.List;
  * (b) a locus string in the UCSC form,  e.g. chr1:100,000-200,000
  * <p/>
  * Note:  Currently the only recognized features are genes
- *
- * @author jrobinso
- */
-
-/**
- * A class for performing search actions.  The class takes a view context and
- * search string as parameters.   The search string can be either
- * (a) a feature (e.g. gene),  or
- * (b) a locus string in the UCSC form,  e.g. chr1:100,000-200,000
  * <p/>
- * Note:  Currently the only recognized features are genes
+ * Custom searchers can be registered, see {@link #registerNamedFeatureSearcher(org.broad.igv.dev.api.NamedFeatureSearcher)}
  *
  * @author jrobinso
  */
@@ -67,6 +60,41 @@ public class SearchCommand {
     Genome genome;
 
 
+    private static Set<NamedFeatureSearcher> nameSearchers;
+
+    private static HashMap<ResultType, String> tokenMatchers;
+
+    static {
+        resetNamedFeatureSearchers();
+
+        //Regexp for a number with commas in it (no periods)
+        String num_withcommas = "(((\\d)+,?)+)";
+
+        //chromosome can include anything except whitespace
+        String chromo_string = "(\\S)+";
+
+        String chromo = chromo_string;
+        //This will match chr1:1-100, chr1:1, chr1  1, chr1 1   100
+        String chromo_range = chromo_string + "(:|(\\s)+)" + num_withcommas + "(-|(\\s)+)?" + num_withcommas + "?(\\s)*";
+
+        //Simple feature
+        String feature = chromo_string;
+        //Amino acid mutation notation. e.g. KRAS:G12C. * is stop codon
+        String featureMutAA = chromo_string + ":[A-Z,a-z,*]" + num_withcommas + "[A-Z,a-z,*]";
+
+        //Nucleotide mutation notation. e.g. KRAS:123A>T
+        String nts = "[A,C,G,T,a,c,g,t]";
+        String featureMutNT = chromo_string + ":" + num_withcommas + nts + "\\>" + nts;
+
+        tokenMatchers = new HashMap<ResultType, String>();
+        tokenMatchers.put(ResultType.CHROMOSOME, chromo);
+        tokenMatchers.put(ResultType.FEATURE, feature);
+        tokenMatchers.put(ResultType.LOCUS, chromo_range);
+        tokenMatchers.put(ResultType.FEATURE_MUT_AA, featureMutAA);
+        tokenMatchers.put(ResultType.FEATURE_MUT_NT, featureMutNT);
+    }
+
+
     public SearchCommand(ReferenceFrame referenceFrame, String searchString) {
         this(referenceFrame, searchString, GenomeManager.getInstance().getCurrentGenome());
     }
@@ -152,17 +180,18 @@ public class SearchCommand {
 
     public void showSearchResult(List<SearchResult> results) {
         int origZoom = referenceFrame.getZoom();
-        SearchResult result = new SearchResult();
         if (results == null || results.size() == 0) {
             results = new ArrayList<SearchResult>();
-            results.add(result);
+
+            results.add(new SearchResult());
         }
+
         boolean showMessage = false;
         boolean success = true;
         String message = "Invalid search string: " + searchString;
 
         if (results.size() == 1) {
-            result = results.get(0);
+            SearchResult result = results.get(0);
             if (result.type != ResultType.ERROR) {//FrameManager.isGeneListMode()) {
                 IGV.getInstance().getSession().setCurrentGeneList(null);
             }
@@ -295,35 +324,10 @@ public class SearchCommand {
     Set<ResultType> checkTokenType(String token) {
         token = token.trim();
 
-        //Regexp for a number with commas in it (no periods)
-        String num_withcommas = "(((\\d)+,?)+)";
-
-        //chromosome can include anything except whitespace
-        String chromo_string = "(\\S)+";
-
-
-        String chromo = chromo_string;
-        //This will match chr1:1-100, chr1:1, chr1  1, chr1 1   100
-        String chromo_range = chromo_string + "(:|(\\s)+)" + num_withcommas + "(-|(\\s)+)?" + num_withcommas + "?(\\s)*";
-
-        //Simple feature
-        String feature = chromo_string;
-        //Amino acid mutation notation. e.g. KRAS:G12C. * is stop codon
-        String featureMutAA = chromo_string + ":[A-Z,a-z,*]" + num_withcommas + "[A-Z,a-z,*]";
-
-        //Nucleotide mutation notation. e.g. KRAS:123A>T
-        String nts = "[A,C,G,T,a,c,g,t]";
-        String featureMutNT = chromo_string + ":" + num_withcommas + nts + "\\>" + nts;
 
         Set<ResultType> possibles = new HashSet<ResultType>();
-        Map<ResultType, String> matchers = new HashMap<ResultType, String>();
-        matchers.put(ResultType.CHROMOSOME, chromo);
-        matchers.put(ResultType.FEATURE, feature);
-        matchers.put(ResultType.LOCUS, chromo_range);
-        matchers.put(ResultType.FEATURE_MUT_AA, featureMutAA);
-        matchers.put(ResultType.FEATURE_MUT_NT, featureMutNT);
-        for (ResultType type : matchers.keySet()) {
-            if (token.matches(matchers.get(type))) { //note: entire string must match
+        for (ResultType type : tokenMatchers.keySet()) {
+            if (token.matches(tokenMatchers.get(type))) { //note: entire string must match
                 possibles.add(type);
             }
         }
@@ -402,7 +406,7 @@ public class SearchCommand {
         }
 
         if (types.contains(ResultType.FEATURE)) {
-            //Check if a feature
+            //Check if we have an exact name for the feature name
             NamedFeature feat = FeatureDB.getFeature(token.toUpperCase().trim());
             if (feat != null) {
                 results.add(new SearchResult(feat));
@@ -411,7 +415,7 @@ public class SearchCommand {
 
             //Check inexact match
             //We will later want to ask the user which of these to keep
-            features = FeatureDB.getFeaturesList(searchString, SEARCH_LIMIT);
+            features = comprehensiveFeatureSearch(token);
             if (features.size() > 0) {
                 askUser |= features.size() >= 2;
                 return getResults(features);
@@ -425,6 +429,54 @@ public class SearchCommand {
 
     }
 
+    /**
+     * Register a {@link org.broad.igv.dev.api.NamedFeatureSearcher} to be used when searching for features,
+     * such as when the user enters text in the box.
+     * This is idempotent, registering the same searcher multiple times is the same as
+     * adding it once.
+     *
+     * @param searcher
+     * @return Whether the searcher was added
+     * @api
+     */
+    public static boolean registerNamedFeatureSearcher(NamedFeatureSearcher searcher) {
+        return nameSearchers.add(searcher);
+    }
+
+    /**
+     * @param searcher
+     * @return Whether the searcher was removed
+     * @api
+     */
+    public static boolean unregisterNamedFeatureSearcher(NamedFeatureSearcher searcher) {
+        return nameSearchers.remove(searcher);
+    }
+
+    static void resetNamedFeatureSearchers() {
+        nameSearchers = new LinkedHashSet<NamedFeatureSearcher>();
+        registerNamedFeatureSearcher(new InexactLoadedFeatureSearcher());
+    }
+
+    /**
+     * Search all known sources for features with the provided name.
+     * This means our own database which gets updated when files are loaded,
+     * as well as others (possibly plugins)
+     *
+     * @param searchString
+     * @return
+     */
+    private List<NamedFeature> comprehensiveFeatureSearch(String searchString) {
+        List<NamedFeature> features = new ArrayList<NamedFeature>();
+        for (NamedFeatureSearcher searcher : nameSearchers) {
+            Collection<? extends NamedFeature> tmp = searcher.search(searchString, SEARCH_LIMIT);
+            if (tmp == null) {
+                log.warn("Error searching with " + searcher);
+            } else {
+                features.addAll(tmp);
+            }
+        }
+        return features;
+    }
 
     /**
      * Parse a string of locus coordinates.
@@ -482,7 +534,14 @@ public class SearchCommand {
 
         if (chromosome != null && !searchString.equals(Globals.CHR_ALL)) {
             if (startEnd != null) {
-                return new SearchResult(ResultType.LOCUS, chr, startEnd[0], startEnd[1]);
+                if(startEnd[1] >= startEnd[0]) {
+                    return new SearchResult(ResultType.LOCUS, chr, startEnd[0], startEnd[1]);
+                }
+                else {
+                    SearchResult error = new SearchResult(ResultType.ERROR, chr, startEnd[0], startEnd[1]);
+                    error.setMessage("End must be greater than start");
+                    return error;
+                }
             }
             return new SearchResult(ResultType.CHROMOSOME, chr, 0, chromosome.getLength() - 1);
         }
@@ -491,8 +550,17 @@ public class SearchCommand {
 
     private void showFlankedRegion(String chr, int start, int end) {
         int flankingRegion = PreferenceManager.getInstance().getAsInt(PreferenceManager.FLANKING_REGION);
-        start = Math.max(0, start - flankingRegion);
-        end = end + flankingRegion;
+        int delta;
+        if((end - start) == 1) {
+            delta = 20; // Don't show flanking region for single base jumps, use 40bp window
+        }
+        else if (flankingRegion < 0) {
+            delta = (-flankingRegion * (end - start)) / 100;
+        } else {
+            delta = flankingRegion;
+        }
+        start = Math.max(0, start - delta);
+        end = end + delta;
 
         if (PreferenceManager.getInstance().getAsBoolean(PreferenceManager.SEARCH_ZOOM)) {
             referenceFrame.jumpTo(chr, start, end);
@@ -513,7 +581,7 @@ public class SearchCommand {
         try {
             String[] posTokens = posString.split("-");
             String startString = posTokens[0].replaceAll(",", "");
-            int start = Math.max(0, Integer.parseInt(startString)) - 1;
+            int start = Math.max(0, Integer.parseInt(startString));
 
             // Default value for end
             int end = start + 1;
@@ -522,7 +590,7 @@ public class SearchCommand {
                 end = Integer.parseInt(endString);
             }
 
-            if (posTokens.length == 1 || (end - start) < 10) {
+            if (posTokens.length == 1 || (end >= start && (end - start) < 10)) {
                 int center = (start + end) / 2;
                 int widen = 20;
                 start = center - widen;
@@ -530,7 +598,7 @@ public class SearchCommand {
                 end = center + widen + 1;
             }
 
-            return new int[]{Math.min(start, end), Math.max(start, end)};
+            return new int[]{Math.min(start, end)-1, Math.max(start, end)};
         } catch (NumberFormatException numberFormatException) {
             return null;
         }
@@ -546,6 +614,13 @@ public class SearchCommand {
         ERROR
     }
 
+    private static class InexactLoadedFeatureSearcher implements NamedFeatureSearcher {
+        @Override
+        public Collection<NamedFeature> search(String name, int limit) {
+            return FeatureDB.getFeaturesList(name, limit);
+        }
+    }
+
     /*
     Container class for search results
      */
@@ -646,6 +721,12 @@ public class SearchCommand {
         public int getEnd() {
             return end;
         }
+
+        //May be null
+        @ForTesting
+        public NamedFeature getFeature() {
+            return feature;
+        }
     }
 
     /**
