@@ -20,9 +20,9 @@ import com.google.common.eventbus.Subscribe;
 import org.apache.log4j.Logger;
 import org.broad.igv.Globals;
 import org.broad.igv.PreferenceManager;
-import org.broad.igv.data.Interval;
 import org.broad.igv.feature.Chromosome;
 import org.broad.igv.feature.Locus;
+import org.broad.igv.feature.Range;
 import org.broad.igv.feature.genome.Genome;
 import org.broad.igv.feature.genome.GenomeManager;
 import org.broad.igv.ui.IGV;
@@ -39,6 +39,8 @@ public class ReferenceFrame {
 
     private static Logger log = Logger.getLogger(ReferenceFrame.class);
 
+    boolean visible = true;
+
     /**
      * The nominal viewport width in pixels.
      */
@@ -63,10 +65,9 @@ public class ReferenceFrame {
     public int maxZoom = 23;
 
     /**
-     * Minimum allowed scale in base pairs / pixel
-     * TODO Does this ever change, couldn't it be static and/or final?
+     * Minimum allowed range in base-pairs
      */
-    private double minScale = 1.0 / 14;
+    protected static final int minBP = 40;
 
     /**
      * The current zoom level.  Zoom level -1 corresponds to the whole
@@ -124,7 +125,6 @@ public class ReferenceFrame {
         this.chrName = otherFrame.chrName;
         this.initialLocus = otherFrame.initialLocus;
         this.locationScale = otherFrame.locationScale;
-        this.minScale = otherFrame.minScale;
         this.minZoom = otherFrame.minZoom;
         this.name = otherFrame.name;
         this.nTiles = otherFrame.nTiles;
@@ -133,6 +133,7 @@ public class ReferenceFrame {
         //this.setEnd = otherFrame.setEnd;
         this.widthInPixels = otherFrame.widthInPixels;
         this.zoom = otherFrame.zoom;
+        this.maxZoom = otherFrame.maxZoom;
         registerEventBuses();
     }
 
@@ -153,6 +154,14 @@ public class ReferenceFrame {
         return eventBus;
     }
 
+    public boolean isVisible() {
+        return visible;
+    }
+
+    public void setVisible(boolean visible) {
+        this.visible = visible;
+    }
+
     /**
      * Set the position and width of the frame, in pixels
      * The origin/end positions are kept fixed iff valid
@@ -207,17 +216,15 @@ public class ReferenceFrame {
         int windowLengthBP = (int) (widthInPixels * getScale());
         double newOrigin;
         if (PreferenceManager.getInstance().getAsBoolean(PreferenceManager.SAM_SHOW_SOFT_CLIPPED)) {
-            newOrigin = Math.max(-1000, Math.min(position, getChromosomeLength() + 1000 - windowLengthBP));
+            newOrigin = Math.max(-1000, Math.min(position, getMaxCoordinate() + 1000 - windowLengthBP));
         } else {
-            newOrigin = Math.max(0, Math.min(position, getChromosomeLength() - windowLengthBP));
+            newOrigin = Math.max(0, Math.min(position, getMaxCoordinate() - windowLengthBP));
         }
         origin = newOrigin;
     }
 
 
-    private synchronized void setZoomWithinLimits(int newZoom) {
-        computeMinZoom();
-        computeMaxZoom();
+    protected synchronized void setZoomWithinLimits(int newZoom) {
         zoom = Math.max(minZoom, Math.min(maxZoom, newZoom));
         nTiles = Math.pow(2, zoom);
     }
@@ -269,7 +276,7 @@ public class ReferenceFrame {
      * and center location
      *
      * @param newZoom
-     * @param newCenter
+     * @param newCenter Center position, in genome coordinates
      */
     public void doSetZoomCenter(final int newZoom, final double newCenter) {
 
@@ -333,6 +340,7 @@ public class ReferenceFrame {
             chrName = name;
             origin = 0;
             this.locationScale = -1;
+            this.calculateMaxZoom();
 
             this.zoom = -1;
             setZoom(0);
@@ -349,7 +357,7 @@ public class ReferenceFrame {
      * {@link #widthInPixels}
      * DOES NOT alter zoom value
      */
-    private synchronized void computeLocationScale() {
+    protected synchronized void computeLocationScale() {
         Genome genome = getGenome();
 
         //Should consider getting rid of this. We don't have
@@ -376,27 +384,11 @@ public class ReferenceFrame {
      * Recalculate the zoom value based on current start/end
      * locationScale is not altered
      */
-    private void computeZoom() {
+    protected void computeZoom() {
         int newZoom = calculateZoom(getOrigin(), getEnd());
         setZoomWithinLimits(newZoom);
     }
 
-    /**
-     * Compute the minimum zoom level for the data panel width.  This is defined as the maximum
-     * zoom level for which all data bins will fit in the window without loss of
-     * data,  i.e. the maximum zoom level for which nBins < nPixels.  The number
-     * of bins is defined as
-     * nBins =  2^z
-     * so minZoom is the value z such that nBins < dataPanelWidth
-     */
-    protected void computeMinZoom() {
-        if (this.chrName.equals(Globals.CHR_ALL)) {
-            minZoom = 0;
-        } else {
-            minZoom = Math.max(0, (int) (Math.log((widthInPixels / binsPerTile)) / Globals.log2));
-        }
-    }
-
     /**
      * Record the current state of the frame in history.
      * It is recommended that this NOT be called from within ReferenceFrame,
@@ -444,12 +436,12 @@ public class ReferenceFrame {
     public void centerOnLocation(double chrLocation) {
         double windowWidth = (widthInPixels * getScale()) / 2;
         setOrigin(Math.round(chrLocation - windowWidth));
-        getEventBus().post(new ViewChange.Result());
+        getEventBus().post(new ViewChange.LocusChangeResult(chrName, origin, chrLocation + windowWidth));
     }
 
     public boolean windowAtEnd() {
         double windowLengthBP = widthInPixels * getScale();
-        return origin + windowLengthBP + 1 > getChromosomeLength();
+        return origin + windowLengthBP + 1 > getMaxCoordinate();
     }
 
     /**
@@ -478,18 +470,16 @@ public class ReferenceFrame {
             }
         }
 
-        Chromosome chromosome = genome == null ? null : genome.getChromosome(chr);
-        if (chromosome != null) {
-            end = Math.min(chromosome.getLength(), end);
-        }
+        end = Math.min(getMaxCoordinate(chr), end);
 
         synchronized (this) {
             this.initialLocus = locus;
             this.chrName = chr;
             if (start >= 0 && end >= 0) {
                 this.origin = start;
+                beforeScaleZoom(locus);
                 computeLocationScale();
-                setZoomWithinLimits(calculateZoom(this.getOrigin(), this.getEnd()));
+                computeZoom();
             }
         }
 
@@ -501,24 +491,16 @@ public class ReferenceFrame {
             log.debug("Scale = " + locationScale);
         }
 
-        getEventBus().post(new ViewChange.Result());
+        getEventBus().post(new ViewChange.LocusChangeResult(chrName, start, end));
     }
 
-
     /**
-     * Compute the maximum zoom level, which is a function of chromosome length.
+     * Called before scaling and zooming, during jumpTo.
+     * Intended to be overridden
+     * @param locus
      */
-    private void computeMaxZoom() {
-        Genome genome = getGenome();
-        // Compute max zoom.  Assume window size @ max zoom of ~ 50 bp
-        if (genome != null && chrName != null && genome.getChromosome(chrName) != null) {
-            if (chrName.equals(Globals.CHR_ALL)) {
-                maxZoom = 0;
-            } else {
-                int chrLength = genome.getChromosome(chrName).getLength();
-                maxZoom = (int) (Math.log(chrLength / 50.0) / Globals.log2) + 1;
-            }
-        }
+    protected void beforeScaleZoom(Locus locus) {
+        calculateMaxZoom();
     }
 
     /**
@@ -530,10 +512,10 @@ public class ReferenceFrame {
      * @return
      */
     protected int calculateZoom(double start, double end) {
-        return (int) (Math.log(getChromosomeLength() / (end - start)) / Globals.log2) + 1;
+        return (int) Math.round((Math.log((getChromosomeLength() / (end - start)) * (((double) widthInPixels) / binsPerTile)) / Globals.log2));
     }
 
-    protected Genome getGenome() {
+    protected static Genome getGenome() {
         return GenomeManager.getInstance().getCurrentGenome();
     }
 
@@ -596,6 +578,11 @@ public class ReferenceFrame {
         }
     }
 
+    protected void calculateMaxZoom() {
+        this.maxZoom = Globals.CHR_ALL.equals(this.chrName) ? 0 :
+                (int) Math.ceil(Math.log(getChromosomeLength() / minBP) / Globals.log2);
+    }
+
     public String getChrName() {
         return chrName;
     }
@@ -638,8 +625,33 @@ public class ReferenceFrame {
         return genome.getChromosome(chrName);
     }
 
+    /**
+     * The maximum coordinate currently allowed.
+     * In genomic coordinates this is the same as the chromosome length.
+     * In exome coordinates, the two are different
+     * (since ExomeReferenceFrame takes input in genomic coordinates)
+     * @see #getChromosomeLength()
+     * @return
+     */
+    public int getMaxCoordinate(){
+        return this.getChromosomeLength();
+    }
 
-    public int getChromosomeLength() {
+    private static int getMaxCoordinate(String chrName){
+        return getChromosomeLength(chrName);
+    }
+
+    /**
+     * Chromosome length, in genomic coordinates.
+     * Intended to be used for scaling
+     * @see #getMaxCoordinate()
+     * @return
+     */
+    public int getChromosomeLength(){
+        return getChromosomeLength(this.chrName);
+    }
+
+    private static int getChromosomeLength(String chrName) {
         Genome genome = getGenome();
 
         if (genome == null) {
@@ -647,22 +659,19 @@ public class ReferenceFrame {
         }
 
         if (chrName.equals("All")) {
-
             // TODO -- remove the hardcoded unit divider ("1000")
             return (int) (genome.getNominalLength() / 1000);
-
-            // return genome.getLength();
         } else {
-            if (getChromosome() == null) {
+            Chromosome chromosome = genome.getChromosome(chrName);
+            if (chromosome == null) {
                 log.error("Null chromosome: " + chrName);
-                if (genome == null || genome.getChromosomes().size() == 0) {
+                if (genome.getChromosomes().size() == 0) {
                     return 1;
                 } else {
                     return genome.getChromosomes().iterator().next().getLength();
                 }
             }
-
-            return getChromosome().getLength();
+            return chromosome.getLength();
         }
     }
 
@@ -730,43 +739,4 @@ public class ReferenceFrame {
         this.name = name;
     }
 
-    public boolean overlaps(Interval interval) {
-        return this.getChrName().equals(interval.getChr()) && this.getOrigin() <= interval.getEnd() && this.getEnd() >= interval.getStart();
-    }
-
-    /**
-     * Return the current range as a locus string.
-     */
-    public String getLocusString() {
-        return Locus.getFormattedLocusString( chrName , (int) origin, (int) this.getEnd());
-    }
-
-    public static class Range {
-        private String chr;
-        private int start;
-        private int end;
-
-        public Range(String chr, int start, int end) {
-            this.chr = chr;
-            this.start = start;
-            this.end = end;
-        }
-
-        public String getChr() {
-            return chr;
-        }
-
-        public int getStart() {
-            return start;
-        }
-
-        public int getEnd() {
-            return end;
-        }
-
-        public int getLength() {
-            return end - start;
-        }
-    }
-
 }
