@@ -223,11 +223,11 @@ public class RulerPanel extends JPanel {
         // Find starting point closest to the current origin
         int nTick = (int) (frame.getOrigin() / spacing) - 1;
         int l = (int) (nTick * spacing);
-        int x = frame.getScreenPosition(l - 1);    // 0 vs 1 based coordinates
+        int x = frame.getScreenPosition(l - 1 + 0.5);    // 0 vs 1 based coordinates, then center over base
         //int strEnd = Integer.MIN_VALUE;
         while (x < getWidth()) {
             l = (int) (nTick * spacing);
-            x = frame.getScreenPosition(l - 1);
+            x = frame.getScreenPosition(l - 1 + 0.5);
             String chrPosition = formatNumber((double) l / ts.getUnitMultiplier()) +
                     " " + ts.getMajorUnit();
             int strWidth = g.getFontMetrics().stringWidth(chrPosition);
@@ -257,8 +257,9 @@ public class RulerPanel extends JPanel {
 
         Genome genome = GenomeManager.getInstance().getCurrentGenome();
         if (genome == null) {
-            log.info("No genome found with id: " + genome.getId());
+            log.warn("No genome found");
             PreferenceManager.getInstance().remove(PreferenceManager.DEFAULT_GENOME_KEY);
+            return;
         }
 
         boolean even = true;
@@ -266,7 +267,7 @@ public class RulerPanel extends JPanel {
         chromosomeRects.clear();
         List<String> chrNames = genome.getLongChromosomeNames();
         if (chrNames == null) {
-            log.info("No chromosomes found for genome: " + genome.getId());
+            log.info("No chromosomes found for genome: " + PreferenceManager.getInstance().getDefaultGenome());
             PreferenceManager.getInstance().remove(PreferenceManager.DEFAULT_GENOME_KEY);
         }
         if (chrNames.size() > 500) {
@@ -356,7 +357,6 @@ public class RulerPanel extends JPanel {
 
                 Graphics2D exomeGraphics = (Graphics2D) g.create();
 
-//Shape clip = exomeGraphics.getClip();
 
                 Color c = idx % 2 == 0 ? grey1 : grey2;
 
@@ -384,6 +384,9 @@ public class RulerPanel extends JPanel {
         List<ExomeReferenceFrame.Gene> genes = frame.getGenes(chr);
 
         int idx = FeatureUtils.getIndexBefore(frame.getOrigin(), genes);
+
+        if(idx < 0) idx = 0;
+
         Rectangle visibleRect = this.getVisibleRect();
         FontMetrics fm = g.getFontMetrics();
 
