@@ -12,25 +12,28 @@
 package org.broad.igv.util;
 
 import biz.source_code.base64Coder.Base64Coder;
-import net.sf.samtools.seekablestream.SeekableStream;
-import net.sf.samtools.util.ftp.FTPClient;
-import net.sf.samtools.util.ftp.FTPStream;
-import net.sf.samtools.util.ftp.FTPUtils;
+import htsjdk.samtools.seekablestream.SeekableStream;
+import htsjdk.samtools.util.ftp.FTPClient;
+import htsjdk.samtools.util.ftp.FTPStream;
 import org.apache.log4j.Logger;
 import org.apache.tomcat.util.HttpDate;
 import org.broad.igv.Globals;
 import org.broad.igv.PreferenceManager;
 import org.broad.igv.exceptions.HttpResponseException;
+import org.broad.igv.ga4gh.OAuthUtils;
 import org.broad.igv.gs.GSUtils;
 import org.broad.igv.ui.IGV;
+import org.broad.igv.ui.util.CancellableProgressDialog;
+import org.broad.igv.ui.util.ProgressMonitor;
+import org.broad.igv.util.collections.CI;
+import org.broad.igv.util.ftp.FTPUtils;
 import org.broad.igv.util.stream.IGVSeekableHTTPStream;
 import org.broad.igv.util.stream.IGVUrlHelper;
 
-import javax.net.ssl.HttpsURLConnection;
-import javax.net.ssl.SSLContext;
-import javax.net.ssl.TrustManager;
-import javax.net.ssl.X509TrustManager;
+import javax.net.ssl.*;
 import java.awt.*;
+import java.awt.event.ActionEvent;
+import java.awt.event.ActionListener;
 import java.io.*;
 import java.net.*;
 import java.security.KeyManagementException;
@@ -38,7 +41,6 @@ import java.security.NoSuchAlgorithmException;
 import java.util.*;
 import java.util.List;
 import java.util.regex.Pattern;
-import java.util.zip.GZIPInputStream;
 
 /**
  * Wrapper utility class... for interacting with HttpURLConnection.
@@ -63,6 +65,7 @@ public class HttpUtils {
 
     // static provided to support unit testing
     private static boolean BYTE_RANGE_DISABLED = false;
+    private Map<URL, Boolean> headURLCache = new HashMap<URL, Boolean>();
 
     /**
      * @return the single instance
@@ -76,12 +79,20 @@ public class HttpUtils {
 
     private HttpUtils() {
 
-        org.broad.tribble.util.ParsingUtils.registerHelperClass(IGVUrlHelper.class);
+        htsjdk.tribble.util.ParsingUtils.registerHelperClass(IGVUrlHelper.class);
 
+        // if (!Globals.checkJavaVersion("1.8")) {
         disableCertificateValidation();
+        // }
         CookieHandler.setDefault(new IGVCookieManager());
         Authenticator.setDefault(new IGVAuthenticator());
 
+        try {
+            System.setProperty("java.net.useSystemProxies", "true");
+        } catch (Exception e) {
+            log.info("Couldn't set useSystemProxies=true");
+        }
+
         byteRangeTestMap = Collections.synchronizedMap(new HashMap());
     }
 
@@ -99,41 +110,6 @@ public class HttpUtils {
         BYTE_RANGE_DISABLED = b;
     }
 
-
-    /**
-     * @param elements
-     * @param joiner
-     * @return
-     * @deprecated HttpUtils.openConnection does URL encoding itself
-     *             <p/>
-     *             Join the {@code elements} with the character {@code joiner},
-     *             URLencoding the {@code elements} along the way. {@code joiner}
-     *             is NOT URLEncoded
-     *             Example:
-     *             String[] parm_list = new String[]{"app les", "oranges", "bananas"};
-     *             String formatted = buildURLString(Arrays.asList(parm_list), "+");
-     *             <p/>
-     *             formatted will be "app%20les+oranges+bananas"
-     */
-    @Deprecated
-    public static String buildURLString(Iterable<String> elements, String joiner) {
-
-        Iterator<String> iter = elements.iterator();
-        if (!iter.hasNext()) {
-            return "";
-        }
-        String wholequery = iter.next();
-        try {
-            while (iter.hasNext()) {
-                wholequery += joiner + URLEncoder.encode(iter.next(), "UTF-8");
-            }
-            return wholequery;
-        } catch (UnsupportedEncodingException e) {
-            throw new IllegalArgumentException("Bad argument in genelist: " + e.getMessage());
-        }
-    }
-
-
     /**
      * Return the contents of the url as a String.  This method should only be used for queries expected to return
      * a small amount of data.
@@ -149,7 +125,9 @@ public class HttpUtils {
         try {
             is = conn.getInputStream();
             return readContents(is);
-
+        } catch (IOException e) {
+            readErrorStream(conn);  // Consume content
+            throw e;
         } finally {
             if (is != null) is.close();
         }
@@ -165,11 +143,41 @@ public class HttpUtils {
             is = conn.getInputStream();
             return readContents(is);
 
+        } catch (IOException e) {
+            readErrorStream(conn);  // Consume content
+            throw e;
         } finally {
             if (is != null) is.close();
         }
     }
 
+    public String doPost(URL url, Map<String, String> params) throws IOException {
+
+        StringBuilder postData = new StringBuilder();
+
+        for (Map.Entry<String, String> param : params.entrySet()) {
+            if (postData.length() != 0) postData.append('&');
+            postData.append(param.getKey());
+            postData.append('=');
+            postData.append(param.getValue());
+        }
+        byte[] postDataBytes = postData.toString().getBytes();
+
+        HttpURLConnection conn = (HttpURLConnection) url.openConnection();
+        conn.setRequestMethod("POST");
+        conn.setRequestProperty("Content-Type", "application/x-www-form-urlencoded");
+        conn.setDoOutput(true);
+        conn.getOutputStream().write(postDataBytes);
+
+        StringBuilder response = new StringBuilder();
+        Reader in = new BufferedReader(new InputStreamReader(conn.getInputStream(), "UTF-8"));
+        for (int c; (c = in.read()) >= 0; ) {
+            response.append((char) c);
+        }
+        return response.toString();
+
+    }
+
     /**
      * Open a connection stream for the URL.
      *
@@ -196,63 +204,114 @@ public class HttpUtils {
     public InputStream openConnectionStream(URL url, Map<String, String> requestProperties) throws IOException {
 
         HttpURLConnection conn = openConnection(url, requestProperties);
-        if (conn == null)
+        if (conn == null) {
             return null;
-        InputStream input = conn.getInputStream();
-        if ("gzip".equals(conn.getContentEncoding())) {
-            input = new GZIPInputStream(input);
         }
-        return input;
-    }
 
+        if ((requestProperties != null) && requestProperties.containsKey("Range") && conn.getResponseCode() != 206) {
+            String msg = "Warning: range requested, but response code = " + conn.getResponseCode();
+            log.error(msg);
+        }
+
+        try {
+            InputStream input = conn.getInputStream();
+            return input;
+        } catch (IOException e) {
+            readErrorStream(conn);  // Consume content
+            throw e;
+        }
+    }
 
     public boolean resourceAvailable(URL url) {
         log.debug("Checking if resource is available: " + url);
         if (url.getProtocol().toLowerCase().equals("ftp")) {
             return FTPUtils.resourceAvailable(url);
-        }
-
-        try {
-            HttpURLConnection conn = openConnection(url, null, "HEAD");
-            int code = conn.getResponseCode();
-            return code == 200;
-        } catch (IOException e) {
-            return false;
+        } else {
+            HttpURLConnection conn = null;
+            try {
+                conn = openConnectionHeadOrGet(url);
+                int code = conn.getResponseCode();
+                return code >= 200 && code < 300;
+            } catch (Exception e) {
+                if (conn != null)
+                    try {
+                        readErrorStream(conn);  // Consume content
+                    } catch (IOException e1) {
+                        e1.printStackTrace();
+                    }
+                return false;
+            }
         }
     }
 
+    /**
+     * First tries a HEAD request, then a GET request if the HEAD fails.
+     * If the GET fails, the exception is thrown
+     *
+     * @param url
+     * @return
+     * @throws IOException
+     */
+    private HttpURLConnection openConnectionHeadOrGet(URL url) throws IOException {
+
+        // Keep track of urls for which "HEAD" does not work (e.g. Amazon with signed urls).
+        boolean tryHead = headURLCache.containsKey(url) ? headURLCache.get(url) : true;
+
+        if (tryHead) {
+            try {
+                HttpURLConnection conn = openConnection(url, null, "HEAD");
+                headURLCache.put(url, true);
+                return conn;
+            } catch (IOException e) {
+                if (e instanceof FileNotFoundException) {
+                    throw e;
+                }
+                log.info("HEAD request failed for url: " + url.toExternalForm() + ".  Trying GET");
+                headURLCache.put(url, false);
+            }
+        }
+        return openConnection(url, null, "GET");
+    }
+
     public String getHeaderField(URL url, String key) throws IOException {
-        HttpURLConnection conn = openConnection(url, null, "HEAD");
+        HttpURLConnection conn = openConnectionHeadOrGet(url);
         if (conn == null) return null;
         return conn.getHeaderField(key);
     }
 
     public long getLastModified(URL url) throws IOException {
-        HttpURLConnection conn = openConnection(url, null, "HEAD");
+        HttpURLConnection conn = openConnectionHeadOrGet(url);
         if (conn == null) return 0;
         return conn.getLastModified();
     }
 
     public long getContentLength(URL url) throws IOException {
 
-        String contentLengthString = getHeaderField(url, "Content-Length");
-        if (contentLengthString == null) {
+        try {
+            String contentLengthString = getHeaderField(url, "Content-Length");
+            if (contentLengthString == null) {
+                return -1;
+            } else {
+                return Long.parseLong(contentLengthString);
+            }
+        } catch (Exception e) {
+            log.error("Error fetching content length", e);
             return -1;
-        } else {
-            return Long.parseLong(contentLengthString);
         }
     }
 
     /**
-     * Compare a local and remote resource.
+     * Compare a local and remote resource, returning true if it is believed that the
+     * remote file is newer than the local file
      *
      * @param file
      * @param url
-     * @return true if the files are "the same", false if the remote file has been modified wrt the local one.
+     * @param compareContentLength Whether to use the content length to compare files. If false, only
+     *                             the modified date is used
+     * @return true if the files are the same or the local file is newer, false if the remote file has been modified wrt the local one.
      * @throws IOException
      */
-    public boolean compareResources(File file, URL url) throws IOException {
-
+    public boolean remoteIsNewer(File file, URL url, boolean compareContentLength) throws IOException {
 
         if (!file.exists()) {
             return false;
@@ -273,7 +332,7 @@ public class HttpUtils {
             }
         }
         if (contentLength != file.length()) {
-            return false;
+            return true;
         }
 
         // Compare last-modified dates
@@ -285,7 +344,7 @@ public class HttpUtils {
             date.parse(lastModifiedString);
             long remoteModifiedTime = date.getTime();
             long localModifiedTime = file.lastModified();
-            return remoteModifiedTime <= localModifiedTime;
+            return remoteModifiedTime > localModifiedTime;
         }
 
 
@@ -316,9 +375,13 @@ public class HttpUtils {
         }
 
         String proxyTypeString = prefMgr.get(PreferenceManager.PROXY_TYPE, "HTTP");
-        Proxy.Type type = proxyTypeString.equals("SOCKS") ? Proxy.Type.SOCKS : Proxy.Type.HTTP;
+        Proxy.Type type = Proxy.Type.valueOf(proxyTypeString.trim().toUpperCase());
 
-        proxySettings = new ProxySettings(useProxy, user, pw, auth, proxyHost, proxyPort, type);
+        String proxyWhitelistString = prefMgr.get(PreferenceManager.PROXY_WHITELIST);
+        Set<String> whitelist = proxyWhitelistString == null ? new HashSet<String>() :
+                new HashSet(Arrays.asList(Globals.commaPattern.split(proxyWhitelistString)));
+
+        proxySettings = new ProxySettings(useProxy, user, pw, auth, proxyHost, proxyPort, type, whitelist);
     }
 
     /**
@@ -331,6 +394,7 @@ public class HttpUtils {
      */
     private Proxy getSystemProxy(String uri) {
         try {
+            log.debug("Getting system proxy for " + uri);
             ProxySelector selector = ProxySelector.getDefault();
             List<Proxy> proxyList = selector.select(new URI(uri));
             return proxyList.get(0);
@@ -339,50 +403,63 @@ public class HttpUtils {
             return null;
         } catch (NullPointerException e) {
             return null;
+        } catch (Exception e) {
+            log.error(e.getMessage(), e);
+            return null;
         }
 
     }
 
-    public boolean downloadFile(String url, File outputFile) throws IOException {
 
-        log.info("Downloading " + url + " to " + outputFile.getAbsolutePath());
+    /**
+     * Calls {@link #downloadFile(String, java.io.File, Frame, String)}
+     * with {@code dialogsParent = null, title = null}
+     *
+     * @param url
+     * @param outputFile
+     * @return RunnableResult
+     * @throws IOException
+     */
+    public RunnableResult downloadFile(String url, File outputFile) throws IOException {
+        URLDownloader downloader = downloadFile(url, outputFile, null, null);
+        return downloader.getResult();
+    }
 
-        HttpURLConnection conn = openConnection(new URL(url), null);
+    /**
+     * @param url
+     * @param outputFile
+     * @param dialogsParent Parent of dialog to show progress. If null, none shown
+     * @return URLDownloader used to perform download
+     * @throws IOException
+     */
+    public URLDownloader downloadFile(String url, File outputFile, Frame dialogsParent, String dialogTitle) throws IOException {
+        final URLDownloader urlDownloader = new URLDownloader(url, outputFile);
+        boolean showProgressDialog = dialogsParent != null;
+        if (!showProgressDialog) {
+            urlDownloader.run();
+            return urlDownloader;
+        } else {
+            ProgressMonitor monitor = new ProgressMonitor();
+            urlDownloader.setMonitor(monitor);
+            ActionListener buttonListener = new ActionListener() {
+                @Override
+                public void actionPerformed(ActionEvent e) {
+                    urlDownloader.cancel(true);
+                }
+            };
+            String permText = "Downloading " + url;
+            String title = dialogTitle != null ? dialogTitle : permText;
+            CancellableProgressDialog dialog = CancellableProgressDialog.showCancellableProgressDialog(dialogsParent, title, buttonListener, false, monitor);
+            dialog.setPermText(permText);
 
-        long contentLength = -1;
-        String contentLengthString = conn.getHeaderField("Content-Length");
-        if (contentLengthString != null) {
-            contentLength = Long.parseLong(contentLengthString);
+            Dimension dms = new Dimension(600, 150);
+            dialog.setPreferredSize(dms);
+            dialog.setSize(dms);
+            dialog.validate();
+
+            LongRunningTask.submit(urlDownloader);
+            return urlDownloader;
         }
-
-
-        log.info("Content length = " + contentLength);
-
-        InputStream is = null;
-        OutputStream out = null;
-
-        try {
-            is = conn.getInputStream();
-            out = new FileOutputStream(outputFile);
-
-            byte[] buf = new byte[64 * 1024];
-            int downloaded = 0;
-            int bytesRead = 0;
-            while ((bytesRead = is.read(buf)) != -1) {
-                out.write(buf, 0, bytesRead);
-                downloaded += bytesRead;
-            }
-            log.info("Download complete.  Total bytes downloaded = " + downloaded);
-        } finally {
-            if (is != null) is.close();
-            if (out != null) {
-                out.flush();
-                out.close();
-            }
-        }
-        long fileLength = outputFile.length();
-
-        return contentLength <= 0 || contentLength == fileLength;
     }
 
 
@@ -464,7 +541,7 @@ public class HttpUtils {
         TrustManager[] trustAllCerts = new TrustManager[]{
                 new X509TrustManager() {
                     public java.security.cert.X509Certificate[] getAcceptedIssuers() {
-                        return new java.security.cert.X509Certificate[0];
+                        return null;
                     }
 
                     public void checkClientTrusted(
@@ -486,6 +563,7 @@ public class HttpUtils {
         } catch (KeyManagementException e) {
         }
 
+
     }
 
     private String readContents(InputStream is) throws IOException {
@@ -498,7 +576,7 @@ public class HttpUtils {
         return new String(bos.toByteArray());
     }
 
-    private String readErrorStream(HttpURLConnection connection) throws IOException {
+    public String readErrorStream(HttpURLConnection connection) throws IOException {
         InputStream inputStream = null;
 
         try {
@@ -518,7 +596,7 @@ public class HttpUtils {
         return openConnection(url, Collections.<String, String>emptyMap(), "DELETE");
     }
 
-    private HttpURLConnection openConnection(URL url, Map<String, String> requestProperties) throws IOException {
+    public HttpURLConnection openConnection(URL url, Map<String, String> requestProperties) throws IOException {
         return openConnection(url, requestProperties, "GET");
     }
 
@@ -538,6 +616,9 @@ public class HttpUtils {
     private HttpURLConnection openConnection(
             URL url, Map<String, String> requestProperties, String method, int redirectCount) throws IOException {
 
+        // Map amazon cname aliases to the full hosts -- neccessary to avoid ssl certificate errors in Java 1.8
+        url = mapCname(url);
+
         //Encode query string portions
         url = StringUtils.encodeURLQueryString(url);
         if (log.isTraceEnabled()) {
@@ -551,15 +632,24 @@ public class HttpUtils {
             url = new URL(newPath);
         }
 
-        Proxy sysProxy = getSystemProxy(url.toExternalForm());
-        boolean igvProxySettingsExist = proxySettings != null && proxySettings.useProxy && proxySettings.proxyHost != null && proxySettings.proxyPort > 0;
-        boolean useProxy = sysProxy != null || igvProxySettingsExist;
+        Proxy sysProxy = null;
+        boolean igvProxySettingsExist = proxySettings != null && proxySettings.useProxy;
+        //Only check for system proxy if igv proxy settings not found
+        if (!igvProxySettingsExist) {
+            sysProxy = getSystemProxy(url.toExternalForm());
+        }
+        boolean useProxy = sysProxy != null ||
+                (igvProxySettingsExist && !proxySettings.getWhitelist().contains(url.getHost()));
 
         HttpURLConnection conn;
         if (useProxy) {
             Proxy proxy = sysProxy;
             if (igvProxySettingsExist) {
-                proxy = new Proxy(proxySettings.type, new InetSocketAddress(proxySettings.proxyHost, proxySettings.proxyPort));
+                if (proxySettings.type == Proxy.Type.DIRECT) {
+                    proxy = Proxy.NO_PROXY;
+                } else {
+                    proxy = new Proxy(proxySettings.type, new InetSocketAddress(proxySettings.proxyHost, proxySettings.proxyPort));
+                }
             }
             conn = (HttpURLConnection) url.openConnection(proxy);
 
@@ -576,17 +666,14 @@ public class HttpUtils {
         if (GSUtils.isGenomeSpace(url)) {
             conn.setRequestProperty("Accept", "application/json,text/plain");
         } else {
-            conn.setRequestProperty("Accept", "text/plain");
+            if (!"HEAD".equals(method))
+                conn.setRequestProperty("Accept", "text/plain");
         }
 
-        //------//
-        //There seems to be a bug with JWS caches
-        //So we avoid caching
-
+        //There seems to be a bug with JWS caches, so we avoid caching
         //This default is persistent, really should be available statically but isn't
         conn.setDefaultUseCaches(false);
         conn.setUseCaches(false);
-        //------//
 
 
         conn.setConnectTimeout(Globals.CONNECT_TIMEOUT);
@@ -600,12 +687,21 @@ public class HttpUtils {
         }
         conn.setRequestProperty("User-Agent", Globals.applicationString());
 
+        if (url.getHost().equals(OAuthUtils.GS_HOST)) {
+            String token = OAuthUtils.getInstance().getAccessToken();
+            if (token != null) conn.setRequestProperty("Authorization", "Bearer " + token);
+        }
+
         if (method.equals("PUT")) {
             return conn;
         } else {
 
             int code = conn.getResponseCode();
 
+            if (log.isDebugEnabled()) {
+                //logHeaders(conn);
+            }
+
             // Redirects.  These can occur even if followRedirects == true if there is a change in protocol,
             // for example http -> https.
             if (code >= 300 && code < 400) {
@@ -617,7 +713,7 @@ public class HttpUtils {
                 String newLocation = conn.getHeaderField("Location");
                 log.debug("Redirecting to " + newLocation);
 
-                return openConnection(new URL(newLocation), requestProperties, method, redirectCount++);
+                return openConnection(new URL(newLocation), requestProperties, method, ++redirectCount);
             }
 
             // TODO -- handle other response codes.
@@ -628,24 +724,54 @@ public class HttpUtils {
                     message = "File not found: " + url.toString();
                     throw new FileNotFoundException(message);
                 } else if (code == 401) {
-                    // Looks like this only happens when user hits "Cancel".
-                    // message = "Not authorized to view this file";
-                    // JOptionPane.showMessageDialog(null, message, "HTTP error", JOptionPane.ERROR_MESSAGE);
-                    redirectCount = MAX_REDIRECTS + 1;
-                    return null;
+                    message = "You must log in to access this file";
+                    throw new HttpResponseException(code, message, "");
+                } else if (code == 403) {
+                    message = "Access forbidden";
+                    throw new HttpResponseException(code, message, "");
                 } else {
                     message = conn.getResponseMessage();
+                    String details = readErrorStream(conn);
+                    throw new HttpResponseException(code, message, details);
                 }
-                String details = readErrorStream(conn);
-                log.error("URL: " + url.toExternalForm() + ". error stream: " + details);
-                log.error("Code: " + code + ". " + message);
-                HttpResponseException exc = new HttpResponseException(code);
-                throw exc;
             }
         }
         return conn;
     }
 
+    private URL mapCname(URL url) {
+
+        String host = url.getHost();
+        String urlString = url.toExternalForm();
+        try {
+            if (host.equals("igv.broadinstitute.org")) {
+                urlString = urlString.replace("igv.broadinstitute.org", "s3.amazonaws.com/igv.broadinstitute.org");
+            } else if (host.equals("igvdata.broadinstitute.org")) {
+                urlString = urlString.replace("igvdata.broadinstitute.org", "dn7ywbm9isq8j.cloudfront.net");
+            } else if (host.equals("www.broadinstitute.org")) {
+                urlString = urlString.replace("www.broadinstitute.org/igvdata", "data.broadinstitute.org/igvdata");
+            }
+
+            // data.broadinstitute.org requires https
+            urlString = urlString.replace("http://data.broadinstitute.org", "https://data.broadinstitute.org");
+
+            return new URL(urlString);
+        } catch (MalformedURLException e) {
+            log.error("Error modifying url", e);
+        }
+        return url;
+    }
+
+
+    //Used for testing sometimes, please do not delete
+    private void logHeaders(HttpURLConnection conn) {
+        Map<String, List<String>> headerFields = conn.getHeaderFields();
+        log.debug("Headers for " + conn.getURL());
+        for (Map.Entry<String, List<String>> header : headerFields.entrySet()) {
+            log.debug(header.getKey() + ": " + StringUtils.join(header.getValue(), ","));
+        }
+    }
+
     public void setDefaultPassword(String defaultPassword) {
         this.defaultPassword = defaultPassword.toCharArray();
     }
@@ -672,116 +798,55 @@ public class HttpUtils {
         if (BYTE_RANGE_DISABLED) return false;
 
         // We can test byte-range success for hosts we can reach.
+        synchronized (byteRangeTestMap) {
+            final String host = url.getHost();
 
-        final String host = url.getHost();
+            if (byteRangeTestMap.containsKey(host)) {
+                return byteRangeTestMap.get(host);
+            } else {
+                SeekableStream str = null;
+                try {
 
-        if (byteRangeTestMap.containsKey(host)) {
-            return byteRangeTestMap.get(host);
-        } else {
-            SeekableStream str = null;
-            try {
-                boolean byteRangeTestSuccess = true;
+                    boolean byteRangeTestSuccess = testByteRange(url);
 
-                if (host.contains("broadinstitute.org")) {
-                    byteRangeTestSuccess = testBroadHost(host);
-                } else {
-                    // Non-broad URL
-                    int l = (int) Math.min(1000, HttpUtils.getInstance().getContentLength(url));
-                    if (l > 1000) {
-
-                        byte[] firstBytes = new byte[l];
-                        str = new IGVSeekableHTTPStream(url);
-                        str.readFully(firstBytes);
-
-                        int end = firstBytes.length;
-                        int start = end - 100;
-                        str.seek(start);
-                        int len = end - start;
-                        byte[] buffer = new byte[len];
-                        int n = 0;
-                        while (n < len) {
-                            int count = str.read(buffer, n, len - n);
-                            if (count < 0)
-                                throw new EOFException();
-                            n += count;
-                        }
-
-                        for (int i = 0; i < len; i++) {
-                            if (buffer[i] != firstBytes[i + start]) {
-                                byteRangeTestSuccess = false;
-                                break;
-                            }
-                        }
+                    if (byteRangeTestSuccess) {
+                        log.info("Range-byte request succeeded");
                     } else {
-                        // Too small a sample to test, return "true" but don't record this host as tested.
-                        return true;
+                        log.info("Range-byte test failed -- Server does not support range-byte requests or problem with client network environment.");
+                    }
+
+                    byteRangeTestMap.put(host, byteRangeTestSuccess);
+                    return byteRangeTestSuccess;
+
+
+                } catch (IOException e) {
+                    log.error("Error while testing byte range " + e.getMessage());
+                    // We could not reach the test server, so we can't know if this client can do byte-range tests or
+                    // not.  Take the "optimistic" view.
+                    return true;
+                } finally {
+                    if (str != null) try {
+                        str.close();
+                    } catch (IOException e) {
+                        log.error("Error closing stream (" + url.toExternalForm() + ")", e);
                     }
                 }
-
-                if (byteRangeTestSuccess) {
-                    log.info("Range-byte request succeeded");
-                } else {
-                    log.info("Range-byte test failed -- problem with client network environment.");
-                }
-
-                byteRangeTestMap.put(host, byteRangeTestSuccess);
-                return byteRangeTestSuccess;
-
-            } catch (IOException e) {
-                log.error("Error while testing byte range " + e.getMessage());
-                // We could not reach the test server, so we can't know if this client can do byte-range tests or
-                // not.  Take the "optimistic" view.
-                return true;
-            } finally {
-                if (str != null) try {
-                    str.close();
-                } catch (IOException e) {
-                    log.error("Error closing stream (" + url.toExternalForm() + ")", e);
-                }
             }
 
         }
     }
 
-    private boolean testBroadHost(String host) throws IOException {
-        // Test broad urls for successful byte range requests.
-        log.info("Testing range-byte request on host: " + host);
-
-        String testURL;
-        if (host.startsWith("igvdata.broadinstitute.org")) {
-            // Amazon cloud front
-            testURL = "http://igvdata.broadinstitute.org/genomes/seq/hg19/chr12.txt";
-        } else if (host.startsWith("igv.broadinstitute.org")) {
-            // Amazon S3
-            testURL = "http://igv.broadinstitute.org/genomes/seq/hg19/chr12.txt";
-        } else {
-            // All others
-            testURL = "http://www.broadinstitute.org/igvdata/annotations/seq/hg19/chr12.txt";
-        }
-
-        byte[] expectedBytes = {'T', 'C', 'G', 'C', 'T', 'T', 'G', 'A', 'A', 'C', 'C', 'C', 'G', 'G',
-                'G', 'A', 'G', 'A', 'G', 'G'};
-        IGVSeekableHTTPStream str = null;
-
-        try {
-            str = new IGVSeekableHTTPStream(new URL(testURL));
-
-            str.seek(25350000);
-            byte[] buffer = new byte[80000];
-            str.read(buffer);
-            String result = new String(buffer);
-            for (int i = 0; i < expectedBytes.length; i++) {
-                if (buffer[i] != expectedBytes[i]) {
-                    return false;
-                }
-            }
-            return true;
-        } finally {
-            if (str != null) str.close();
-        }
+    public boolean testByteRange(URL url) throws IOException {
+        Map<String, String> params = new HashMap();
+        String byteRange = "bytes=" + 0 + "-" + 10;
+        params.put("Range", byteRange);
+        HttpURLConnection conn = HttpUtils.getInstance().openConnection(url, params);
+        int statusCode = conn.getResponseCode();
+        boolean byteRangeTestSuccess = (statusCode == 206);
+        readFully(conn.getInputStream(), new byte[10]);
+        return byteRangeTestSuccess;
     }
 
-
     public void shutdown() {
         // Do any cleanup required here
     }
@@ -800,9 +865,10 @@ public class HttpUtils {
         String[] params = query.split("&");
         Map<String, String> map = new HashMap<String, String>();
         for (String param : params) {
-            String name = param.split("=")[0];
-            String value = param.split("=")[1];
-            map.put(name, value);
+            String[] name_val = param.split("=", 2);
+            if (name_val.length == 2) {
+                map.put(name_val[0], name_val[1]);
+            }
         }
         return map;
     }
@@ -816,13 +882,11 @@ public class HttpUtils {
         String proxyHost;
         int proxyPort = -1;
         Proxy.Type type;
+        Set<String> whitelist;
 
-        public ProxySettings(boolean useProxy, String user, String pw, boolean auth, String proxyHost, int proxyPort) {
-            this(useProxy, user, pw, auth, proxyHost, proxyPort, Proxy.Type.HTTP);
-            this.auth = auth;
-        }
 
-        public ProxySettings(boolean useProxy, String user, String pw, boolean auth, String proxyHost, int proxyPort, Proxy.Type proxyType) {
+        public ProxySettings(boolean useProxy, String user, String pw, boolean auth, String proxyHost, int proxyPort,
+                             Proxy.Type proxyType, Set<String> whitelist) {
             this.auth = auth;
             this.proxyHost = proxyHost;
             this.proxyPort = proxyPort;
@@ -830,6 +894,11 @@ public class HttpUtils {
             this.useProxy = useProxy;
             this.user = user;
             this.type = proxyType;
+            this.whitelist = whitelist;
+        }
+
+        public Set<String> getWhitelist() {
+            return whitelist;
         }
     }
 
@@ -907,6 +976,16 @@ public class HttpUtils {
     }
 
 
+    static boolean isExpectedRangeMissing(URLConnection conn, Map<String, String> requestProperties) {
+        final boolean rangeRequested = (requestProperties != null) && (new CI.CIHashMap<String>(requestProperties)).containsKey("Range");
+        if (!rangeRequested) return false;
+
+        Map<String, List<String>> headerFields = conn.getHeaderFields();
+        boolean rangeReceived = (headerFields != null) && (new CI.CIHashMap<List<String>>(headerFields)).containsKey("Content-Range");
+        return !rangeReceived;
+    }
+
+
     /**
      * Provide override for unit tests
      */
@@ -923,6 +1002,25 @@ public class HttpUtils {
     }
 
 
+    /**
+     * Useful helper function
+     */
+    public static void readFully(InputStream is, byte b[]) throws IOException {
+        int len = b.length;
+        if (len < 0) {
+            throw new IndexOutOfBoundsException();
+        }
+        int n = 0;
+        while (n < len) {
+            int count = is.read(b, n, len - n);
+            if (count < 0) {
+                throw new EOFException();
+            }
+            n += count;
+        }
+    }
+
+
     /**
      * Extension of CookieManager that grabs cookies from the GenomeSpace identity server to store locally.
      * This is to support the GenomeSpace "single sign-on". Examples ...
@@ -930,19 +1028,27 @@ public class HttpUtils {
      * gs-token=HnR9rBShNO4dTXk8cKXVJT98Oe0jWVY+; Domain=.genomespace.org; Expires=Mon, 21-Jul-2031 03:27:23 GMT; Path=/
      */
 
-    static class IGVCookieManager extends CookieManager {
+    static class IGVCookieManager extends CookieHandler {
+
+
+        CookieManager wrappedManager;
+
+        public IGVCookieManager() {
+            wrappedManager = new CookieManager();
+        }
 
         @Override
         public Map<String, List<String>> get(URI uri, Map<String, List<String>> requestHeaders) throws IOException {
-            Map<String, List<String>> headers = new HashMap<String, List<String>>();
-            headers.putAll(super.get(uri, requestHeaders));
 
+            Map<String, List<String>> headers = new HashMap<String, List<String>>();
+            headers.putAll(wrappedManager.get(uri, requestHeaders));
 
             if (GSUtils.isGenomeSpace(uri.toURL())) {
                 String token = GSUtils.getGSToken();
                 if (token != null) {
                     List<String> cookieList = headers.get("Cookie");
                     boolean needsTokenCookie = true;
+                    boolean needsToolCookie = true;
                     if (cookieList == null) {
                         cookieList = new ArrayList<String>(1);
                         headers.put("Cookie", cookieList);
@@ -951,12 +1057,16 @@ public class HttpUtils {
                     for (String cookie : cookieList) {
                         if (cookie.startsWith("gs-token")) {
                             needsTokenCookie = false;
-                            break;
+                        } else if (cookie.startsWith("gs-toolname")) {
+                            needsToolCookie = false;
                         }
                     }
                     if (needsTokenCookie) {
                         cookieList.add("gs-token=" + token);
                     }
+                    if (needsToolCookie) {
+                        cookieList.add("gs-toolname=IGV");
+                    }
                 }
             }
 
@@ -985,8 +1095,9 @@ public class HttpUtils {
                     }
                 }
             }
-            super.put(uri, responseHeaders);
+            wrappedManager.put(uri, responseHeaders);
         }
     }
 
+
 }
