@@ -55,8 +55,8 @@ public class StringUtils {
 
 
     public static List<String> breakQuotedString(String string, char splitToken) {
-
-        ArrayList<String> strings = new ArrayList();
+        if(string == null) return null;
+        ArrayList<String> strings = new ArrayList<String >();
         if (string.length() == 0) {
             return strings;
         }
@@ -201,10 +201,13 @@ public class StringUtils {
             int ii = 0;
             for (String kv : params) {
                 String[] kvs = kv.split("\\=", 2);
-                String encString = String.format("%s=%s", StringUtils.encodeURL(kvs[0]), StringUtils.encodeURL(kvs[1]));
+                String encString = StringUtils.encodeURL(kvs[0]);
+                if(kvs.length == 2){
+                    encString = String.format("%s=%s", encString, StringUtils.encodeURL(kvs[1]));
+                }
                 encParms[ii++] = encString;
             }
-            String encQuery = org.apache.commons.lang.StringUtils.join(encParms, "&");
+            String encQuery = StringUtils.join(encParms, "&");
             String newPath = parts[0] + "?" + encQuery;
             url = new URL(newPath);
         }
@@ -248,6 +251,27 @@ public class StringUtils {
         return res;
     }
 
+    public static String join(List list, String separator) {
+        return join(list.toArray(), separator);
+    }
+
+    /**
+     * Join the objects in {@code array} with the given {@code separator}
+     * Useful for making a comma separated list, or URL query string
+     * @param array
+     * @param separator
+     * @return
+     */
+    public static String join(Object[] array, String separator) {
+        int num = array.length;
+        if(num == 0) return "";
+        String out = array[0].toString();
+        for(int ii=1; ii < num; ii++) {
+            out += (separator + array[ii].toString());
+        }
+        return out;
+    }
+
 
     /**
      * This must exist in the jdk ?
@@ -265,4 +289,19 @@ public class StringUtils {
         return cnt;
 
     }
+
+    /**
+     * Strips quotation marks from string.  Quotes can be single or double, but must match each other.
+     *
+     * @param fileString
+     * @return
+     */
+    public static String stripQuotes(String fileString) {
+        if ((fileString.startsWith("\"") && fileString.endsWith("\"")) ||
+                (fileString.startsWith("'") && fileString.endsWith("'"))) {
+            fileString = fileString.substring(1, fileString.length() - 1);
+        }
+        return fileString;
+    }
+
 }
